@@ -2,24 +2,22 @@ import ast
 
 from .ast_helpers import get_id
 
-
 get_id  # quiten pyflakes; this should when code is updated to use ast_helpers
 
-IGNORED_MODULE_SET = set(
-    [
-        "typing",
-        "enum",
-        "dataclasses",
-        "ctypes",
-        "math",
-        "__future__",
-        "asyncio",
-        "sys",
-        "os",
-        "adt",
-        "py2many.result",
-    ]
-)
+IGNORED_MODULE_SET = {
+    "typing",
+    "enum",
+    "dataclasses",
+    "ctypes",
+    "math",
+    "__future__",
+    "asyncio",
+    "sys",
+    "os",
+    "adt",
+    "py2many.result",
+    "py2many.smt",
+}
 
 
 def add_imports(node):
@@ -45,6 +43,14 @@ def is_mutable(scopes, target):
     return False
 
 
+def is_ellipsis(node):
+    return (
+        isinstance(node, ast.Expr)
+        and isinstance(node.value, ast.Constant)
+        and node.value.value == ...
+    )
+
+
 class ReturnFinder(ast.NodeVisitor):
     returns = False
 
