@@ -1,19 +1,7 @@
 import ast
-from py2many.astx import ASTxIf
 from typing import cast
 
-try:
-    from ast import unparse  # noqa: introduced in py3.9
-except ImportError:
-    from astor import to_source as unparse
-
-    # Note ast-compat and astunparse packages fail internally with:
-    # AttributeError: 'Constant' object has no attribute 'kind'
-
-    # https://github.com/isidentical/backports.ast_unparse/commit/e7b1aea is broken
-
-
-unparse  # ignore pyflakes
+from py2many.astx import ASTxIf
 
 
 def get_id(var):
