@@ -1,84 +1,58 @@
 import argparse
 import ast
-import functools
 import os
 import sys
-
-
-from distutils import spawn
+import tempfile
 from functools import lru_cache
 from pathlib import Path
 from subprocess import run
-from typing import List, Set, Tuple
-from unittest.mock import Mock
-
+from typing import List, Optional, Set, Tuple
 
 from .analysis import add_imports
 from .annotation_transformer import add_annotation_flags
-
-from .context import add_assignment_context, add_variable_context, add_list_calls
+from .context import add_assignment_context, add_list_calls, add_variable_context
 from .exceptions import AstErrorBase
-from .inference import infer_types
+from .inference import infer_types, infer_types_typpete
 from .language import LanguageSettings
 from .mutability_transformer import detect_mutable_vars
 from .nesting_transformer import detect_nesting_levels
-from .python_transformer import PythonTranspiler, RestoreMainRewriter
+from .raises_transformer import detect_raises
+from .registry import ALL_SETTINGS, FAKE_ARGS, _get_all_settings
+from .rewriters import (
+    ComplexDestructuringRewriter,
+    DocStringToCommentRewriter,
+    FStringJoinRewriter,
+    IgnoredAssignRewriter,
+    LoopElseRewriter,
+    PrintBoolRewriter,
+    PythonMainRewriter,
+    StrStrRewriter,
+    UnpackScopeRewriter,
+    WithToBlockTransformer,
+)
 from .scope import add_scope_context
 from .toposort_modules import toposort
-
-from pycpp.transpiler import CppTranspiler, CppListComparisonRewriter
-from pyrs.inference import infer_rust_types
-from pyrs.transpiler import (
-    RustTranspiler,
-    RustLoopIndexRewriter,
-    RustNoneCompareRewriter,
-    RustStringJoinRewriter,
-)
-from pyjl.transpiler import JuliaTranspiler, JuliaMethodCallRewriter
-from pykt.inference import infer_kotlin_types
-from pykt.transpiler import KotlinTranspiler, KotlinPrintRewriter, KotlinBitOpRewriter
-from pynim.inference import infer_nim_types
-from pynim.transpiler import NimTranspiler, NimNoneCompareRewriter
-from pydart.transpiler import DartTranspiler, DartIntegerDivRewriter
-from pygo.inference import infer_go_types
-from pygo.transpiler import (
-    GoTranspiler,
-    GoMethodCallRewriter,
-    GoNoneCompareRewriter,
-    GoPropagateTypeAnnotation,
-    GoVisibilityRewriter,
-    GoIfExpRewriter,
-)
-from pyv.inference import infer_v_types
-from pyv.transpiler import VTranspiler, VNoneCompareRewriter, VDictRewriter
-
-from py2many.rewriters import (
-    ComplexDestructuringRewriter,
-    FStringJoinRewriter,
-    InferredAnnAssignRewriter,
-    PythonMainRewriter,
-    DocStringToCommentRewriter,
-    PrintBoolRewriter,
-    StrStrRewriter,
-    WithToBlockTransformer,
-    IgnoredAssignRewriter,
-    UnpackScopeRewriter,
-)
+from .version import __version__
 
 PY2MANY_DIR = Path(__file__).parent
 ROOT_DIR = PY2MANY_DIR.parent
+STDIN = "-"
+STDOUT = "-"
 CWD = Path.cwd()
 
 
-def core_transformers(tree, trees):
+def core_transformers(tree, trees, args):
     add_variable_context(tree, trees)
     add_scope_context(tree)
     add_assignment_context(tree)
     add_list_calls(tree)
     detect_mutable_vars(tree)
     detect_nesting_levels(tree)
+    detect_raises(tree)
     add_annotation_flags(tree)
-    infer_meta = infer_types(tree)
+    infer_meta = (
+        infer_types_typpete(tree) if args and args.typpete else infer_types(tree)
+    )
     add_imports(tree)
     return tree, infer_meta
 
@@ -87,12 +61,17 @@ def _transpile(
     filenames: List[Path],
     sources: List[str],
     settings: LanguageSettings,
+    args: Optional[argparse.Namespace] = None,
     _suppress_exceptions=Exception,
 ):
     """
     Transpile a single python translation unit (a python script) into
     target language
     """
+    if hasattr(args, "llm") and args.llm:
+        from .llm_transpile import llm_transpile
+
+        return llm_transpile(filenames, sources, settings, args)
     transpiler = settings.transpiler
     rewriters = settings.rewriters
     transformers = settings.transformers
@@ -119,6 +98,9 @@ def _transpile(
         StrStrRewriter(language),
         UnpackScopeRewriter(language),
     ]
+    if settings.ext != ".py":
+        generic_post_rewriters.append(LoopElseRewriter(language))
+
     rewriters = generic_rewriters + rewriters
     post_rewriters = generic_post_rewriters + post_rewriters
     outputs = {}
@@ -126,7 +108,7 @@ def _transpile(
     for filename, tree in zip(topo_filenames, trees):
         try:
             output = _transpile_one(
-                trees, tree, transpiler, rewriters, transformers, post_rewriters
+                trees, tree, transpiler, rewriters, transformers, post_rewriters, args
             )
             successful.append(filename)
             outputs[filename] = output
@@ -146,7 +128,9 @@ def _transpile(
     return output_list, successful
 
 
-def _transpile_one(trees, tree, transpiler, rewriters, transformers, post_rewriters):
+def _transpile_one(
+    trees, tree, transpiler, rewriters, transformers, post_rewriters, args
+):
     # This is very basic and needs to be run before and after
     # rewrites. Revisit if running it twice becomes a perf issue
     add_scope_context(tree)
@@ -154,7 +138,7 @@ def _transpile_one(trees, tree, transpiler, rewriters, transformers, post_rewrit
     for rewriter in rewriters:
         tree = rewriter.visit(tree)
     # Language independent core transformers
-    tree, infer_meta = core_transformers(tree, trees)
+    tree, infer_meta = core_transformers(tree, trees, args)
     # Language specific transformers
     for tx in transformers:
         tx(tree)
@@ -162,7 +146,7 @@ def _transpile_one(trees, tree, transpiler, rewriters, transformers, post_rewrit
     for rewriter in post_rewriters:
         tree = rewriter.visit(tree)
     # Rerun core transformers
-    tree, infer_meta = core_transformers(tree, trees)
+    tree, infer_meta = core_transformers(tree, trees, args)
     out = []
     code = transpiler.visit(tree) + "\n"
     headers = transpiler.headers(infer_meta)
@@ -174,6 +158,10 @@ def _transpile_one(trees, tree, transpiler, rewriters, transformers, post_rewrit
     usings = transpiler.usings()
     if usings:
         out.append(usings)
+    # languages like zig have aliases that come after usings
+    aliases = transpiler.aliases()
+    if aliases:
+        out.append(aliases)
     out.append(code)
     if transpiler.extension:
         out.append(transpiler.extension_module(tree))
@@ -192,187 +180,13 @@ def _create_cmd(parts, filename, **kw):
     return [*parts, str(filename)]
 
 
-def python_settings(args, env=os.environ):
-    return LanguageSettings(
-        PythonTranspiler(),
-        ".py",
-        "Python",
-        formatter=["black"],
-        rewriters=[RestoreMainRewriter()],
-        post_rewriters=[InferredAnnAssignRewriter()],
-    )
-
-
-def cpp_settings(args, env=os.environ):
-    clang_format_style = env.get("CLANG_FORMAT_STYLE")
-    cxx = env.get("CXX")
-    default_cxx = ["clang++", "g++-11", "g++"]
-    if cxx:
-        if not spawn.find_executable(cxx):
-            print(f"Warning: CXX({cxx}) not found")
-            cxx = None
-    if not cxx:
-        for exe in default_cxx:
-            if spawn.find_executable(exe):
-                cxx = exe
-                break
-        else:
-            cxx = default_cxx[0]
-    cxx_flags = env.get("CXXFLAGS")
-    if cxx_flags:
-        cxx_flags = cxx_flags.split()
-    else:
-        cxx_flags = ["-std=c++14", "-Wall", "-Werror"]
-    cxx_flags = ["-I", str(ROOT_DIR)] + cxx_flags
-    if cxx.startswith("clang++") and not sys.platform == "win32":
-        cxx_flags += ["-stdlib=libc++"]
-
-    if clang_format_style:
-        clang_format_cmd = ["clang-format", f"-style={clang_format_style}", "-i"]
-    else:
-        clang_format_cmd = ["clang-format", "-i"]
-
-    return LanguageSettings(
-        CppTranspiler(args.extension, args.no_prologue),
-        ".cpp",
-        "C++",
-        clang_format_cmd,
-        None,
-        [CppListComparisonRewriter()],
-        linter=[cxx, *cxx_flags],
-    )
-
-
-def rust_settings(args, env=os.environ):
-    return LanguageSettings(
-        RustTranspiler(args.extension, args.no_prologue),
-        ".rs",
-        "Rust",
-        ["rustfmt", "--edition=2018"],
-        None,
-        [RustNoneCompareRewriter()],
-        [functools.partial(infer_rust_types, extension=args.extension)],
-        [RustLoopIndexRewriter(), RustStringJoinRewriter()],
-    )
-
-
-def julia_settings(args, env=os.environ):
-    format_jl = spawn.find_executable("format.jl")
-    if format_jl:
-        format_jl = ["julia", "-O0", "--compile=min", "--startup=no", format_jl, "-v"]
-    else:
-        format_jl = ["format.jl", "-v"]
-    return LanguageSettings(
-        JuliaTranspiler(),
-        ".jl",
-        "Julia",
-        format_jl,
-        None,
-        [],
-        [],
-        [JuliaMethodCallRewriter()],
-    )
-
-
-def kotlin_settings(args, env=os.environ):
-    return LanguageSettings(
-        KotlinTranspiler(),
-        ".kt",
-        "Kotlin",
-        ["ktlint", "-F"],
-        rewriters=[KotlinBitOpRewriter()],
-        transformers=[infer_kotlin_types],
-        post_rewriters=[KotlinPrintRewriter()],
-        linter=["ktlint"],
-    )
-
-
-def nim_settings(args, env=os.environ):
-    nim_args = {}
-    nimpretty_args = []
-    if args.indent is not None:
-        nim_args["indent"] = args.indent
-        nimpretty_args.append(f"--indent:{args.indent}")
-    return LanguageSettings(
-        NimTranspiler(**nim_args),
-        ".nim",
-        "Nim",
-        ["nimpretty", *nimpretty_args],
-        None,
-        [NimNoneCompareRewriter()],
-        [infer_nim_types],
-    )
-
-
-def dart_settings(args, env=os.environ):
-    return LanguageSettings(
-        DartTranspiler(),
-        ".dart",
-        "Dart",
-        ["dart", "format"],
-        post_rewriters=[DartIntegerDivRewriter()],
-    )
-
-
-def go_settings(args, env=os.environ):
-    config_filename = "revive.toml"
-    if os.path.exists(CWD / config_filename):
-        revive_config = CWD / config_filename
-    elif os.path.exists(PY2MANY_DIR / config_filename):
-        revive_config = PY2MANY_DIR / config_filename
-    else:
-        revive_config = None
-    return LanguageSettings(
-        GoTranspiler(),
-        ".go",
-        "Go",
-        ["gofmt", "-w"],
-        None,
-        [GoNoneCompareRewriter(), GoVisibilityRewriter(), GoIfExpRewriter()],
-        [infer_go_types],
-        [GoMethodCallRewriter(), GoPropagateTypeAnnotation()],
-        linter=(
-            ["revive", "--config", str(revive_config)] if revive_config else ["revive"]
-        ),
-    )
-
-
-def vlang_settings(args, env=os.environ):
-    v_args = {}
-    vfmt_args = ["fmt", "-w"]
-    if args.indent is not None:
-        v_args["indent"] = args.indent
-        vfmt_args.append(f"--indent:{args.indent}")
-    return LanguageSettings(
-        VTranspiler(**v_args),
-        ".v",
-        "V",
-        ["v", *vfmt_args],
-        None,
-        [VNoneCompareRewriter(), VDictRewriter()],
-        [infer_v_types],
-    )
-
-
-def _get_all_settings(args, env=os.environ):
-    return {
-        "python": python_settings(args, env=env),
-        "cpp": cpp_settings(args, env=env),
-        "rust": rust_settings(args, env=env),
-        "julia": julia_settings(args, env=env),
-        "kotlin": kotlin_settings(args, env=env),
-        "nim": nim_settings(args, env=env),
-        "dart": dart_settings(args, env=env),
-        "go": go_settings(args, env=env),
-        "vlang": vlang_settings(args, env=env),
-    }
-
-
 def _relative_to_cwd(absolute_path):
     return Path(os.path.relpath(absolute_path, CWD))
 
 
 def _get_output_path(filename, ext, outdir):
+    if filename.name == STDIN:
+        return Path(STDOUT)
     directory = outdir / filename.parent
     if not directory.is_dir():
         directory.mkdir(parents=True)
@@ -392,6 +206,24 @@ def _process_one(settings: LanguageSettings, filename: Path, outdir: str, args,
     output_path = _get_output_path(
         filename.relative_to(filename.parent), suffix, outdir
     )
+
+    if filename.name == STDIN:
+        # special case for simple pipes
+        output = _process_one_data(sys.stdin.read(), Path("test.py"), settings)
+        tmp_name = None
+        try:
+            with tempfile.NamedTemporaryFile(suffix=settings.ext, delete=False) as f:
+                tmp_name = f.name
+                f.write(output.encode("utf-8"))
+            if _format_one(settings, tmp_name, env):
+                sys.stdout.write(open(tmp_name).read())
+            else:
+                sys.stderr.write("Formatting failed")
+        finally:
+            if tmp_name is not None:
+                os.remove(tmp_name)
+        return ({filename}, {filename})
+
     if filename.resolve() == output_path.resolve() and not args.force:
         print(f"Refusing to overwrite {filename}. Use --force to overwrite")
         return False
@@ -403,9 +235,9 @@ def _process_one(settings: LanguageSettings, filename: Path, outdir: str, args,
     if dunder_init and not source_data:
         print("Detected empty __init__; skipping")
         return True
-    result = _transpile([filename], [source_data], settings)
-    with open(output_path, "w") as f:
-        f.write(result[0][0])
+    result = _transpile([filename], [source_data], settings, args)
+    with open(output_path, "wb") as f:
+        f.write(result[0][0].encode("utf-8"))
 
     if settings.formatter:
         return _format_one(settings, output_path, env)
@@ -451,6 +283,8 @@ def _format_one(settings, output_path, env=None):
         if restore_cwd:
             os.chdir(restore_cwd)
     except Exception as e:
+        if settings.ignore_formatter_errors:
+            return True
         print(f"Error: Could not format: {output_path}")
         print(f"Due to: {e.__class__.__name__} {e}")
         return False
@@ -475,10 +309,7 @@ def _process_many(
             source_data.append(f.read())
 
     outputs, successful = _transpile(
-        filenames,
-        source_data,
-        settings,
-        _suppress_exceptions=_suppress_exceptions,
+        filenames, source_data, settings, _suppress_exceptions=_suppress_exceptions
     )
 
     output_paths = [
@@ -499,8 +330,21 @@ def _process_many(
     return (successful, format_errors)
 
 
-def _process_dir(settings, source, outdir, env=None, _suppress_exceptions=Exception):
+def _process_dir(
+    settings, source, outdir, project, env=None, _suppress_exceptions=Exception
+):
     print(f"Transpiling whole directory to {outdir}:")
+
+    if settings.create_project is not None and project:
+        cmd = settings.create_project + [f"{outdir}"]
+        proc = run(cmd, env=env, capture_output=True)
+        if proc.returncode:
+            cmd_str = " ".join(cmd)
+            print(f"Error: running {cmd_str}: {proc.stderr}")
+            return (set(), set(), set())
+        if settings.project_subdir is not None:
+            outdir = outdir / settings.project_subdir
+
     successful = []
     failures = []
     input_paths = []
@@ -537,7 +381,7 @@ def _process_dir(settings, source, outdir, env=None, _suppress_exceptions=Except
 
 def main(args=None, env=os.environ):
     parser = argparse.ArgumentParser()
-    LANGS = _get_all_settings(Mock(indent=4))
+    LANGS = _get_all_settings(FAKE_ARGS)
     for lang, settings in LANGS.items():
         parser.add_argument(
             f"--{lang}",
@@ -559,6 +403,20 @@ def main(args=None, env=os.environ):
         action="store_true",
         help="Place unsupported constructs in comments",
     )
+    parser.add_argument(
+        "--no-strict",
+        dest="strict",
+        default=True,
+        action="store_false",
+        help="Skip over unsupported constructs and generate some code",
+    )
+    parser.add_argument(
+        "--ignore-formatter-errors",
+        dest="ignore_formatter_errors",
+        default=False,
+        action="store_true",
+        help="Ignore formatter error if its not installed",
+    )
     parser.add_argument(
         "--extension",
         action="store_true",
@@ -577,48 +435,71 @@ def main(args=None, env=os.environ):
         default=False,
         help="When output and input are the same file, force overwriting",
     )
+    parser.add_argument(
+        "--typpete",
+        action="store_true",
+        default=False,
+        help="Use typpete for inference",
+    )
+    parser.add_argument(
+        "--version",
+        action="store_true",
+        default=False,
+        help="Display version number",
+    )
+    parser.add_argument(
+        "--project", default=True, help="Create a project when using directory mode"
+    )
+    parser.add_argument(
+        "--llm",
+        action="store_true",
+        default=False,
+        help="Use llm for transpiling",
+    )
+    parser.add_argument(
+        "--llm-model",
+        type=str,
+        default="",
+        help="LLM Model to use. Configurable other ways too",
+    )
     args, rest = parser.parse_known_args(args=args)
 
+    if args.version:
+        print(__version__)
+        return 0
+
     # Validation of the args
     if args.extension and not args.rust:
         print("extension supported only with rust via pyo3")
         return -1
 
+    settings_func = ALL_SETTINGS["cpp"]
+    for lang, func in ALL_SETTINGS.items():
+        arg = getattr(args, lang)
+        if arg:
+            settings_func = func
+            break
+    settings = settings_func(args, env=env)
+
     if args.comment_unsupported:
         print("Wrapping unimplemented in comments")
+        settings.transpiler.set_continue_on_unimplemented()
+
+    if not args.strict:
+        print("Warning: some code may be experimental and incorrect")
+        settings.transpiler.set_continue_on_unimplemented()
+
+    settings.ignore_formatter_errors = args.ignore_formatter_errors
 
     for filename in rest:
-        settings = cpp_settings(args, env=env)
-        if args.cpp:
-            pass
-        if args.rust:
-            settings = rust_settings(args, env=env)
-        elif args.python:
-            settings = python_settings(args, env=env)
-        elif args.julia:
-            settings = julia_settings(args, env=env)
-        elif args.kotlin:
-            settings = kotlin_settings(args, env=env)
-        elif args.nim:
-            settings = nim_settings(args, env=env)
-        elif args.dart:
-            settings = dart_settings(args, env=env)
-        elif args.go:
-            settings = go_settings(args, env=env)
-        elif args.vlang:
-            settings = vlang_settings(args, env=env)
-
-        if args.comment_unsupported:
-            settings.transpiler._throw_on_unimplemented = False
-
         source = Path(filename)
         if args.outdir is None:
             outdir = source.parent
         else:
             outdir = Path(args.outdir)
 
-        if source.is_file():
-            print(f"Writing to: {outdir}")
+        if source.is_file() or source.name == STDIN:
+            print(f"Writing to: {outdir}", file=sys.stderr)
             try:
                 rv = _process_one(settings, source, outdir, args, env)
             except Exception as e:
@@ -626,16 +507,19 @@ def main(args=None, env=os.environ):
 
                 formatted_lines = traceback.format_exc().splitlines()
                 if isinstance(e, AstErrorBase):
-                    print(f"{source}:{e.lineno}:{e.col_offset}: {formatted_lines[-1]}")
+                    print(
+                        f"{source}:{e.lineno}:{e.col_offset}: {formatted_lines[-1]}",
+                        file=sys.stderr,
+                    )
                 else:
-                    print(f"{source}: {formatted_lines[-1]}")
+                    print(f"{source}: {formatted_lines[-1]}", file=sys.stderr)
                 rv = False
         else:
             if args.outdir is None:
                 outdir = source.parent / f"{source.name}-py2many"
 
             successful, format_errors, failures = _process_dir(
-                settings, source, outdir, env=env
+                settings, source, outdir, args.project, env=env
             )
             rv = not (failures or format_errors)
         rv = 0 if rv is True else 1
