@@ -1,27 +1,45 @@
 import ast
 import importlib
+import io  # noqa: F401
 import logging
-import math
-import os
-import random
+import math  # noqa: F401
+import os  # noqa: F401
+import random  # noqa: F401
 import sys
-import time
-
-from pathlib import Path
-from typing import Any, Dict, OrderedDict
+import time  # noqa: F401
 
 # Fixed width ints and aliases
-from ctypes import (
-    c_int8 as i8,
-    c_int16 as i16,
-    c_int32 as i32,
-    c_int64 as i64,
-    c_uint8 as u8,
-    c_uint16 as u16,
-    c_uint32 as u32,
-    c_uint64 as u64,
+from ctypes import c_int8 as i8
+from ctypes import c_int16 as i16
+from ctypes import c_int32 as i32
+from ctypes import c_int64 as i64
+from ctypes import c_uint8 as u8
+from ctypes import c_uint16 as u16
+from ctypes import c_uint32 as u32
+from ctypes import c_uint64 as u64
+from pathlib import Path
+from typing import (  # noqa: F401
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    OrderedDict,
+    Tuple,
+    Union,
 )
 
+from py2many.analysis import IGNORED_MODULE_SET, get_id
+from py2many.astx import LifeTime
+from py2many.exceptions import (
+    AstCouldNotInfer,
+    AstEmptyNodeFound,
+    AstNotImplementedError,
+    AstTypeNotSupported,
+    TypeNotSupported,
+)
+from py2many.result import Result  # noqa: F401
+
 ilong = i64
 ulong = u64
 isize = i64
@@ -35,25 +53,6 @@ c_uint16 = u16
 c_uint32 = u32
 c_uint64 = u64
 
-
-from py2many.analysis import get_id, IGNORED_MODULE_SET
-from py2many.astx import LifeTime
-from py2many.exceptions import (
-    AstCouldNotInfer,
-    AstEmptyNodeFound,
-    AstNotImplementedError,
-    AstTypeNotSupported,
-    TypeNotSupported,
-)
-from py2many.result import Result
-from typing import List, Optional, Tuple, Union
-
-os.path  # silence pyflakes
-math.pi  # silence pyflakes
-time.time  # silence pyflakes
-random.random  # silence pyflakes
-Result  # silence pyflakes
-
 symbols = {
     ast.Eq: "==",
     ast.Is: "==",
@@ -91,13 +90,23 @@ logger = logging.Logger("py2many")
 def class_for_typename(typename, default_type, locals=None) -> Union[str, object]:
     if typename is None:
         return None
-    if typename == "super" or typename.startswith("super()"):
+    if isinstance(typename, str) and (
+        typename == "super" or typename.startswith("super()")
+    ):
         # Cant eval super; causes RuntimeError
         return None
     try:
         # TODO: take into account any imports happening in the file being parsed
         # and pass them into eval
         typeclass = eval(typename, globals(), locals)
+        if hasattr(typeclass, "__self__") and not isinstance(
+            typeclass.__self__, type(sys)
+        ):
+            # Method of an instance instead of a class.
+            return getattr(typeclass.__self__.__class__, typeclass.__name__)
+
+        if not isinstance(typeclass, (type, type(open), type(class_for_typename))):
+            return typeclass.__class__
         return typeclass
     except (NameError, SyntaxError, AttributeError, TypeError):
         logger.info(f"could not evaluate {typename}")
@@ -116,15 +125,17 @@ class CLikeTranspiler(ast.NodeVisitor):
     NAME: str
 
     builtin_constants = frozenset(["True", "False"])
+    _default_type = _AUTO
+    _type_map = {}
+    _container_type_map = {}
 
     def __init__(self):
-        self._type_map = {}
-        self._headers = set([])
-        self._usings = set([])
+        """Note __init__ is called in ._reset() to reset the transpiler state."""
+        self._headers = set()
+        self._usings = set()
+        self._aliases: Dict[str, str] = {}
         self._imported_names: Dict[str, Any] = {}
-        self._features = set([])
-        self._container_type_map = {}
-        self._default_type = _AUTO
+        self._features = set()
         self._statement_separator = ";"
         self._main_signature_arg_names = []
         self._extension = False
@@ -139,12 +150,28 @@ class CLikeTranspiler(ast.NodeVisitor):
         self._keywords = {}
         self._throw_on_unimplemented = True
 
+    def _reset(self):
+        # Save some settings
+        extension = self._extension
+        throw_on_unimplemented = self._throw_on_unimplemented
+        no_prologue = getattr(self, "_no_prologue", False)
+
+        self.__init__()
+
+        # Re-apply settings
+        self._no_prologue = no_prologue
+        self._extension = extension
+        self._throw_on_unimplemented = throw_on_unimplemented
+
     def headers(self, meta=None):
         return ""
 
     def usings(self):
         return ""
 
+    def aliases(self):
+        return ""
+
     def features(self):
         return ""
 
@@ -161,7 +188,8 @@ class CLikeTranspiler(ast.NodeVisitor):
     def _cast(self, name: str, to) -> str:
         return f"({to}) {name}"
 
-    def _slice_value(self, node: ast.Subscript):
+    @staticmethod
+    def _slice_value(node: ast.Subscript):
         # 3.9 compatibility shim
         if sys.version_info < (3, 9, 0):
             if isinstance(node.slice, ast.Index):
@@ -174,22 +202,27 @@ class CLikeTranspiler(ast.NodeVisitor):
             slice_value = node.slice
         return slice_value
 
-    def _map_type(self, typename, lifetime=LifeTime.UNKNOWN) -> str:
+    @classmethod
+    def _map_type(cls, typename, lifetime=LifeTime.UNKNOWN) -> str:
         if isinstance(typename, list):
             raise NotImplementedError(f"{typename} not supported in this context")
-        typeclass = class_for_typename(typename, self._default_type)
-        return self._type_map.get(typeclass, typename)
+        typeclass = class_for_typename(typename, cls._default_type)
+        return cls._type_map.get(typeclass, typename)
 
-    def _map_types(self, typenames: List[str]) -> List[str]:
-        return [self._map_type(e) for e in typenames]
+    @classmethod
+    def _map_types(cls, typenames: List[str]) -> List[str]:
+        return [cls._map_type(e) for e in typenames]
 
-    def _map_container_type(self, typename) -> str:
-        return self._container_type_map.get(typename, self._default_type)
+    @classmethod
+    def _map_container_type(cls, typename) -> str:
+        return cls._container_type_map.get(typename, cls._default_type)
 
-    def _combine_value_index(self, value_type, index_type) -> str:
+    @classmethod
+    def _combine_value_index(cls, value_type, index_type) -> str:
         return f"{value_type}<{index_type}>"
 
-    def _visit_container_type(self, typename: Tuple) -> str:
+    @classmethod
+    def _visit_container_type(cls, typename: Tuple) -> str:
         value_type, index_type = typename
         if isinstance(index_type, List):
             index_contains_default = "Any" in index_type
@@ -200,19 +233,22 @@ class CLikeTranspiler(ast.NodeVisitor):
         else:
             index_contains_default = index_type == "Any"
         # Avoid types like HashMap<_, foo>. Prefer default_type instead
-        if index_contains_default or value_type == self._default_type:
-            return self._default_type
-        return self._combine_value_index(value_type, index_type)
+        if index_contains_default or value_type == cls._default_type:
+            return cls._default_type
+        return cls._combine_value_index(value_type, index_type)
 
-    def _typename_from_type_node(self, node) -> Union[List, str, None]:
+    @classmethod
+    def _typename_from_type_node(cls, node) -> Union[List, str, None]:
         if isinstance(node, ast.Name):
-            return self._map_type(
+            return cls._map_type(
                 get_id(node), getattr(node, "lifetime", LifeTime.UNKNOWN)
             )
+        elif isinstance(node, ast.Constant) and node.value is not None:
+            return node.value
         elif isinstance(node, ast.ClassDef):
             return get_id(node)
         elif isinstance(node, ast.Tuple):
-            return [self._typename_from_type_node(e) for e in node.elts]
+            return [cls._typename_from_type_node(e) for e in node.elts]
         elif isinstance(node, ast.Attribute):
             node_id = get_id(node)
             if node_id.startswith("typing."):
@@ -222,16 +258,17 @@ class CLikeTranspiler(ast.NodeVisitor):
             # Store a tuple like (List, int) or (Dict, (str, int)) for container types
             # in node.container_type
             # And return a target specific type
-            slice_value = self._slice_value(node)
+            slice_value = cls._slice_value(node)
             (value_type, index_type) = tuple(
-                map(self._typename_from_type_node, (node.value, slice_value))
+                map(cls._typename_from_type_node, (node.value, slice_value))
             )
-            value_type = self._map_container_type(value_type)
+            value_type = cls._map_container_type(value_type)
             node.container_type = (value_type, index_type)
-            return self._combine_value_index(value_type, index_type)
-        return self._default_type
+            return cls._combine_value_index(value_type, index_type)
+        return cls._default_type
 
-    def _generic_typename_from_type_node(self, node) -> Union[List, str, None]:
+    @classmethod
+    def _generic_typename_from_type_node(cls, node) -> Union[List, str, None]:
         if isinstance(node, ast.Name):
             return get_id(node)
         elif isinstance(node, ast.Constant):
@@ -239,51 +276,53 @@ class CLikeTranspiler(ast.NodeVisitor):
         elif isinstance(node, ast.ClassDef):
             return get_id(node)
         elif isinstance(node, ast.Tuple):
-            return [self._generic_typename_from_type_node(e) for e in node.elts]
+            return [cls._generic_typename_from_type_node(e) for e in node.elts]
         elif isinstance(node, ast.Attribute):
             node_id = get_id(node)
             if node_id.startswith("typing."):
                 node_id = node_id.split(".")[1]
             return node_id
         elif isinstance(node, ast.Subscript):
-            slice_value = self._slice_value(node)
+            slice_value = cls._slice_value(node)
             (value_type, index_type) = tuple(
-                map(self._generic_typename_from_type_node, (node.value, slice_value))
+                map(cls._generic_typename_from_type_node, (node.value, slice_value))
             )
             node.generic_container_type = (value_type, index_type)
             return f"{value_type}[{index_type}]"
-        return self._default_type
+        return cls._default_type
 
-    def _typename_from_annotation(self, node, attr="annotation") -> str:
-        default_type = self._default_type
+    @classmethod
+    def _typename_from_annotation(cls, node, attr="annotation") -> str:
+        default_type = cls._default_type
         typename = default_type
         if hasattr(node, attr):
             type_node = getattr(node, attr)
-            typename = self._typename_from_type_node(type_node)
+            typename = cls._typename_from_type_node(type_node)
             if isinstance(type_node, ast.Subscript):
                 node.container_type = type_node.container_type
                 try:
-                    return self._visit_container_type(type_node.container_type)
+                    return cls._visit_container_type(type_node.container_type)
                 except TypeNotSupported as e:
                     raise AstTypeNotSupported(str(e), node)
             if typename is None:
                 raise AstCouldNotInfer(type_node, node)
         return typename
 
+    @classmethod
     def _generic_typename_from_annotation(
-        self, node, attr="annotation"
+        cls, node, attr="annotation"
     ) -> Optional[str]:
         """Unlike the one above, this doesn't do any target specific mapping."""
         typename = None
         if hasattr(node, attr):
             type_node = getattr(node, attr)
-            ret = self._generic_typename_from_type_node(type_node)
+            ret = cls._generic_typename_from_type_node(type_node)
             if isinstance(type_node, ast.Subscript):
                 node.generic_container_type = type_node.generic_container_type
             return ret
         return typename
 
-    def visit(self, node):
+    def visit(self, node) -> str:
         if node is None:
             raise AstEmptyNodeFound
         if type(node) in symbols:
@@ -296,18 +335,16 @@ class CLikeTranspiler(ast.NodeVisitor):
             except Exception as e:
                 raise AstNotImplementedError(e, node) from e
 
-    def visit_Pass(self, node):
+    def visit_Pass(self, node) -> str:
         return self.comment("pass")
 
-    def visit_Module(self, node):
+    def visit_Module(self, node) -> str:
         docstring = getattr(node, "docstring_comment", None)
         buf = [self.comment(docstring.value)] if docstring is not None else []
         filename = getattr(node, "__file__", None)
+        self._reset()
         if filename is not None:
             self._module = Path(filename).stem
-        # TODO: generalize this to reset all state that needs to be reset
-        self._imported_names = {}
-        self._usings.clear()
         body_dict: Dict[ast.AST, str] = OrderedDict()
         for b in node.body:
             if not isinstance(b, ast.FunctionDef):
@@ -324,13 +361,13 @@ class CLikeTranspiler(ast.NodeVisitor):
     def visit_alias(self, node):
         return (node.name, node.asname)
 
-    def _import(self, name: str) -> str:
-        ...
+    def _import(self, name: str) -> str: ...
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
-        ...
+    def _import_from(
+        self, module_name: str, names: List[str], level: int = 0
+    ) -> str: ...
 
-    def visit_Import(self, node):
+    def visit_Import(self, node) -> str:
         names = [self.visit(n) for n in node.names]
         imports = [
             self._import(name)
@@ -346,7 +383,7 @@ class CLikeTranspiler(ast.NodeVisitor):
                 self._imported_names[asname] = imported_name
         return "\n".join(imports)
 
-    def visit_ImportFrom(self, node):
+    def visit_ImportFrom(self, node) -> str:
         if node.module in self._ignored_module_set:
             return ""
 
@@ -369,17 +406,17 @@ class CLikeTranspiler(ast.NodeVisitor):
             else:
                 self._imported_names[asname] = (imported_name, name)
         names = [n for n, _ in names]
-        return self._import_from(imported_name, names)
+        return self._import_from(imported_name, names, node.level)
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id in self.builtin_constants:
             return node.id.lower()
         return node.id
 
-    def visit_Ellipsis(self, node):
+    def visit_Ellipsis(self, node) -> str:
         return self.comment("...")
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is True:
             return "true"
         elif node.value is False:
@@ -391,14 +428,14 @@ class CLikeTranspiler(ast.NodeVisitor):
         else:
             return node.value
 
-    def visit_Constant(self, node):
+    def visit_Constant(self, node) -> str:
         if isinstance(node.value, str):
             return self.visit_Str(node)
         elif isinstance(node.value, bytes):
             return self.visit_Bytes(node)
         return str(self.visit_NameConstant(node))
 
-    def visit_Expr(self, node):
+    def visit_Expr(self, node) -> str:
         s = self.visit(node.value)
         if isinstance(node.value, ast.Constant) and node.value.value is Ellipsis:
             return s
@@ -412,12 +449,15 @@ class CLikeTranspiler(ast.NodeVisitor):
         else:
             return s
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         node_str = node.value
         node_str = node_str.replace('"', '\\"')
+        node_str = node_str.replace("\n", "\\n")
+        node_str = node_str.replace("\r", "\\r")
+        node_str = node_str.replace("\t", "\\t")
         return f'"{node_str}"'
 
-    def visit_Bytes(self, node):
+    def visit_Bytes(self, node) -> str:
         bytes_str = node.s
         byte_array = ", ".join([hex(c) for c in bytes_str])
         return f"{{{byte_array}}}"
@@ -429,9 +469,9 @@ class CLikeTranspiler(ast.NodeVisitor):
         typenames, args = map(list, zip(*args))
         return typenames, args
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         if node.value:
-            return "return {0};".format(self.visit(node.value))
+            return f"return {self.visit(node.value)};"
         return "return;"
 
     def _make_block(self, node):
@@ -455,16 +495,16 @@ class CLikeTranspiler(ast.NodeVisitor):
             and node.rewritten
         )
 
-    def visit_If(self, node, use_parens=True):
+    def visit_If(self, node, use_parens=True) -> str:
         buf = []
         make_block = self.is_block(node)
         if make_block:
             return self._make_block(node)
         else:
             if use_parens:
-                buf.append("if({0}) {{".format(self.visit(node.test)))
+                buf.append(f"if({self.visit(node.test)}) {{")
             else:
-                buf.append("if {0} {{".format(self.visit(node.test)))
+                buf.append(f"if {self.visit(node.test)} {{")
         body = [self.visit(child) for child in node.body]
         body = [b for b in body if b is not None]
         buf.extend(body)
@@ -478,23 +518,23 @@ class CLikeTranspiler(ast.NodeVisitor):
             buf.append("}")
         return "\n".join(buf)
 
-    def visit_Continue(self, node):
+    def visit_Continue(self, node) -> str:
         return "continue;"
 
-    def visit_Break(self, node):
+    def visit_Break(self, node) -> str:
         return "break;"
 
-    def visit_While(self, node, use_parens=True):
+    def visit_While(self, node, use_parens=True) -> str:
         buf = []
         if use_parens:
-            buf.append("while ({0}) {{".format(self.visit(node.test)))
+            buf.append(f"while ({self.visit(node.test)}) {{")
         else:
-            buf.append("while {0} {{".format(self.visit(node.test)))
+            buf.append(f"while {self.visit(node.test)} {{")
         buf.extend([self.visit(n) for n in node.body])
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_Compare(self, node):
+    def visit_Compare(self, node) -> str:
         if isinstance(node.ops[0], ast.In):
             return self.visit_In(node)
 
@@ -502,28 +542,27 @@ class CLikeTranspiler(ast.NodeVisitor):
         op = self.visit(node.ops[0])
         right = self.visit(node.comparators[0])
 
-        return "{0} {1} {2}".format(left, op, right)
+        return f"{left} {op} {right}"
 
-    def visit_BoolOp(self, node):
+    def visit_BoolOp(self, node) -> str:
         op = self.visit(node.op)
         return op.join([self.visit(v) for v in node.values])
 
-    def visit_UnaryOp(self, node):
-        return "{0}({1})".format(self.visit(node.op), self.visit(node.operand))
+    def visit_UnaryOp(self, node) -> str:
+        return f"{self.visit(node.op)}({self.visit(node.operand)})"
 
-    def _visit_AssignOne(self, node, target) -> str:
-        ...
+    def _visit_AssignOne(self, node, target) -> str: ...
 
-    def visit_Assign(self, node):
+    def visit_Assign(self, node) -> str:
         return "\n".join(
             [self._visit_AssignOne(node, target) for target in node.targets]
         )
 
-    def visit_AugAssign(self, node):
+    def visit_AugAssign(self, node) -> str:
         target = self.visit(node.target)
         op = self.visit(node.op)
         val = self.visit(node.value)
-        return "{0} {1}= {2};".format(target, op, val)
+        return f"{target} {op}= {val};"
 
     def visit_AnnAssign(self, node):
         target = self.visit(node.target)
@@ -541,7 +580,7 @@ class CLikeTranspiler(ast.NodeVisitor):
     def set_continue_on_unimplemented(self):
         self._throw_on_unimplemented = False
 
-    def visit_unsupported_body(self, node, name, body):
+    def visit_unsupported(self, node, name) -> str:
         if self._throw_on_unimplemented:
             raise AstNotImplementedError(f"{name} not implemented", node)
         else:
@@ -549,48 +588,59 @@ class CLikeTranspiler(ast.NodeVisitor):
                 f"{name} unimplemented on line {node.lineno}:{node.col_offset}"
             )
 
-    def visit_NamedExpr(self, node):
+    def visit_unsupported_body(self, node, name, body) -> str:
+        # TODO: body should not be ignored
+        return self.visit_unsupported(node, name)
+
+    def visit_NamedExpr(self, node) -> str:
         target = self.visit(node.target)
         return self.visit_unsupported_body(node, f"named expr {target}", node.value)
 
-    def visit_Delete(self, node):
+    def visit_Delete(self, node) -> str:
         body = [self.visit(t) for t in node.targets]
         return self.visit_unsupported_body(node, "del", body)
 
-    def visit_Await(self, node):
+    def visit_Starred(self, node) -> str:
+        return self.visit_unsupported_body(node, "starred", node.value)
+
+    def visit_Await(self, node) -> str:
         return self.visit_unsupported_body(node, "await", node.value)
 
-    def visit_AsyncFor(self, node):
+    def visit_AsyncFor(self, node) -> str:
         target = self.visit(node.target)
         iter = self.visit(node.iter)
         return self.visit_unsupported_body(
             node, f"async for {target} in {iter}", node.body
         )
 
-    def visit_AsyncWith(self, node):
+    def visit_AsyncWith(self, node) -> str:
         items = [self.visit(i) for i in node.items]
         return self.visit_unsupported_body(node, f"async with {items}", node.body)
 
-    def visit_YieldFrom(self, node):
+    def visit_YieldFrom(self, node) -> str:
         return self.visit_unsupported_body(node, "yield from", node.value)
 
-    def visit_AsyncFunctionDef(self, node):
+    def visit_AsyncFunctionDef(self, node) -> str:
         return self.visit_unsupported_body(node, "async def", node.body)
 
-    def visit_Nonlocal(self, node):
+    def visit_Nonlocal(self, node) -> str:
         return self.visit_unsupported_body(node, "nonlocal", node.names)
 
-    def visit_DictComp(self, node):
+    def visit_DictComp(self, node) -> str:
         key = self.visit(node.key)
         value = self.visit(node.value)
         return self.visit_unsupported_body(
             node, f"dict comprehension ({key}, {value})", node.generators
         )
 
-    def visit_ListComp(self, node):
+    def visit_GeneratorExp(self, node) -> str:
+        self.visit_unsupported_body(node, "generator expression elts", node.elt)
+        return self.visit_unsupported_body(node, "generators", node.generators)
+
+    def visit_ListComp(self, node) -> str:
         return self.visit_GeneratorExp(node)  # by default, they are the same
 
-    def visit_SetComp(self, node):
+    def visit_SetComp(self, node) -> str:
         return self.visit_GeneratorExp(node)  # by default, they are the same
 
     def visit_ClassDef(self, node):
@@ -606,21 +656,44 @@ class CLikeTranspiler(ast.NodeVisitor):
         if bases == ["IntFlag"]:
             return self.visit_IntFlag(node)
 
-    def visit_StrEnum(self, node):
+    def visit_StrEnum(self, node) -> str:
         raise Exception("Unimplemented")
 
-    def visit_IntEnum(self, node):
+    def visit_IntEnum(self, node) -> str:
         raise Exception("Unimplemented")
 
-    def visit_IntFlag(self, node):
+    def visit_IntFlag(self, node) -> str:
         raise Exception("Unimplemented")
 
-    def visit_IfExp(self, node):
+    def visit_IfExp(self, node) -> str:
         body = self.visit(node.body)
         orelse = self.visit(node.orelse)
         test = self.visit(node.test)
         return f"({test}? ({{ {body}; }}) : ({{ {orelse}; }}))"
 
+    def visit_ExceptHandler(self, node) -> str:
+        return self.visit_unsupported_body(node, "except handler", node.body)
+
+    def visit_Try(self, node, finallybody=None) -> str:
+        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
+
+        for handler in node.handlers:
+            buf += self.visit(handler)
+        # buf.append("\n".join(excepts));
+
+        if finallybody:
+            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
+
+        return "\n".join(buf)
+
+    def visit_Raise(self, node) -> str:
+        if node.exc is not None:
+            return self.visit_unsupported_body(node, "raise", node.exc)
+            return f"raise!({self.visit(node.exc)}); //unsupported"
+        # This handles the case where `raise` is used without
+        # specifying the exception.
+        return self.visit_unsupported(node, "raise")
+
     def _func_for_lookup(self, fname) -> Union[str, object]:
         func = class_for_typename(fname, None, self._imported_names)
         if func is None:
