@@ -1,4 +1,5 @@
 import ast
+
 from .scope import ScopeMixin
 
 
@@ -113,7 +114,12 @@ class VariableTransformer(ast.NodeTransformer, ScopeMixin):
 
     def visit_For(self, node):
         node.target.assigned_from = node
-        node.vars = [node.target]
+        if isinstance(node.target, ast.Name):
+            node.vars = [node.target]
+        elif isinstance(node.target, ast.Tuple):
+            node.vars = [*node.target.elts]
+        else:
+            node.vars = []
         self.generic_visit(node)
         return node
 
@@ -136,6 +142,7 @@ class VariableTransformer(ast.NodeTransformer, ScopeMixin):
             if isinstance(target, ast.Name):
                 target.assigned_from = node
                 self.scope.vars.append(target)
+        self.generic_visit(node)
         return node
 
     def visit_AnnAssign(self, node):
@@ -143,6 +150,7 @@ class VariableTransformer(ast.NodeTransformer, ScopeMixin):
         if isinstance(target, ast.Name):
             target.assigned_from = node
             self.scope.vars.append(target)
+        self.generic_visit(node)
         return node
 
     def visit_AugAssign(self, node):
@@ -150,6 +158,7 @@ class VariableTransformer(ast.NodeTransformer, ScopeMixin):
         if isinstance(target, ast.Name):
             target.assigned_from = node
             self.scope.vars.append(target)
+        self.generic_visit(node)
         return node
 
 
