@@ -1,9 +1,16 @@
 import ast
-
-from ctypes import c_int8, c_int16, c_int32, c_int64
-from ctypes import c_uint8, c_uint16, c_uint32, c_uint64
+from ctypes import (
+    c_int8,
+    c_int16,
+    c_int32,
+    c_int64,
+    c_uint8,
+    c_uint16,
+    c_uint32,
+    c_uint64,
+)
 from dataclasses import dataclass
-from typing import cast, Optional
+from typing import Optional, Set, cast
 
 from py2many.analysis import get_id
 from py2many.ast_helpers import create_ast_node
@@ -12,6 +19,15 @@ from py2many.clike import CLikeTranspiler, class_for_typename
 from py2many.exceptions import AstIncompatibleAssign, AstUnrecognisedBinOp
 from py2many.tracer import is_enum
 
+try:
+    from typpete.context import Context
+    from typpete.inference_runner import infer as infer_types_ast
+    from typpete.z3_types import TypesSolver
+except ModuleNotFoundError:
+
+    def infer_types_ast(node):
+        return node
+
 
 @dataclass
 class InferMeta:
@@ -24,6 +40,17 @@ def infer_types(node) -> InferMeta:
     return InferMeta(visitor.has_fixed_width_ints)
 
 
+def infer_types_typpete(node) -> InferMeta:
+    solver = TypesSolver(node)
+    context = Context(node, node.body, solver)
+    for stmt in node.body:
+        infer_types_ast(stmt, context, solver)
+
+    solver.push()
+
+    return InferMeta(True)
+
+
 def get_inferred_type(node):
     if isinstance(node, ast.Name):
         if not hasattr(node, "scopes"):
@@ -32,7 +59,7 @@ def get_inferred_type(node):
         # Prevent infinite recursion
         if definition != node and definition is not None:
             return get_inferred_type(definition)
-    elif isinstance(node, ast.Constant) or isinstance(node, ast.NameConstant):
+    elif isinstance(node, ast.Constant):
         return InferTypesTransformer._infer_primitive(node.value)
     if hasattr(node, "annotation"):
         return node.annotation
@@ -78,6 +105,51 @@ def is_compatible(
     return True
 
 
+class LanguageInferenceBase:
+    """Base class for language-specific type inference"""
+
+    TYPE_MAP = {}  # Should be overridden by subclasses
+    CONTAINER_TYPE_MAP = {}  # Should be overridden by subclasses
+    WIDTH_RANK = {}  # Should be overridden by subclasses
+
+    @classmethod
+    def map_type(cls, typename):
+        typeclass = class_for_typename(typename, None)
+        return cls.TYPE_MAP.get(typeclass, typename)
+
+    @classmethod
+    def get_inferred_language_type(cls, node, annotation_attr):
+        if hasattr(node, annotation_attr):
+            return getattr(node, annotation_attr)
+        if isinstance(node, ast.Call):
+            fname = get_id(node.func)
+            if fname in {"max", "min", "floor"}:
+                return cls.TYPE_MAP.get(float, "float64")
+        if isinstance(node, ast.Name):
+            if not hasattr(node, "scopes"):
+                return None
+            definition = node.scopes.find(get_id(node))
+            # Prevent infinite recursion
+            if definition != node:
+                return cls.get_inferred_language_type(definition, annotation_attr)
+        python_type = get_inferred_type(node)
+        ret = cls.map_type(get_id(python_type))
+        if node is not None:
+            setattr(node, annotation_attr, ret)
+        return ret
+
+    @classmethod
+    def handle_overflow(cls, op, left_id, right_id):
+        """Handle numeric type widening for binary operations"""
+        left_lang_id = cls.map_type(left_id)
+        right_lang_id = cls.map_type(right_id)
+
+        left_rank = cls.WIDTH_RANK.get(left_lang_id, -1)
+        right_rank = cls.WIDTH_RANK.get(right_lang_id, -1)
+
+        return left_lang_id if left_rank > right_rank else right_lang_id
+
+
 class InferTypesTransformer(ast.NodeTransformer):
     """
     Tries to infer types
@@ -146,8 +218,6 @@ class InferTypesTransformer(ast.NodeTransformer):
     def __init__(self):
         self.handling_annotation = False
         self.has_fixed_width_ints = False
-        # TODO: remove this and make the methods into classmethods
-        self._clike = CLikeTranspiler()
 
     @staticmethod
     def _infer_primitive(value) -> Optional[ast.AST]:
@@ -195,10 +265,17 @@ class InferTypesTransformer(ast.NodeTransformer):
             if getattr(node, "is_annotation", False):
                 return node
             else:
-                elt_types = set([get_id(get_inferred_type(e)) for e in elements])
-                if len(elt_types) == 1 and hasattr(elements[0], "annotation"):
-                    elt_type = get_id(elements[0].annotation)
-                    self._annotate(node, f"List[{elt_type}]")
+                elt_types: Set[str] = set()
+                for e in elements:
+                    typ = get_inferred_type(e)
+                    if typ is not None:
+                        elt_types.add(ast.unparse(typ))
+                if len(elt_types) == 0:
+                    node.annotation = ast.Name(id="List")
+                elif len(elt_types) == 1:
+                    self._annotate(node, f"List[{elt_types.pop()}]")
+                else:
+                    self._annotate(node, f"List[Union[{', '.join(elt_types)}]]")
         else:
             if not hasattr(node, "annotation"):
                 node.annotation = ast.Name(id="List")
@@ -208,7 +285,7 @@ class InferTypesTransformer(ast.NodeTransformer):
         self.generic_visit(node)
         if len(node.elts) > 0:
             elements = [self.visit(e) for e in node.elts]
-            elt_types = set([get_id(get_inferred_type(e)) for e in elements])
+            elt_types = {get_id(get_inferred_type(e)) for e in elements}
             if len(elt_types) == 1:
                 if hasattr(elements[0], "annotation"):
                     elt_type = get_id(elements[0].annotation)
@@ -224,28 +301,26 @@ class InferTypesTransformer(ast.NodeTransformer):
 
             def typename(e):
                 get_inferred_type(e)  # populates e.annotation
-                return self._clike._generic_typename_from_annotation(e)
+                return CLikeTranspiler._generic_typename_from_annotation(e)
 
-            key_types = set([typename(e) for e in node.keys])
+            key_types = {typename(e) for e in node.keys}
             only_key_type = next(iter(key_types))
             if len(key_types) == 1:
                 key_type = only_key_type
             else:
                 key_type = "Any"
-            value_types = set([typename(e) for e in node.values])
+            value_types = {typename(e) for e in node.values}
             only_value_type = next(iter(value_types))
             if len(value_types) == 1:
                 value_type = only_value_type
             else:
                 value_type = "Any"
             self._annotate(node, f"Dict[{key_type}, {value_type}]")
-            lifetimes = set(
-                [
-                    getattr(e.annotation, "lifetime", None)
-                    for e in node.values
-                    if hasattr(e, "annotation")
-                ]
-            )
+            lifetimes = {
+                getattr(e.annotation, "lifetime", None)
+                for e in node.values
+                if hasattr(e, "annotation")
+            }
             only_lifetime = next(iter(lifetimes)) if len(lifetimes) == 1 else None
             if len(lifetimes) == 1 and only_lifetime != None:
                 lifetime = only_lifetime
@@ -259,6 +334,7 @@ class InferTypesTransformer(ast.NodeTransformer):
 
     def visit_Assign(self, node: ast.Assign) -> ast.AST:
         self.generic_visit(node)
+        self.visit(node.value)
 
         annotation = getattr(node.value, "annotation", None)
         if annotation is None:
@@ -282,11 +358,11 @@ class InferTypesTransformer(ast.NodeTransformer):
 
         node.target.annotation = node.annotation
         target = node.target
-        target_typename = self._clike._typename_from_annotation(target)
+        target_typename = CLikeTranspiler._typename_from_annotation(target)
         if target_typename in self.FIXED_WIDTH_INTS_NAME:
             self.has_fixed_width_ints = True
         annotation = get_inferred_type(node.value)
-        value_typename = self._clike._generic_typename_from_type_node(annotation)
+        value_typename = CLikeTranspiler._generic_typename_from_type_node(annotation)
         target_class = class_for_typename(target_typename, None)
         value_class = class_for_typename(value_typename, None)
         if (
@@ -462,10 +538,7 @@ class InferTypesTransformer(ast.NodeTransformer):
             node.annotation = ast.Name(id=left_id)
             return node
 
-        LEGAL_COMBINATIONS = {
-            ("str", ast.Mod),
-            ("List", ast.Add),
-        }
+        LEGAL_COMBINATIONS = {("str", ast.Mod), ("List", ast.Add)}
 
         if left_id is not None and (left_id, type(node.op)) not in LEGAL_COMBINATIONS:
             raise AstUnrecognisedBinOp(left_id, right_id, node)
@@ -487,9 +560,14 @@ class InferTypesTransformer(ast.NodeTransformer):
     def visit_Call(self, node):
         fname = get_id(node.func)
         if fname is not None:
+            # Handle methods calls by looking up the method name
+            # without the prefix
+            # TODO: use remove suffix
+            if fname.startswith("self."):
+                fname = fname.split(".", 1)[1]
             fn = node.scopes.find(fname)
             if isinstance(fn, ast.ClassDef):
-                node.annotation = fn
+                self._annotate(node, fn.name)
             elif isinstance(fn, ast.FunctionDef):
                 return_type = (
                     fn.returns if hasattr(fn, "returns") and fn.returns else None
@@ -511,13 +589,38 @@ class InferTypesTransformer(ast.NodeTransformer):
     def visit_Subscript(self, node):
         definition = node.scopes.find(get_id(node.value))
         if hasattr(definition, "annotation"):
-            self._clike._typename_from_annotation(definition)
+            CLikeTranspiler._typename_from_annotation(definition)
             if hasattr(definition, "container_type"):
                 container_type, element_type = definition.container_type
                 if container_type == "Dict" or isinstance(element_type, list):
                     element_type = element_type[1]
-                node.annotation = ast.Name(id=element_type)
+                # we could check for lower/upper also
+                if hasattr(node.slice, "step"):
+                    node.annotation = definition.annotation
+                else:
+                    node.annotation = ast.Name(id=element_type)
                 if hasattr(definition.annotation, "lifetime"):
                     node.annotation.lifetime = definition.annotation.lifetime
         self.generic_visit(node)
         return node
+
+    def visit_For(self, node):
+        self.visit(node.target)
+        self.visit(node.iter)
+        if hasattr(node.iter, "annotation") and isinstance(
+            node.iter.annotation, ast.Subscript
+        ):
+            typ = CLikeTranspiler._slice_value(node.iter.annotation)
+
+            if isinstance(node.target, ast.Name):
+                node.target.annotation = typ
+            elif isinstance(node.target, ast.Tuple) and isinstance(typ, ast.Subscript):
+                typ = CLikeTranspiler._slice_value(typ)
+                for e in node.target.elts:
+                    e.annotation = typ
+        self.generic_visit(node)
+        return node
+
+    def visit_BoolOp(self, node: ast.BoolOp):
+        node.annotation = ast.Name(id="bool")
+        return node
