@@ -14,40 +14,27 @@ class NestingTransformer(ast.NodeTransformer):
     def __init__(self):
         self.level = 0
 
-    def visit_FunctionDef(self, node):
+    def _visit_level(self, node) -> ast.AST:
         node.level = self.level
         self.level += 1
         self.generic_visit(node)
         self.level -= 1
         return node
 
+    def visit_FunctionDef(self, node):
+        return self._visit_level(node)
+
     def visit_ClassDef(self, node):
-        node.level = self.level
-        self.level += 1
-        self.generic_visit(node)
-        self.level -= 1
-        return node
+        return self._visit_level(node)
 
     def visit_If(self, node):
-        node.level = self.level
-        self.level += 1
-        self.generic_visit(node)
-        self.level -= 1
-        return node
+        return self._visit_level(node)
 
     def visit_While(self, node):
-        node.level = self.level
-        self.level += 1
-        self.generic_visit(node)
-        self.level -= 1
-        return node
+        return self._visit_level(node)
 
     def visit_For(self, node):
-        node.level = self.level
-        self.level += 1
-        self.generic_visit(node)
-        self.level -= 1
-        return node
+        return self._visit_level(node)
 
     def visit_Assign(self, node):
         node.level = self.level
