@@ -1,24 +1,22 @@
 import ast
-import textwrap
-
 from typing import List
 
-from .clike import CLikeTranspiler
-from .plugins import (
-    ATTR_DISPATCH_TABLE,
-    CLASS_DISPATCH_TABLE,
-    FUNC_DISPATCH_TABLE,
-    MODULE_DISPATCH_TABLE,
-    DISPATCH_MAP,
-    SMALL_DISPATCH_MAP,
-    SMALL_USINGS_MAP,
-)
-
 from py2many.analysis import get_id, is_mutable, is_void_function
 from py2many.clike import class_for_typename
 from py2many.declaration_extractor import DeclarationExtractor
 from py2many.inference import get_inferred_type
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_self_arg
+from py2many.tracer import defined_before, is_class_or_module, is_list, is_self_arg
+
+from .clike import CLikeTranspiler
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    FUNC_DISPATCH_TABLE,
+    MODULE_DISPATCH_TABLE,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
 
 class DartIntegerDivRewriter(ast.NodeTransformer):
@@ -27,7 +25,7 @@ class DartIntegerDivRewriter(ast.NodeTransformer):
         if isinstance(node.op, ast.Div):
             left_type = get_id(get_inferred_type(node.left))
             right_type = get_id(get_inferred_type(node.right))
-            if set([left_type, right_type]) == {"int"}:
+            if {left_type, right_type} == {"int"}:
                 # This attribute should technically be on node.op
                 # But python seems to use the same AST node for other
                 # division operations?
@@ -38,17 +36,9 @@ class DartIntegerDivRewriter(ast.NodeTransformer):
 class DartTranspiler(CLikeTranspiler):
     NAME = "dart"
 
-    CONTAINER_TYPE_MAP = {
-        "List": "List",
-        "Dict": "Map",
-        "Set": "Set",
-        "Optional": "Nothing",
-    }
-
     def __init__(self):
         super().__init__()
-        self._container_type_map = self.CONTAINER_TYPE_MAP
-        self._default_type = "var"
+        CLikeTranspiler._default_type = "var"
         self._temp = 0
         self._dispatch_map = DISPATCH_MAP
         self._small_dispatch_map = SMALL_DISPATCH_MAP
@@ -64,9 +54,9 @@ class DartTranspiler(CLikeTranspiler):
     def usings(self):
         usings = sorted(list(set(self._usings)))
         uses = "\n".join(f"import '{mod}';" for mod in usings)
-        return f"// @dart=2.9\n{uses}" if uses else ""
+        return f"// @dart=3.4\n{uses}" if uses else ""
 
-    def visit_FunctionDef(self, node):
+    def visit_FunctionDef(self, node) -> str:
         body = "\n".join([self.visit(n) for n in node.body])
         typenames, args = self.visit(node.args)
 
@@ -87,7 +77,7 @@ class DartTranspiler(CLikeTranspiler):
                 continue
 
             if typename == "T":
-                typename = "T{0}".format(index)
+                typename = f"T{index}"
                 typedecls.append(typename)
                 index += 1
             args_list.append(f"{typename} {arg}")
@@ -104,15 +94,15 @@ class DartTranspiler(CLikeTranspiler):
 
         template = ""
         if len(typedecls) > 0:
-            template = "<{0}>".format(", ".join(typedecls))
+            template = "<{}>".format(", ".join(typedecls))
 
         args = ", ".join(args_list)
         funcdef = f"{return_type} {node.name}{template}({args}) {{"
         return funcdef + "\n" + body + "}\n\n"
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         if node.value:
-            return "return {0};".format(self.visit(node.value))
+            return f"return {self.visit(node.value)};"
         return "return;"
 
     def visit_arg(self, node):
@@ -124,13 +114,13 @@ class DartTranspiler(CLikeTranspiler):
             typename = self._typename_from_annotation(node)
         return (typename, id)
 
-    def visit_Lambda(self, node):
+    def visit_Lambda(self, node) -> str:
         _, args = self.visit(node.args)
         args_string = ", ".join(args)
         body = self.visit(node.body)
         return f"({args_string}) => {body}"
 
-    def visit_Attribute(self, node):
+    def visit_Attribute(self, node) -> str:
         attr = node.attr
 
         value_id = self.visit(node.value)
@@ -154,7 +144,7 @@ class DartTranspiler(CLikeTranspiler):
 
         return f"{value_id}.{attr}"
 
-    def visit_Call(self, node):
+    def visit_Call(self, node) -> str:
         fname = self.visit(node.func)
         vargs = []
 
@@ -172,23 +162,23 @@ class DartTranspiler(CLikeTranspiler):
             args = ""
         return f"{fname}({args})"
 
-    def visit_For(self, node):
+    def visit_For(self, node) -> str:
         target = self.visit(node.target)
         it = self.visit(node.iter)
         buf = []
-        buf.append("for (final {0} in {1}) {{".format(target, it))
+        buf.append(f"for (final {target} in {it}) {{")
         buf.extend([self.visit(c) for c in node.body])
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         return "" + super().visit_Str(node) + ""
 
-    def visit_Bytes(self, node):
-        bytes_str = "{0}".format(node.s)
+    def visit_Bytes(self, node) -> str:
+        bytes_str = f"{node.s}"
         return bytes_str.replace("'", '"')  # replace single quote with double quote
 
-    def visit_Compare(self, node):
+    def visit_Compare(self, node) -> str:
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
         if hasattr(node.comparators[0], "annotation"):
@@ -203,9 +193,9 @@ class DartTranspiler(CLikeTranspiler):
             right = right[:-2]
 
         if isinstance(node.ops[0], ast.In):
-            return "{0}.contains({1})".format(right, left)
+            return f"{right}.contains({left})"
         elif isinstance(node.ops[0], ast.NotIn):
-            return "!({0}.contains({1}))".format(right, left)
+            return f"!({right}.contains({left}))"
         elif isinstance(node.ops[0], ast.Eq):
             if hasattr(node.left, "annotation"):
                 self._generic_typename_from_annotation(node.left)
@@ -218,7 +208,7 @@ class DartTranspiler(CLikeTranspiler):
 
         return super().visit_Compare(node)
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         exception_name_map = {"ZeroDivisionError": "IntegerDivisionByZeroException"}
         if node.id == "None":
             return "null"
@@ -227,7 +217,7 @@ class DartTranspiler(CLikeTranspiler):
         else:
             return super().visit_Name(node)
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is None:
             return "null"
         else:
@@ -240,9 +230,9 @@ class DartTranspiler(CLikeTranspiler):
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_If(self, node):
-        body_vars = set([get_id(v) for v in node.scopes[-1].body_vars])
-        orelse_vars = set([get_id(v) for v in node.scopes[-1].orelse_vars])
+    def visit_If(self, node) -> str:
+        body_vars = {get_id(v) for v in node.scopes[-1].body_vars}
+        orelse_vars = {get_id(v) for v in node.scopes[-1].orelse_vars}
         node.common_vars = body_vars.intersection(orelse_vars)
 
         var_definitions = []
@@ -256,29 +246,17 @@ class DartTranspiler(CLikeTranspiler):
 
         return decls + "\n\n" + super().visit_If(node)
 
-    def visit_UnaryOp(self, node):
+    def visit_UnaryOp(self, node) -> str:
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_BinOp(self, node):
-        if (
-            isinstance(node.left, ast.List)
-            and isinstance(node.op, ast.Mult)
-            and isinstance(node.right, ast.Num)
-        ):
-            return "std::vector ({0},{1})".format(
-                self.visit(node.right), self.visit(node.left.elts[0])
-            )
-        else:
-            return super().visit_BinOp(node)
-
-    def visit_ClassDef(self, node):
+    def visit_ClassDef(self, node) -> str:
         extractor = DeclarationExtractor(DartTranspiler())
         extractor.visit(node)
         declarations = node.declarations = extractor.get_declarations()
@@ -303,7 +281,7 @@ class DartTranspiler(CLikeTranspiler):
         constructor = ""
         for declaration, typename in declarations.items():
             if typename == None:
-                typename = "ST{0}".format(index)
+                typename = f"ST{index}"
                 index += 1
             fields.append(f"{typename} {declaration};")
 
@@ -322,70 +300,48 @@ class DartTranspiler(CLikeTranspiler):
 
         return f"class {node.name} {{\n{fields}\n\n {body}\n}}\n"
 
-    def visit_IntEnum(self, node):
-        # TODO: Consider using Vnum instead of the language built-in
-        # Any python enum which specifies a non default value will break this
+    def visit_IntEnum(self, node) -> str:
         fields = []
         for member, var in node.class_assignments.items():
             var = self.visit(var)
             if var == "auto()":
-                fields.append(f"{member},")
+                fields.append(f"{member}")
             else:
-                fields.append(f"{member} = {var},")
-        fields = "\n".join(fields)
-        return f"enum {node.name} {{\n{fields}\n}}\n\n"
+                fields.append(f"{member}({var})")
+        fields = ",\n".join(fields)
+        constructor = "\n"
+        return f"enum {node.name} {{\n{fields};\n{constructor}}}\n\n"
 
-    def visit_StrEnum(self, node):
-        # TODO: Dedup with other enum implementations
-        self._usings.add("package:vnum/vnum.dart")
+    def visit_StrEnum(self, node) -> str:
         fields = []
-        ename = node.name
-        for i, (member, var) in enumerate(node.class_assignments.items()):
+        for member, var in node.class_assignments.items():
             var = self.visit(var)
             if var == "auto()":
-                var = f'"{member}"'
-            fields.append(
-                f"            static final {member} = const {ename}.define({var});"
-            )
-        fields = "\n".join(["    " * 2 + f for f in fields])
-        return textwrap.dedent(
-            f"""\
-            @VnumDefinition
-            class {node.name} extends Vnum<String> {{\n{fields}
+                fields.append(f"{member}")
+            else:
+                fields.append(f"{member}({var})")
+        fields = ",\n".join(fields)
+        constructor = f"const {node.name}(this.__private);\n"
+        constructor += "final String __private;"
+        return f"enum {node.name} {{\n{fields};\n{constructor}}}\n\n"
 
-            const {node.name}.define(String fromValue) : super.define(fromValue);
-              factory {node.name}(String value) => Vnum.fromValue(value, {node.name});
-            }}
-            """
-        )
-
-    def visit_IntFlag(self, node):
-        self._usings.add("package:vnum/vnum.dart")
+    def visit_IntFlag(self, node) -> str:
         fields = []
-        ename = node.name
-        for i, (member, var) in enumerate(node.class_assignments.items()):
+        for member, var in node.class_assignments.items():
             var = self.visit(var)
             if var == "auto()":
-                var = i
-            fields.append(
-                f"            static final {member} = const {ename}.define({var});"
-            )
-        fields = "\n".join(["    " * 2 + f for f in fields])
-        return textwrap.dedent(
-            f"""\
-            @VnumDefinition
-            class {node.name} extends Vnum<int> {{\n{fields}
-
-            const {node.name}.define(int fromValue) : super.define(fromValue);
-              factory {node.name}(int value) => Vnum.fromValue(value, {node.name});
-            }}
-            """
-        )
+                fields.append(f"{member}")
+            else:
+                fields.append(f"{member}({var})")
+        fields = ",\n".join(fields)
+        constructor = f"const {node.name}(this.__private);\n"
+        constructor += "final int __private;"
+        return f"enum {node.name} {{\n{fields};\n{constructor}}}\n\n"
 
     def _import(self, name: str) -> str:
         return f'import "{name}";'
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         if len(names) == 1:
             # TODO: make this more generic so it works for len(names) > 1
             name = names[0]
@@ -398,37 +354,37 @@ class DartTranspiler(CLikeTranspiler):
         names = ", ".join(names)
         return f'import "{module_name}";  // {names}'
 
-    def visit_List(self, node):
+    def visit_List(self, node) -> str:
         if len(node.elts) > 0:
             elements = [self.visit(e) for e in node.elts]
-            return "[{0}]".format(", ".join(elements))
+            return "[{}]".format(", ".join(elements))
 
         else:
             return "[]"
 
-    def visit_Dict(self, node):
+    def visit_Dict(self, node) -> str:
         keys = [self.visit(k) for k in node.keys]
         values = [self.visit(k) for k in node.values]
         kv_pairs = ", ".join([f"{k} : {v}" for k, v in zip(keys, values)])
         return f"{{{kv_pairs}}}"
 
-    def visit_Subscript(self, node):
+    def visit_Subscript(self, node) -> str:
         value = self.visit(node.value)
         index = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
-            if value in self.CONTAINER_TYPE_MAP:
-                value = self.CONTAINER_TYPE_MAP[value]
+            if value in self._container_type_map:
+                value = self._container_type_map[value]
             if value == "Tuple":
-                return "({0})".format(index)
-            return "{0}<{1}>".format(value, index)
+                return f"({index})"
+            return f"{value}<{index}>"
         if getattr(node, "lhs", False):
-            return "{0}[{1}]".format(value, index)
-        return '({0}[{1}] ?? (throw Exception("key not found")))'.format(value, index)
+            return f"{value}[{index}]"
+        return f'({value}[{index}] ?? (throw Exception("key not found")))'
 
-    def visit_Index(self, node):
+    def visit_Index(self, node) -> str:
         return self.visit(node.value)
 
-    def visit_Slice(self, node):
+    def visit_Slice(self, node) -> str:
         lower = ""
         if node.lower:
             lower = self.visit(node.lower)
@@ -436,21 +392,21 @@ class DartTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Tuple(self, node):
+    def visit_Tuple(self, node) -> str:
         elts = [self.visit(e) for e in node.elts]
         elts = ", ".join(elts)
         if hasattr(node, "is_annotation"):
             return elts
-        return "({0})".format(elts)
+        return f"({elts})"
 
-    def visit_Raise(self, node):
+    def visit_Raise(self, node) -> str:
         if node.exc is not None:
-            return "throw new {};".format(self.visit(node.exc))
+            return f"throw new {self.visit(node.exc)};"
         return "throw new Exception();"
 
-    def visit_Try(self, node):
+    def visit_Try(self, node) -> str:
         buf = []
         buf.append("try {")
         buf.extend([self.visit(child) for child in node.body])
@@ -466,7 +422,7 @@ class DartTranspiler(CLikeTranspiler):
 
         return "\n".join(buf)
 
-    def visit_ExceptHandler(self, node):
+    def visit_ExceptHandler(self, node) -> str:
         buf = []
         if node.type:
             type_str = self.visit(node.type)
@@ -480,15 +436,15 @@ class DartTranspiler(CLikeTranspiler):
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_Assert(self, node):
+    def visit_Assert(self, node) -> str:
         condition = self.visit(node.test)
         return f"assert({condition});"
 
-    def visit_AnnAssign(self, node):
+    def visit_AnnAssign(self, node) -> str:
         target, type_str, val = super().visit_AnnAssign(node)
         return f"{type_str} {target} = {val};"
 
-    def _visit_AssignOne(self, node, target):
+    def _visit_AssignOne(self, node, target) -> str:
         kw = "var" if is_mutable(node.scopes, get_id(target)) else "final"
 
         if isinstance(target, ast.Tuple):
@@ -503,6 +459,12 @@ class DartTranspiler(CLikeTranspiler):
                 buf.extend([f"{elt} = {tmp_var}.item{i+1};"])
             return "\n".join(buf)
 
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is not None:
+            kw = ""
+        else:
+            definition = node.scopes.find(get_id(target))
+
         if isinstance(node.scopes[-1], ast.If):
             outer_if = node.scopes[-1]
             target_id = self.visit(target)
@@ -515,9 +477,6 @@ class DartTranspiler(CLikeTranspiler):
             value = self.visit(node.value)
             return f"{target} = {value};"
 
-        definition = node.scopes.parent_scopes.find(get_id(target))
-        if definition is None:
-            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             target = self.visit(target)
             value = self.visit(node.value)
@@ -535,27 +494,14 @@ class DartTranspiler(CLikeTranspiler):
 
             return f"{kw} {typename} {target} = {value};"
 
-    def visit_Delete(self, node):
-        target = node.targets[0]
-        return "{0}.drop()".format(self.visit(target))
-
-    def visit_Await(self, node):
-        return "await!({0})".format(self.visit(node.value))
-
-    def visit_AsyncFunctionDef(self, node):
-        return "#[async]\n{0}".format(self.visit_FunctionDef(node))
-
-    def visit_Yield(self, node):
-        return "//yield is unimplemented"
-
-    def visit_Print(self, node):
+    def visit_Print(self, node) -> str:
         buf = []
         for n in node.values:
             value = self.visit(n)
-            buf.append('println("{{:?}}",{0})'.format(value))
+            buf.append(f'println("{{:?}}",{value})')
         return "\n".join(buf)
 
-    def visit_GeneratorExp(self, node):
+    def visit_GeneratorExp(self, node) -> str:
         elt = self.visit(node.elt)
         generator = node.generators[0]
         target = self.visit(generator.target)
@@ -565,30 +511,30 @@ class DartTranspiler(CLikeTranspiler):
         if not iter.endswith("keys()") or iter.endswith("values()"):
             iter += ".iter()"
 
-        map_str = ".map(|{0}| {1})".format(target, elt)
+        map_str = f".map(|{target}| {elt})"
         filter_str = ""
         if generator.ifs:
-            filter_str = ".cloned().filter(|&{0}| {1})".format(
+            filter_str = ".cloned().filter(|&{}| {})".format(
                 target, self.visit(generator.ifs[0])
             )
 
-        return "{0}{1}{2}.collect::<Vec<_>>()".format(iter, filter_str, map_str)
+        return f"{iter}{filter_str}{map_str}.collect::<Vec<_>>()"
 
-    def visit_ListComp(self, node):
+    def visit_ListComp(self, node) -> str:
         return self.visit_GeneratorExp(node)  # right now they are the same
 
-    def visit_Global(self, node):
-        return "//global {0}".format(", ".join(node.names))
+    def visit_Global(self, node) -> str:
+        return "//global {}".format(", ".join(node.names))
 
-    def visit_Starred(self, node):
-        return "starred!({0})/*unsupported*/".format(self.visit(node.value))
+    def visit_Starred(self, node) -> str:
+        return f"starred!({self.visit(node.value)})/*unsupported*/"
 
-    def visit_Set(self, node):
+    def visit_Set(self, node) -> str:
         elements = [self.visit(e) for e in node.elts]
         elements_str = ", ".join(elements)
         return f"new Set.from([{elements_str}])"
 
-    def visit_IfExp(self, node):
+    def visit_IfExp(self, node) -> str:
         body = self.visit(node.body)
         orelse = self.visit(node.orelse)
         test = self.visit(node.test)
