@@ -1,27 +1,26 @@
 import ast
 import textwrap
-
 from typing import List
 
+from py2many.analysis import IGNORED_MODULE_SET, get_id, is_global, is_void_function
+from py2many.clike import _AUTO_INVOKED, class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.exceptions import AstClassUsedBeforeDeclaration, AstCouldNotInfer
+from py2many.rewriters import camel_case, capitalize_first, rename
+from py2many.tracer import defined_before, is_class_or_module, is_enum, is_list
+
 from .clike import CLikeTranspiler
 from .inference import get_inferred_go_type
 from .plugins import (
     ATTR_DISPATCH_TABLE,
     CLASS_DISPATCH_TABLE,
+    DISPATCH_MAP,
     FUNC_DISPATCH_TABLE,
     MODULE_DISPATCH_TABLE,
-    DISPATCH_MAP,
     SMALL_DISPATCH_MAP,
     SMALL_USINGS_MAP,
 )
 
-from py2many.analysis import IGNORED_MODULE_SET, get_id, is_global, is_void_function
-from py2many.clike import _AUTO_INVOKED, class_for_typename
-from py2many.declaration_extractor import DeclarationExtractor
-from py2many.exceptions import AstClassUsedBeforeDeclaration, AstCouldNotInfer
-from py2many.rewriters import capitalize_first, rename, camel_case
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_enum
-
 
 class GoMethodCallRewriter(ast.NodeTransformer):
     def visit_Call(self, node):
@@ -131,12 +130,9 @@ class GoIfExpRewriter(ast.NodeTransformer):
 class GoTranspiler(CLikeTranspiler):
     NAME = "go"
 
-    CONTAINER_TYPE_MAP = {"List": "[]", "Dict": None, "Set": None, "Optional": "nil"}
-
     def __init__(self):
         super().__init__()
-        self._default_type = None
-        self._container_type_map = self.CONTAINER_TYPE_MAP
+        CLikeTranspiler._default_type = None
         self._dispatch_map = DISPATCH_MAP
         self._small_dispatch_map = SMALL_DISPATCH_MAP
         self._small_usings_map = SMALL_USINGS_MAP
@@ -154,13 +150,14 @@ class GoTranspiler(CLikeTranspiler):
             buf += ")\n"
         return buf + "\n\n"
 
-    def _combine_value_index(self, value_type, index_type) -> str:
+    @classmethod
+    def _combine_value_index(cls, value_type, index_type) -> str:
         return f"{value_type}{index_type}"
 
     def comment(self, text):
         return f"// {text}\n"
 
-    def visit_FunctionDef(self, node):
+    def visit_FunctionDef(self, node) -> str:
         body = "\n".join([self.visit(n) for n in node.body])
         typenames, args = self.visit(node.args)
 
@@ -175,7 +172,7 @@ class GoTranspiler(CLikeTranspiler):
             arg = args[i]
 
             if typename == "T":
-                typename = "T{0} any".format(index)
+                typename = f"T{index} any"
                 typedecls.append(typename)
                 index += 1
             args_list.append(f"{arg} {typename}")
@@ -193,13 +190,13 @@ class GoTranspiler(CLikeTranspiler):
 
         template = ""
         if len(typedecls) > 0:
-            template = "[{0}]".format(", ".join(typedecls))
+            template = "[{}]".format(", ".join(typedecls))
 
         args = ", ".join(args_list)
         funcdef = f"func {node.name}{template}({args}){return_type} {{"
         return f"{funcdef}\n{body}}}\n\n"
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         if node.value:
             ret = self.visit(node.value)
             fndef = None
@@ -228,7 +225,7 @@ class GoTranspiler(CLikeTranspiler):
             typename = self._typename_from_annotation(node)
         return (typename, id)
 
-    def visit_Lambda(self, node):
+    def visit_Lambda(self, node) -> str:
         typenames, args = self.visit(node.args)
         # HACK: to pass unit tests. TODO: infer types
         # Need to get it from the annotation on the lhs of `node`
@@ -239,7 +236,7 @@ class GoTranspiler(CLikeTranspiler):
         body = self.visit(node.body)
         return f"func({args_string}) {return_type} {{ return {body} }}"
 
-    def visit_Attribute(self, node):
+    def visit_Attribute(self, node) -> str:
         attr = node.attr
 
         value_id = self.visit(node.value)
@@ -260,7 +257,7 @@ class GoTranspiler(CLikeTranspiler):
 
         return f"{value_id}.{attr}"
 
-    def _visit_struct_literal(self, node, fname: str, fndef: ast.ClassDef):
+    def _visit_struct_literal(self, node, fname: str, fndef: ast.ClassDef) -> str:
         vargs = []  # visited args
         if not hasattr(fndef, "declarations"):
             raise AstClassUsedBeforeDeclaration(fndef, node)
@@ -275,7 +272,7 @@ class GoTranspiler(CLikeTranspiler):
         args = ", ".join(vargs)
         return f"{fname}{{{args}}}"
 
-    def visit_Call(self, node):
+    def visit_Call(self, node) -> str:
         fname = self.visit(node.func)
         fndef = node.scopes.find(fname)
 
@@ -298,7 +295,7 @@ class GoTranspiler(CLikeTranspiler):
             args = ""
         return f"{fname}({args})"
 
-    def visit_For(self, node):
+    def visit_For(self, node) -> str:
         target = self.visit(node.target)
         it = self.visit(node.iter)
         buf = []
@@ -313,21 +310,21 @@ class GoTranspiler(CLikeTranspiler):
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_While(self, node):
+    def visit_While(self, node) -> str:
         buf = []
-        buf.append("for {0} {{".format(self.visit(node.test)))
+        buf.append(f"for {self.visit(node.test)} {{")
         buf.extend([self.visit(c) for c in node.body])
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         return "" + super().visit_Str(node) + ""
 
-    def visit_Bytes(self, node):
-        bytes_str = "{0}".format(node.s)
+    def visit_Bytes(self, node) -> str:
+        bytes_str = f"{node.s}"
         return bytes_str.replace("'", '"')  # replace single quote with double quote
 
-    def _visit_container_compare(self, node):
+    def _visit_container_compare(self, node) -> str:
         left = self.visit(node.left)
         op = self.visit(node.ops[0])
         right = self.visit(node.comparators[0])
@@ -335,7 +332,7 @@ class GoTranspiler(CLikeTranspiler):
             return f"cmp.Equal({left}, {right})"
         return super().visit_Compare(node)
 
-    def visit_Compare(self, node):
+    def visit_Compare(self, node) -> str:
         left = node.left
         right = node.comparators[0]
         if hasattr(left, "annotation") or hasattr(right, "annotation"):
@@ -353,7 +350,7 @@ class GoTranspiler(CLikeTranspiler):
         right = self.visit(node.comparators[0])
         return super().visit_Compare(node)
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id == "None":
             return "None"
         elif node.id == "StringsContains":
@@ -363,7 +360,7 @@ class GoTranspiler(CLikeTranspiler):
         else:
             return super().visit_Name(node)
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is None:
             return "nil"
         else:
@@ -376,7 +373,7 @@ class GoTranspiler(CLikeTranspiler):
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_If(self, node):
+    def visit_If(self, node) -> str:
         body_vars = {get_id(v): v for v in node.scopes[-1].body_vars}
         orelse_vars = {get_id(v): v for v in node.scopes[-1].orelse_vars}
         node.common_vars = set(body_vars.keys()).intersection(set(orelse_vars.keys()))
@@ -389,29 +386,17 @@ class GoTranspiler(CLikeTranspiler):
         else:
             return super().visit_If(node)
 
-    def visit_UnaryOp(self, node):
+    def visit_UnaryOp(self, node) -> str:
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_BinOp(self, node):
-        if (
-            isinstance(node.left, ast.List)
-            and isinstance(node.op, ast.Mult)
-            and isinstance(node.right, ast.Num)
-        ):
-            return "std::vector ({0},{1})".format(
-                self.visit(node.right), self.visit(node.left.elts[0])
-            )
-        else:
-            return super().visit_BinOp(node)
-
-    def visit_ClassDef(self, node):
+    def visit_ClassDef(self, node) -> str:
         extractor = DeclarationExtractor(GoTranspiler())
         extractor.visit(node)
         declarations = node.declarations = extractor.get_declarations()
@@ -434,7 +419,7 @@ class GoTranspiler(CLikeTranspiler):
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
-                typename = "ST{0}".format(index)
+                typename = f"ST{index}"
                 index += 1
             fields.append(f"{declaration} {typename}")
 
@@ -447,7 +432,7 @@ class GoTranspiler(CLikeTranspiler):
         body = "\n".join(body)
         return f"type {node.name} struct {{\n{fields}\n}}\n{body}\n"
 
-    def _visit_enum(self, node, typename, members):
+    def _visit_enum(self, node, typename, members) -> str:
         ret = f"type {node.name} {typename}\n\n"
         fields = []
         for i, (member, var) in enumerate(members):
@@ -459,7 +444,7 @@ class GoTranspiler(CLikeTranspiler):
         fields = "\n".join(fields)
         return f"{ret} const (\n{fields}\n)\n\n"
 
-    def visit_IntEnum(self, node):
+    def visit_IntEnum(self, node) -> str:
         members = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -469,7 +454,7 @@ class GoTranspiler(CLikeTranspiler):
                 members.append((member, var))
         return self._visit_enum(node, "int", members)
 
-    def visit_IntFlag(self, node):
+    def visit_IntFlag(self, node) -> str:
         members = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -479,7 +464,7 @@ class GoTranspiler(CLikeTranspiler):
                 members.append((member, var))
         return self._visit_enum(node, "int", members)
 
-    def visit_StrEnum(self, node):
+    def visit_StrEnum(self, node) -> str:
         members = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -489,7 +474,7 @@ class GoTranspiler(CLikeTranspiler):
     def _import(self, name: str) -> str:
         return f'import ("{name}")'
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         if len(names) == 1:
             # TODO: make this more generic so it works for len(names) > 1
             name = names[0]
@@ -501,7 +486,7 @@ class GoTranspiler(CLikeTranspiler):
         module_name = module_name.replace(".", "/")
         return "\n".join([f'import ("{module_name}/{name}")' for name in names])
 
-    def visit_List(self, node):
+    def visit_List(self, node) -> str:
         _ = self._typename_from_annotation(node)
         element_type = self._default_type
         if hasattr(node, "container_type"):
@@ -510,7 +495,7 @@ class GoTranspiler(CLikeTranspiler):
         elements_str = ", ".join(elements)
         return f"[]{element_type}{{{elements_str}}}"
 
-    def visit_Set(self, node):
+    def visit_Set(self, node) -> str:
         _ = self._typename_from_annotation(node)
         element_type = self._default_type
         if hasattr(node, "container_type"):
@@ -519,7 +504,7 @@ class GoTranspiler(CLikeTranspiler):
         kv_pairs = ", ".join([f"{k}: true" for k in elements])
         return f"map[{element_type}]bool{{{kv_pairs}}}"
 
-    def visit_Dict(self, node):
+    def visit_Dict(self, node) -> str:
         keys = [self.visit(k) for k in node.keys]
         values = [self.visit(k) for k in node.values]
         kv_pairs = ", ".join([f"{k}: {v}" for k, v in zip(keys, values)])
@@ -532,21 +517,21 @@ class GoTranspiler(CLikeTranspiler):
                 key_typename = "int"
         return f"map[{key_typename}]{value_typename}{{{kv_pairs}}}"
 
-    def visit_Subscript(self, node):
+    def visit_Subscript(self, node) -> str:
         value = self.visit(node.value)
         index = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
             if value in self.CONTAINER_TYPE_MAP:
                 value = self.CONTAINER_TYPE_MAP[value]
             if value == "Tuple":
-                return "({0})".format(index)
-            return "{0}{1}".format(value, index)
-        return "{0}[{1}]".format(value, index)
+                return f"({index})"
+            return f"{value}{index}"
+        return f"{value}[{index}]"
 
-    def visit_Index(self, node):
+    def visit_Index(self, node) -> str:
         return self.visit(node.value)
 
-    def visit_Slice(self, node):
+    def visit_Slice(self, node) -> str:
         lower = ""
         if node.lower:
             lower = self.visit(node.lower)
@@ -554,40 +539,20 @@ class GoTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Tuple(self, node):
+    def visit_Tuple(self, node) -> str:
         elts = [self.visit(e) for e in node.elts]
         elts = ", ".join(elts)
         if hasattr(node, "is_annotation"):
             return elts
-        return "{0}".format(elts)
+        return f"{elts}"
 
-    def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
-
-        for handler in node.handlers:
-            buf += self.visit(handler)
-        # buf.append("\n".join(excepts));
-
-        if finallybody:
-            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
-
-        return "\n".join(buf)
-
-    def visit_ExceptHandler(self, node):
-        exception_type = ""
-        if node.type:
-            exception_type = self.visit(node.type)
-        name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(node, name, node.body)
-        return body
-
-    def visit_Assert(self, node):
+    def visit_Assert(self, node) -> str:
         condition = self.visit(node.test)
         return f'if !({condition}) {{ panic("assert") }}'
 
-    def visit_AnnAssign(self, node):
+    def visit_AnnAssign(self, node) -> str:
         target = self.visit(node.target)
         type_str = self._typename_from_annotation(node)
         val = self.visit(node.value) if node.value is not None else None
@@ -606,30 +571,30 @@ class GoTranspiler(CLikeTranspiler):
         )
 
     def _assign_cast(
-        self, value_str: str, cast_to: str, python_annotation, rust_annotation
+        self, value_str: str, cast_to: str, python_annotation, go_annotation
     ) -> str:
-        # python/rust annotations provided to customize the cast if necessary
+        # python/go annotations provided to customize the cast if necessary
         return f"{cast_to}({value_str})"
 
-    def _visit_AssignOne(self, node, target):
+    def _visit_AssignOne(self, node, target) -> str:
         if isinstance(target, ast.Tuple):
             elts = [self.visit(e) for e in target.elts]
             value = self.visit(node.value)
-            return "var {0} = {1}".format(", ".join(elts), value)
+            return "var {} = {}".format(", ".join(elts), value)
 
         if isinstance(node.scopes[-1], ast.If):
             outer_if = node.scopes[-1]
             target_id = self.visit(target)
             if target_id in outer_if.common_vars:
                 value = self.visit(node.value)
-                return "{0} = {1}".format(target_id, value)
+                return f"{target_id} = {value}"
 
         if isinstance(target, ast.Subscript) or isinstance(target, ast.Attribute):
             target = self.visit(target)
             value = self.visit(node.value)
             if value == None:
                 value = "None"
-            return "{0} = {1}".format(target, value)
+            return f"{target} = {value}"
 
         typename = self._typename_from_annotation(target)
         needs_cast = self._needs_cast(target, node.value)
@@ -663,58 +628,10 @@ class GoTranspiler(CLikeTranspiler):
                 return f"var {target_str} = {value}"
             return f"{target_str} := {value}"
 
-    def visit_Delete(self, node):
-        target = node.targets[0]
-        return "{0}.drop()".format(self.visit(target))
-
-    def visit_Raise(self, node):
-        if node.exc is not None:
-            return "raise!({0}); //unsupported".format(self.visit(node.exc))
-        # This handles the case where `raise` is used without
-        # specifying the exception.
-        return "raise!(); //unsupported"
-
-    def visit_Await(self, node):
-        return "await!({0})".format(self.visit(node.value))
-
-    def visit_AsyncFunctionDef(self, node):
-        return "#[async]\n{0}".format(self.visit_FunctionDef(node))
-
-    def visit_Yield(self, node):
-        return "//yield is unimplemented"
-
-    def visit_Print(self, node):
+    def visit_Print(self, node) -> str:
         buf = []
         self._usings.add('"fmt"')
         for n in node.values:
             value = self.visit(n)
-            buf.append('fmt.Printf("%v\n",{0})'.format(value))
+            buf.append(f'fmt.Printf("%v\n",{value})')
         return "\n".join(buf)
-
-    def visit_GeneratorExp(self, node):
-        elt = self.visit(node.elt)
-        generator = node.generators[0]
-        target = self.visit(generator.target)
-        iter = self.visit(generator.iter)
-
-        # HACK for dictionary iterators to work
-        if not iter.endswith("keys()") or iter.endswith("values()"):
-            iter += ".iter()"
-
-        map_str = ".map(|{0}| {1})".format(target, elt)
-        filter_str = ""
-        if generator.ifs:
-            filter_str = ".cloned().filter(|&{0}| {1})".format(
-                target, self.visit(generator.ifs[0])
-            )
-
-        return "{0}{1}{2}.collect::<Vec<_>>()".format(iter, filter_str, map_str)
-
-    def visit_ListComp(self, node):
-        return self.visit_GeneratorExp(node)  # right now they are the same
-
-    def visit_Global(self, node):
-        return "//global {0}".format(", ".join(node.names))
-
-    def visit_Starred(self, node):
-        return "starred!({0})/*unsupported*/".format(self.visit(node.value))
