@@ -1,11 +1,17 @@
 import ast
-
-from ctypes import c_int8, c_int16, c_int32, c_int64
-from ctypes import c_uint8, c_uint16, c_uint32, c_uint64
+from ctypes import (
+    c_int8,
+    c_int16,
+    c_int32,
+    c_int64,
+    c_uint8,
+    c_uint16,
+    c_uint32,
+    c_uint64,
+)
 
 from py2many.clike import CLikeTranspiler as CommonCLikeTranspiler
 
-
 julia_type_map = {
     bool: "Bool",
     int: "Int64",
@@ -22,6 +28,13 @@ julia_type_map = {
     c_uint64: "UInt64",
 }
 
+JULIA_CONTAINER_TYPE_MAP = {
+    "List": "Array",
+    "Dict": "Dict",
+    "Set": "Set",
+    "Optional": "Nothing",
+}
+
 # allowed as names in Python but treated as keywords in Julia
 julia_keywords = frozenset(
     [
@@ -61,48 +74,50 @@ julia_keywords = frozenset(
     ]
 )
 
-jl_symbols = {ast.BitXor: "⊻"}
+JL_SYMBOLS = {
+    ast.BitXor: "⊻",
+    ast.And: "&&",
+    ast.Or: "||",
+    ast.Is: "===",
+    ast.IsNot: "!==",
+}
 
 
 def jl_symbol(node):
     """Find the equivalent Julia symbol for a Python ast symbol node"""
     symbol_type = type(node)
-    return jl_symbols[symbol_type]
+    return JL_SYMBOLS[symbol_type]
 
 
 class CLikeTranspiler(CommonCLikeTranspiler):
     def __init__(self):
         super().__init__()
-        self._type_map = julia_type_map
+        CommonCLikeTranspiler._type_map = julia_type_map
 
-    def visit(self, node):
-        if type(node) in jl_symbols:
+    def visit(self, node) -> str:
+        if type(node) in JL_SYMBOLS:
             return jl_symbol(node)
         else:
             return super().visit(node)
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id in julia_keywords:
             return node.id + "_"
         return super().visit_Name(node)
 
-    def visit_BinOp(self, node):
+    def visit_BinOp(self, node) -> str:
+        if isinstance(node.op, ast.Mult):
+            return f"{self.visit(node.left)}*{self.visit(node.right)}"
         if isinstance(node.op, ast.Pow):
-            return "pow({0}, {1})".format(self.visit(node.left), self.visit(node.right))
+            return f"{self.visit(node.left)}^{self.visit(node.right)}"
 
-        # Multiplication and division binds tighter (has higher precedence) than addition and subtraction.
-        # To visually communicate this we omit spaces when multiplying and dividing.
-        if isinstance(node.op, (ast.Mult, ast.Div)):
-            return "({0}{1}{2})".format(
-                self.visit(node.left), self.visit(node.op), self.visit(node.right)
-            )
+        bin_op = (
+            f"{self.visit(node.left)} {self.visit(node.op)} {self.visit(node.right)}"
+        )
+        is_nested = getattr(node, "isnested", None)
+        return bin_op if not is_nested else f"({bin_op})"
 
-        else:
-            return "({0} {1} {2})".format(
-                self.visit(node.left), self.visit(node.op), self.visit(node.right)
-            )
-
-    def visit_In(self, node):
+    def visit_In(self, node) -> str:
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
         return f"{left} in {right}"
