@@ -1,24 +1,23 @@
 import ast
 import textwrap
+from typing import List, Tuple
 
-from .clike import CLikeTranspiler
+from py2many.analysis import get_id, is_void_function
+from py2many.clike import _AUTO_INVOKED, class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.tracer import defined_before, is_class_or_module, is_enum, is_list
+
+from .clike import JULIA_CONTAINER_TYPE_MAP, CLikeTranspiler, julia_type_map
 from .plugins import (
     ATTR_DISPATCH_TABLE,
     CLASS_DISPATCH_TABLE,
+    DISPATCH_MAP,
     FUNC_DISPATCH_TABLE,
     MODULE_DISPATCH_TABLE,
-    DISPATCH_MAP,
     SMALL_DISPATCH_MAP,
     SMALL_USINGS_MAP,
 )
 
-from py2many.analysis import get_id, is_void_function
-from py2many.declaration_extractor import DeclarationExtractor
-from py2many.clike import _AUTO_INVOKED, class_for_typename
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_enum
-
-from typing import List, Tuple
-
 
 class JuliaMethodCallRewriter(ast.NodeTransformer):
     def visit_Call(self, node):
@@ -40,18 +39,12 @@ class JuliaMethodCallRewriter(ast.NodeTransformer):
 class JuliaTranspiler(CLikeTranspiler):
     NAME = "julia"
 
-    CONTAINER_TYPE_MAP = {
-        "List": "Array",
-        "Dict": "Dict",
-        "Set": "Set",
-        "Optional": "Nothing",
-    }
-
     def __init__(self):
         super().__init__()
-        self._headers = set([])
-        self._default_type = ""
-        self._container_type_map = self.CONTAINER_TYPE_MAP
+        self._headers = set()
+        CLikeTranspiler._default_type = ""
+        CLikeTranspiler._type_map = julia_type_map
+        CLikeTranspiler._container_type_map = JULIA_CONTAINER_TYPE_MAP
         self._dispatch_map = DISPATCH_MAP
         self._small_dispatch_map = SMALL_DISPATCH_MAP
         self._small_usings_map = SMALL_USINGS_MAP
@@ -66,10 +59,11 @@ class JuliaTranspiler(CLikeTranspiler):
     def comment(self, text):
         return f"# {text}"
 
-    def _combine_value_index(self, value_type, index_type) -> str:
+    @classmethod
+    def _combine_value_index(cls, value_type, index_type) -> str:
         return f"{value_type}{{{index_type}}}"
 
-    def visit_Constant(self, node):
+    def visit_Constant(self, node) -> str:
         if node.value is True:
             return "true"
         elif node.value is False:
@@ -84,7 +78,7 @@ class JuliaTranspiler(CLikeTranspiler):
         else:
             return super().visit_Constant(node)
 
-    def visit_FunctionDef(self, node):
+    def visit_FunctionDef(self, node) -> str:
         body = "\n".join([self.visit(n) for n in node.body])
         typenames, args = self.visit(node.args)
 
@@ -101,10 +95,10 @@ class JuliaTranspiler(CLikeTranspiler):
             typename = typenames[i]
             arg = args[i]
             if typename == "T":
-                typename = "T{0}".format(index)
+                typename = f"T{index}"
                 typedecls.append(typename)
                 index += 1
-            args_list.append("{0}::{1}".format(arg, typename))
+            args_list.append(f"{arg}::{typename}")
 
         return_type = ""
         if not is_void_function(node):
@@ -126,9 +120,9 @@ class JuliaTranspiler(CLikeTranspiler):
             maybe_main = "\nmain()"
         return f"{funcdef}\n{body}\nend\n{maybe_main}"
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         if node.value:
-            return "return {0}".format(self.visit(node.value))
+            return f"return {self.visit(node.value)}"
         return "return"
 
     def visit_arg(self, node):
@@ -140,13 +134,13 @@ class JuliaTranspiler(CLikeTranspiler):
             typename = self._typename_from_annotation(node)
         return (typename, id)
 
-    def visit_Lambda(self, node):
+    def visit_Lambda(self, node) -> str:
         _, args = self.visit(node.args)
         args_string = ", ".join(args)
         body = self.visit(node.body)
-        return "({0}) -> {1}".format(args_string, body)
+        return f"({args_string}) -> {body}"
 
-    def visit_Attribute(self, node):
+    def visit_Attribute(self, node) -> str:
         attr = node.attr
 
         value_id = self.visit(node.value)
@@ -175,14 +169,14 @@ class JuliaTranspiler(CLikeTranspiler):
             return f"({vargs[0]}:{vargs[2]}:{vargs[1]}-1)"
 
         raise Exception(
-            "encountered range() call with unknown parameters: range({})".format(vargs)
+            f"encountered range() call with unknown parameters: range({vargs})"
         )
 
     def _visit_print(self, node, vargs: List[str]) -> str:
         args = ", ".join(vargs)
         return f'println(join([{args}], " "))'
 
-    def visit_Call(self, node):
+    def visit_Call(self, node) -> str:
         fname = self.visit(node.func)
         fndef = node.scopes.find(fname)
         vargs = []
@@ -211,24 +205,24 @@ class JuliaTranspiler(CLikeTranspiler):
         args = ", ".join(converted)
         return f"{fname}({args})"
 
-    def visit_For(self, node):
+    def visit_For(self, node) -> str:
         target = self.visit(node.target)
         it = self.visit(node.iter)
         buf = []
-        buf.append("for {0} in {1}".format(target, it))
+        buf.append(f"for {target} in {it}")
         buf.extend([self.visit(c) for c in node.body])
         buf.append("end")
         return "\n".join(buf)
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         return "" + super().visit_Str(node) + ""
 
-    def visit_Bytes(self, node):
+    def visit_Bytes(self, node) -> str:
         bytes_str = node.s
         bytes_str = bytes_str.replace(b'"', b'\\"')
         return 'b"' + bytes_str.decode("ascii", "backslashreplace") + '"'
 
-    def visit_Compare(self, node):
+    def visit_Compare(self, node) -> str:
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
 
@@ -241,19 +235,19 @@ class JuliaTranspiler(CLikeTranspiler):
                 right = f"keys({right})"
 
         if isinstance(node.ops[0], ast.In):
-            return "{0} in {1}".format(left, right)
+            return f"{left} in {right}"
         elif isinstance(node.ops[0], ast.NotIn):
-            return "{0} not in {1}".format(left, right)
+            return f"{left} not in {right}"
 
         return super().visit_Compare(node)
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id == "None":
             return "None"
         else:
             return super().visit_Name(node)
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is True:
             return "true"
         elif node.value is False:
@@ -263,9 +257,9 @@ class JuliaTranspiler(CLikeTranspiler):
         else:
             return super().visit_NameConstant(node)
 
-    def visit_If(self, node):
-        body_vars = set([get_id(v) for v in node.scopes[-1].body_vars])
-        orelse_vars = set([get_id(v) for v in node.scopes[-1].orelse_vars])
+    def visit_If(self, node) -> str:
+        body_vars = {get_id(v) for v in node.scopes[-1].body_vars}
+        orelse_vars = {get_id(v) for v in node.scopes[-1].orelse_vars}
         node.common_vars = body_vars.intersection(orelse_vars)
 
         buf = []
@@ -283,36 +277,38 @@ class JuliaTranspiler(CLikeTranspiler):
 
         return "\n".join(buf)
 
-    def visit_While(self, node):
+    def visit_While(self, node) -> str:
         buf = []
-        buf.append("while {0}".format(self.visit(node.test)))
+        buf.append(f"while {self.visit(node.test)}")
         buf.extend([self.visit(n) for n in node.body])
         buf.append("end")
         return "\n".join(buf)
 
-    def visit_UnaryOp(self, node):
+    def visit_UnaryOp(self, node) -> str:
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_BinOp(self, node):
+    def visit_BinOp(self, node) -> str:
         if (
             isinstance(node.left, ast.List)
             and isinstance(node.op, ast.Mult)
             and isinstance(node.right, ast.Num)
         ):
-            return "std::vector ({0},{1})".format(
+            return "std::vector ({},{})".format(
                 self.visit(node.right), self.visit(node.left.elts[0])
             )
+        elif isinstance(node.op, ast.MatMult):
+            return f"({self.visit(node.left)}*{self.visit(node.right)})"
         else:
             return super().visit_BinOp(node)
 
-    def visit_ClassDef(self, node):
+    def visit_ClassDef(self, node) -> str:
         extractor = DeclarationExtractor(JuliaTranspiler())
         extractor.visit(node)
         declarations = node.declarations = extractor.get_declarations()
@@ -335,7 +331,7 @@ class JuliaTranspiler(CLikeTranspiler):
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
-                typename = "ST{0}".format(index)
+                typename = f"ST{index}"
                 index += 1
             fields.append(f"{declaration}::{typename}")
 
@@ -347,7 +343,7 @@ class JuliaTranspiler(CLikeTranspiler):
         body = "\n".join([self.visit(b) for b in node.body])
         return f"{struct_def}\n{body}"
 
-    def _visit_enum(self, node, typename: str, fields: List[Tuple]):
+    def _visit_enum(self, node, typename: str, fields: List[Tuple]) -> str:
         self._usings.add("SuperEnum")
         fields_list = []
 
@@ -366,7 +362,7 @@ class JuliaTranspiler(CLikeTranspiler):
             """
         )
 
-    def visit_StrEnum(self, node):
+    def visit_StrEnum(self, node) -> str:
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -375,7 +371,7 @@ class JuliaTranspiler(CLikeTranspiler):
             fields.append((member, var))
         return self._visit_enum(node, "String", fields)
 
-    def visit_IntEnum(self, node):
+    def visit_IntEnum(self, node) -> str:
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -384,7 +380,7 @@ class JuliaTranspiler(CLikeTranspiler):
             fields.append((member, var))
         return self._visit_enum(node, "Int64", fields)
 
-    def visit_IntFlag(self, node):
+    def visit_IntFlag(self, node) -> str:
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -396,7 +392,7 @@ class JuliaTranspiler(CLikeTranspiler):
     def _import(self, name: str) -> str:
         return f"import {name}"
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         if len(names) == 1:
             # TODO: make this more generic so it works for len(names) > 1
             name = names[0]
@@ -409,45 +405,45 @@ class JuliaTranspiler(CLikeTranspiler):
         names = ", ".join(names)
         return f"using {module_name}: {names}"
 
-    def visit_List(self, node):
+    def visit_List(self, node) -> str:
         elements = [self.visit(e) for e in node.elts]
         elements_str = ", ".join(elements)
         return f"[{elements_str}]"
 
-    def visit_Set(self, node):
+    def visit_Set(self, node) -> str:
         elements = [self.visit(e) for e in node.elts]
         elements_str = ", ".join(elements)
         return f"Set([{elements_str}])"
 
-    def visit_Dict(self, node):
+    def visit_Dict(self, node) -> str:
         keys = [self.visit(k) for k in node.keys]
         values = [self.visit(k) for k in node.values]
         kv_pairs = ", ".join([f"{k} => {v}" for k, v in zip(keys, values)])
         return f"Dict({kv_pairs})"
 
-    def visit_Subscript(self, node):
+    def visit_Subscript(self, node) -> str:
         value = self.visit(node.value)
         index = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
             if value in self.CONTAINER_TYPE_MAP:
                 value = self.CONTAINER_TYPE_MAP[value]
             if value == "Tuple":
-                return "({0})".format(index)
-            return "{0}{{{1}}}".format(value, index)
+                return f"({index})"
+            return f"{value}{{{index}}}"
         # TODO: optimize this. We need to compute value_type once per definition
         self._generic_typename_from_annotation(node.value)
         if hasattr(node.value, "annotation"):
             value_type = getattr(node.value.annotation, "generic_container_type", None)
             if value_type is not None and value_type[0] == "List":
                 # Julia array indices start at 1
-                return "{0}[{1} + 1]".format(value, index)
+                return f"{value}[{index} + 1]"
 
-        return "{0}[{1}]".format(value, index)
+        return f"{value}[{index}]"
 
-    def visit_Index(self, node):
+    def visit_Index(self, node) -> str:
         return self.visit(node.value)
 
-    def visit_Slice(self, node):
+    def visit_Slice(self, node) -> str:
         lower = ""
         if node.lower:
             lower = self.visit(node.lower)
@@ -455,69 +451,79 @@ class JuliaTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Tuple(self, node):
+    def visit_Tuple(self, node) -> str:
         elts = [self.visit(e) for e in node.elts]
         elts = ", ".join(elts)
         if hasattr(node, "is_annotation"):
             return elts
-        return "({0})".format(elts)
-
-    def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
-
-        for handler in node.handlers:
-            buf += self.visit(handler)
-        # buf.append("\n".join(excepts));
-
-        if finallybody:
-            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
+        return f"({elts})"
 
+    def visit_Try(self, node, finallybody=None) -> str:
+        buf = []
+        buf.append("try")
+        buf.extend([self.visit(child) for child in node.body])
+        if len(node.handlers) > 0:
+            buf.append("catch exn")
+            for handler in node.handlers:
+                buf.append(self.visit(handler))
+        if node.finalbody:
+            buf.append("finally")
+            buf.extend([self.visit(child) for child in node.finalbody])
+        buf.append("end")
         return "\n".join(buf)
 
-    def visit_ExceptHandler(self, node):
-        exception_type = ""
+    def visit_ExceptHandler(self, node) -> str:
+        buf = []
+        name = "exn"
+        if node.name:
+            buf.append(f" let {node.name} = {name}")
+            name = node.name
         if node.type:
-            exception_type = self.visit(node.type)
-        name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(node, name, node.body)
-        return body
+            type_str = self.visit(node.type)
+            buf.append(f"if {name} isa {type_str}")
+        buf.extend([self.visit(child) for child in node.body])
+        if node.type:
+            buf.append("end")
+        if node.name:
+            buf.append("end")
+        return "\n".join(buf)
 
-    def visit_Assert(self, node):
-        return "@assert({0})".format(self.visit(node.test))
+    def visit_Assert(self, node) -> str:
+        return f"@assert({self.visit(node.test)})"
 
-    def visit_AnnAssign(self, node):
+    def visit_AnnAssign(self, node) -> str:
         target, type_str, val = super().visit_AnnAssign(node)
         if type_str == self._default_type:
             return f"{target} = {val}"
         return f"{target}::{type_str} = {val}"
 
-    def visit_AugAssign(self, node):
+    def visit_AugAssign(self, node) -> str:
         target = self.visit(node.target)
         op = self.visit(node.op)
         val = self.visit(node.value)
-        return "{0} {1}= {2}".format(target, op, val)
+        return f"{target} {op}= {val}"
 
-    def _visit_AssignOne(self, node, target):
+    def _visit_AssignOne(self, node, target) -> str:
         if isinstance(target, ast.Tuple):
             elts = [self.visit(e) for e in target.elts]
             value = self.visit(node.value)
-            return "{0} = {1}".format(", ".join(elts), value)
+            return "{} = {}".format(", ".join(elts), value)
 
         if isinstance(node.scopes[-1], ast.If):
             outer_if = node.scopes[-1]
             target_id = self.visit(target)
             if target_id in outer_if.common_vars:
                 value = self.visit(node.value)
-                return "{0} = {1}".format(target_id, value)
+                return f"{target_id} = {value}"
 
         if isinstance(target, ast.Subscript) or isinstance(target, ast.Attribute):
             target = self.visit(target)
             value = self.visit(node.value)
             if value == None:
                 value = "None"
-            return "{0} = {1}".format(target, value)
+            return f"{target} = {value}"
 
         definition = node.scopes.parent_scopes.find(get_id(target))
         if definition is None:
@@ -531,62 +537,64 @@ class JuliaTranspiler(CLikeTranspiler):
             value = self.visit(node.value)
             return f"{target} = {value}"
 
-    def visit_Delete(self, node):
-        target = node.targets[0]
-        return "{0}.drop()".format(self.visit(target))
-
-    def visit_Raise(self, node):
+    def visit_Raise(self, node) -> str:
         if node.exc is not None:
-            return "raise!({0}) # unsupported".format(self.visit(node.exc))
+            return f"throw({self.visit(node.exc)})"
         # This handles the case where `raise` is used without
         # specifying the exception.
-        return "raise!() # unsupported"
+        return "error()"
 
-    def visit_Await(self, node):
-        return "await!({0})".format(self.visit(node.value))
-
-    def visit_AsyncFunctionDef(self, node):
-        return "#[async]\n{0}".format(self.visit_FunctionDef(node))
-
-    def visit_Yield(self, node):
-        return "//yield is unimplemented"
-
-    def visit_Print(self, node):
+    def visit_Print(self, node) -> str:
         buf = []
         for n in node.values:
             value = self.visit(n)
-            buf.append('println("{{:?}}",{0})'.format(value))
+            buf.append(f'println("{{:?}}",{value})')
         return "\n".join(buf)
 
-    def visit_GeneratorExp(self, node):
+    def visit_GeneratorExp(self, node) -> str:
         elt = self.visit(node.elt)
-        generator = node.generators[0]
-        target = self.visit(generator.target)
-        iter = self.visit(generator.iter)
+        generators = node.generators
+        gen_expr = self._visit_generators(generators)
+        return f"({elt} {gen_expr})"
 
-        # HACK for dictionary iterators to work
-        if not iter.endswith("keys()") or iter.endswith("values()"):
-            iter += ".iter()"
+    def visit_ListComp(self, node) -> str:
+        elt = self.visit(node.elt)
+        generators = node.generators
+        list_comp = self._visit_generators(generators)
+        return f"[{elt} {list_comp}]"
 
-        map_str = ".map(|{0}| {1})".format(target, elt)
-        filter_str = ""
-        if generator.ifs:
-            filter_str = ".cloned().filter(|&{0}| {1})".format(
-                target, self.visit(generator.ifs[0])
-            )
+    def visit_DictComp(self, node: ast.DictComp) -> str:
+        key = self.visit(node.key)
+        value = self.visit(node.value)
+        generators = node.generators
+        dict_comp = f"{key} => {value} " + self._visit_generators(generators)
 
-        return "{0}{1}{2}.collect::<Vec<_>>()".format(iter, filter_str, map_str)
+        return f"Dict({dict_comp})"
 
-    def visit_ListComp(self, node):
-        return self.visit_GeneratorExp(node)  # right now they are the same
+    def _visit_generators(self, generators):
+        gen_exp = []
+        for i in range(len(generators)):
+            generator = generators[i]
+            target = self.visit(generator.target)
+            iter = self.visit(generator.iter)
+            exp = f"for {target} in {iter}"
+            gen_exp.append(exp) if i == 0 else gen_exp.append(f" {exp}")
+            filter_str = ""
+            if len(generator.ifs) == 1:
+                filter_str += f" if {self.visit(generator.ifs[0])} "
+            else:
+                for i in range(0, len(generator.ifs)):
+                    gen_if = generator.ifs[i]
+                    filter_str += (
+                        f" if {self.visit(gen_if)}"
+                        if i == 0
+                        else f" && {self.visit(gen_if)} "
+                    )
+            gen_exp.append(filter_str)
 
-    def visit_Global(self, node):
-        return "//global {0}".format(", ".join(node.names))
+        return "".join(gen_exp)
 
-    def visit_Starred(self, node):
-        return "starred!({0})/*unsupported*/".format(self.visit(node.value))
-
-    def visit_IfExp(self, node):
+    def visit_IfExp(self, node) -> str:
         body = self.visit(node.body)
         orelse = self.visit(node.orelse)
         test = self.visit(node.test)
