@@ -1,25 +1,24 @@
 import ast
-
 from typing import List, Tuple
 
+from py2many.analysis import get_id, is_mutable, is_void_function
+from py2many.ast_helpers import create_ast_block
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.tracer import defined_before, is_class_or_module, is_list, is_self_arg
+
 from .clike import CLikeTranspiler
 from .inference import get_inferred_kotlin_type
 from .plugins import (
     ATTR_DISPATCH_TABLE,
     CLASS_DISPATCH_TABLE,
+    DISPATCH_MAP,
     FUNC_DISPATCH_TABLE,
     MODULE_DISPATCH_TABLE,
-    DISPATCH_MAP,
     SMALL_DISPATCH_MAP,
     SMALL_USINGS_MAP,
 )
 
-from py2many.analysis import get_id, is_mutable, is_void_function
-from py2many.ast_helpers import create_ast_block
-from py2many.clike import class_for_typename
-from py2many.declaration_extractor import DeclarationExtractor
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_self_arg
-
 
 class KotlinPrintRewriter(ast.NodeTransformer):
     def __init__(self):
@@ -82,17 +81,9 @@ class KotlinBitOpRewriter(ast.NodeTransformer):
 class KotlinTranspiler(CLikeTranspiler):
     NAME = "kotlin"
 
-    CONTAINER_TYPE_MAP = {
-        "List": "Array",
-        "Dict": "Dict",
-        "Set": "Set",
-        "Optional": "Nothing",
-    }
-
     def __init__(self):
         super().__init__()
-        self._default_type = ""
-        self._container_type_map = self.CONTAINER_TYPE_MAP
+        CLikeTranspiler._default_type = ""
         self._dispatch_map = DISPATCH_MAP
         self._small_dispatch_map = SMALL_DISPATCH_MAP
         self._small_usings_map = SMALL_USINGS_MAP
@@ -105,7 +96,7 @@ class KotlinTranspiler(CLikeTranspiler):
         uses = "\n".join(f"import {mod}" for mod in usings)
         return uses
 
-    def visit_FunctionDef(self, node):
+    def visit_FunctionDef(self, node) -> str:
         body = "\n".join([self.visit(n) for n in node.body])
         typenames, args = self.visit_arguments(node.args)
 
@@ -126,7 +117,7 @@ class KotlinTranspiler(CLikeTranspiler):
             if is_python_main and arg == "argc":
                 continue
             if typename == "T":
-                typename = "T{0}".format(index)
+                typename = f"T{index}"
                 typedecls.append(typename)
                 index += 1
             args_list.append(f"{arg}: {typename}")
@@ -143,13 +134,13 @@ class KotlinTranspiler(CLikeTranspiler):
 
         template = ""
         if len(typedecls) > 0:
-            template = "<{0}>".format(", ".join(typedecls))
+            template = "<{}>".format(", ".join(typedecls))
 
         args = ", ".join(args_list)
         funcdef = f"fun {node.name}{template}({args}){return_type} {{"
         return funcdef + "\n" + body + "}\n\n"
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         if node.value:
             ret = self.visit(node.value)
             fndef = None
@@ -175,13 +166,13 @@ class KotlinTranspiler(CLikeTranspiler):
             typename = self._typename_from_annotation(node)
         return (typename, id)
 
-    def visit_Lambda(self, node):
+    def visit_Lambda(self, node) -> str:
         _, args = self.visit(node.args)
         args_string = ", ".join(args)
         body = self.visit(node.body)
         return f"{{ {args_string} -> {body} }}"
 
-    def visit_Attribute(self, node):
+    def visit_Attribute(self, node) -> str:
         attr = node.attr
 
         value_id = self.visit(node.value)
@@ -201,7 +192,7 @@ class KotlinTranspiler(CLikeTranspiler):
 
         return f"{value_id}.{attr}"
 
-    def visit_Call(self, node):
+    def visit_Call(self, node) -> str:
         fname = self.visit(node.func)
         vargs = []
 
@@ -230,29 +221,29 @@ class KotlinTranspiler(CLikeTranspiler):
 
         return f"{fname}({args})"
 
-    def visit_For(self, node):
+    def visit_For(self, node) -> str:
         target = self.visit(node.target)
         it = self.visit(node.iter)
         buf = []
-        buf.append("for ({0} in {1}) {{".format(target, it))
+        buf.append(f"for ({target} in {it}) {{")
         buf.extend([self.visit(c) for c in node.body])
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         return "" + super().visit_Str(node) + ""
 
-    def visit_Bytes(self, node):
-        bytes_str = "{0}".format(node.s)
+    def visit_Bytes(self, node) -> str:
+        bytes_str = f"{node.s}"
         return bytes_str.replace("'", '"')  # replace single quote with double quote
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id == "None":
             return "None"
         else:
             return super().visit_Name(node)
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is None:
             return "null"
         else:
@@ -265,23 +256,23 @@ class KotlinTranspiler(CLikeTranspiler):
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_If(self, node):
-        body_vars = set([get_id(v) for v in node.scopes[-1].body_vars])
-        orelse_vars = set([get_id(v) for v in node.scopes[-1].orelse_vars])
+    def visit_If(self, node) -> str:
+        body_vars = {get_id(v) for v in node.scopes[-1].body_vars}
+        orelse_vars = {get_id(v) for v in node.scopes[-1].orelse_vars}
         node.common_vars = body_vars.intersection(orelse_vars)
         return super().visit_If(node)
 
-    def visit_UnaryOp(self, node):
+    def visit_UnaryOp(self, node) -> str:
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_BinOp(self, node):
+    def visit_BinOp(self, node) -> str:
         if (
             isinstance(node.left, ast.List)
             and isinstance(node.op, ast.Mult)
@@ -293,7 +284,7 @@ class KotlinTranspiler(CLikeTranspiler):
         else:
             return super().visit_BinOp(node)
 
-    def visit_ClassDef(self, node):
+    def visit_ClassDef(self, node) -> str:
         extractor = DeclarationExtractor(KotlinTranspiler())
         extractor.visit(node)
         declarations = node.declarations = extractor.get_declarations()
@@ -316,7 +307,7 @@ class KotlinTranspiler(CLikeTranspiler):
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
-                typename = "ST{0}".format(index)
+                typename = f"ST{index}"
                 index += 1
             mut = is_mutable(node.scopes, get_id(declaration))
             mut = "var" if mut else "val"
@@ -337,7 +328,7 @@ class KotlinTranspiler(CLikeTranspiler):
             body = "\n".join(body)
             return f"class {node.name} {{\n{fields}\n\n {body}\n}}\n"
 
-    def _visit_enum(self, node, typename: str, fields: List[Tuple]):
+    def _visit_enum(self, node, typename: str, fields: List[Tuple]) -> str:
         fields_list = []
 
         for field, value in fields:
@@ -349,7 +340,7 @@ class KotlinTranspiler(CLikeTranspiler):
         fields_str = "".join(fields_list)
         return f"enum class {node.name}(val value: {typename}) {{\n{fields_str}\n}}"
 
-    def visit_StrEnum(self, node):
+    def visit_StrEnum(self, node) -> str:
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -358,7 +349,7 @@ class KotlinTranspiler(CLikeTranspiler):
             fields.append((member, var))
         return self._visit_enum(node, "String", fields)
 
-    def visit_IntEnum(self, node):
+    def visit_IntEnum(self, node) -> str:
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -367,7 +358,7 @@ class KotlinTranspiler(CLikeTranspiler):
             fields.append((member, var))
         return self._visit_enum(node, "Int", fields)
 
-    def visit_IntFlag(self, node):
+    def visit_IntFlag(self, node) -> str:
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
@@ -379,7 +370,7 @@ class KotlinTranspiler(CLikeTranspiler):
     def _import(self, name: str) -> str:
         return f"import {name};"
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         if len(names) == 1:
             # TODO: make this more generic so it works for len(names) > 1
             name = names[0]
@@ -389,31 +380,31 @@ class KotlinTranspiler(CLikeTranspiler):
                 return f"from {kt_module_name} import {kt_name}"
         return "\n".join([f"import {module_name}.{name}" for name in names])
 
-    def visit_List(self, node):
+    def visit_List(self, node) -> str:
         elements = [self.visit(e) for e in node.elts]
         elements_str = ", ".join(elements)
         return f"arrayOf({elements_str})"
 
-    def visit_Set(self, node):
+    def visit_Set(self, node) -> str:
         elements = [self.visit(e) for e in node.elts]
         elements_str = ", ".join(elements)
         return f"setOf({elements_str})"
 
-    def visit_Dict(self, node):
+    def visit_Dict(self, node) -> str:
         keys = [self.visit(k) for k in node.keys]
         values = [self.visit(k) for k in node.values]
         kv_pairs = ", ".join([f"{k} to {v}" for k, v in zip(keys, values)])
         return f"hashMapOf({kv_pairs})"
 
-    def visit_Subscript(self, node):
+    def visit_Subscript(self, node) -> str:
         value = self.visit(node.value)
         index = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
-            if value in self.CONTAINER_TYPE_MAP:
-                value = self.CONTAINER_TYPE_MAP[value]
+            if value in self._container_type_map:
+                value = self._container_type_map[value]
             if value == "Tuple":
-                return "({0})".format(index)
-            return "{0}<{1}>".format(value, index)
+                return f"({index})"
+            return f"{value}<{index}>"
         self._generic_typename_from_annotation(node.value)
         container_type = hasattr(node.value, "annotation") and getattr(
             node.value.annotation, "generic_container_type", None
@@ -423,10 +414,10 @@ class KotlinTranspiler(CLikeTranspiler):
         )
         return f"{value}[{index}]!!" if is_container_dict else f"{value}[{index}]"
 
-    def visit_Index(self, node):
+    def visit_Index(self, node) -> str:
         return self.visit(node.value)
 
-    def visit_Slice(self, node):
+    def visit_Slice(self, node) -> str:
         lower = ""
         if node.lower:
             lower = self.visit(node.lower)
@@ -434,40 +425,20 @@ class KotlinTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Tuple(self, node):
+    def visit_Tuple(self, node) -> str:
         elts = [self.visit(e) for e in node.elts]
         elts = ", ".join(elts)
         if hasattr(node, "is_annotation"):
             return elts
-        return "({0})".format(elts)
+        return f"({elts})"
 
-    def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
-
-        for handler in node.handlers:
-            buf += self.visit(handler)
-        # buf.append("\n".join(excepts));
-
-        if finallybody:
-            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
-
-        return "\n".join(buf)
-
-    def visit_ExceptHandler(self, node):
-        exception_type = ""
-        if node.type:
-            exception_type = self.visit(node.type)
-        name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(node, name, node.body)
-        return body
-
-    def visit_Assert(self, node):
+    def visit_Assert(self, node) -> str:
         condition = self.visit(node.test)
         return f"assert({condition})"
 
-    def visit_AnnAssign(self, node):
+    def visit_AnnAssign(self, node) -> str:
         target = self.visit(node.target)
         type_str = self._typename_from_annotation(node)
         val = self.visit(node.value) if node.value is not None else None
@@ -475,7 +446,7 @@ class KotlinTranspiler(CLikeTranspiler):
             return f"var {target} = {val}"
         return f"var {target}: {type_str} = {val}"
 
-    def _visit_AssignOne(self, node, target):
+    def _visit_AssignOne(self, node, target) -> str:
         kw = "var" if is_mutable(node.scopes, get_id(target)) else "val"
 
         if isinstance(target, ast.Tuple):
@@ -517,44 +488,17 @@ class KotlinTranspiler(CLikeTranspiler):
 
             return f"{kw} {target} = {value}"
 
-    def visit_Delete(self, node):
-        target = node.targets[0]
-        return "{0}.drop()".format(self.visit(target))
-
-    def visit_Raise(self, node):
+    def visit_Raise(self, node) -> str:
         if node.exc is not None:
             exc = self.visit(node.exc)
             return f"throw Exception({exc})"
         return "throw Exception()"
 
-    def visit_Await(self, node):
-        expr = self.visit(node.value)
-        return f"{expr}.await()"
-
-    def visit_AsyncFunctionDef(self, node):
-        fn = self.visit_FunctionDef(node)
-        return f"suspend {fn}"
-
-    def visit_Yield(self, node):
-        return "//yield is unimplemented"
-
-    def visit_Print(self, node):
+    def visit_Print(self, node) -> str:
         vargs_str = " ".join([f"${arg}" for arg in node.values])
         return f'println("{vargs_str}")'
 
-    def visit_GeneratorExp(self, node):
-        return "GeneratorExp /*unimplemented()*/"
-
-    def visit_ListComp(self, node):
-        return self.visit_GeneratorExp(node)  # right now they are the same
-
-    def visit_Global(self, node):
-        return "//global {0}".format(", ".join(node.names))
-
-    def visit_Starred(self, node):
-        return "starred!({0})/*unsupported*/".format(self.visit(node.value))
-
-    def visit_IfExp(self, node):
+    def visit_IfExp(self, node) -> str:
         body = self.visit(node.body)
         orelse = self.visit(node.orelse)
         test = self.visit(node.test)
