@@ -1,26 +1,25 @@
 import ast
+from typing import List
+
+from py2many.analysis import get_id, is_mutable, is_void_function
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.exceptions import AstClassUsedBeforeDeclaration
+from py2many.tracer import defined_before, is_list
 
 from .clike import CLikeTranspiler
 from .inference import get_inferred_nim_type
 from .plugins import (
     ATTR_DISPATCH_TABLE,
     CLASS_DISPATCH_TABLE,
+    DISPATCH_MAP,
     FUNC_DISPATCH_TABLE,
     FUNC_USINGS_MAP,
     MODULE_DISPATCH_TABLE,
-    DISPATCH_MAP,
     SMALL_DISPATCH_MAP,
     SMALL_USINGS_MAP,
 )
 
-from py2many.analysis import get_id, is_mutable, is_void_function
-from py2many.clike import class_for_typename
-from py2many.declaration_extractor import DeclarationExtractor
-from py2many.exceptions import AstClassUsedBeforeDeclaration
-from py2many.tracer import is_list, defined_before
-
-from typing import List
-
 
 class NimNoneCompareRewriter(ast.NodeTransformer):
     def visit_Compare(self, node):
@@ -40,19 +39,11 @@ class NimNoneCompareRewriter(ast.NodeTransformer):
 class NimTranspiler(CLikeTranspiler):
     NAME = "nim"
 
-    CONTAINER_TYPE_MAP = {
-        "List": "seq",
-        "Dict": "Table",
-        "Set": "set",
-        "Optional": "Option",
-    }
-
     def __init__(self, indent=2):
         super().__init__()
-        self._headers = set([])
+        self._headers = set()
         self._indent = " " * indent
-        self._default_type = "var"
-        self._container_type_map = self.CONTAINER_TYPE_MAP
+        CLikeTranspiler._default_type = "var"
         if "math" in self._ignored_module_set:
             self._ignored_module_set.remove("math")
         self._dispatch_map = DISPATCH_MAP
@@ -70,13 +61,14 @@ class NimTranspiler(CLikeTranspiler):
         uses = "\n".join(f"import {mod}" for mod in usings)
         return uses
 
-    def _combine_value_index(self, value_type, index_type) -> str:
+    @classmethod
+    def _combine_value_index(cls, value_type, index_type) -> str:
         return f"{value_type}[{index_type}]"
 
     def comment(self, text):
         return f"# {text}\n"
 
-    def visit_FunctionDef(self, node):
+    def visit_FunctionDef(self, node) -> str:
         body = "\n".join([self.indent(self.visit(n)) for n in node.body])
         typenames, args = self.visit(node.args)
 
@@ -107,7 +99,7 @@ class NimTranspiler(CLikeTranspiler):
             maybe_main = "\nmain()"
         return f"{funcdef}\n{body}\n{maybe_main}"
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         if node.value:
             ret = self.visit(node.value)
             fndef = None
@@ -139,7 +131,7 @@ class NimTranspiler(CLikeTranspiler):
                 typename = f"var {typename}"
         return (typename, id)
 
-    def visit_Lambda(self, node):
+    def visit_Lambda(self, node) -> str:
         typenames, args = self.visit(node.args)
         # HACK: to pass unit tests. TODO: infer types
         typenames = ["int"] * len(args)
@@ -149,7 +141,7 @@ class NimTranspiler(CLikeTranspiler):
         body = self.visit(node.body)
         return f"proc({args_string}):{return_type} = return {body}"
 
-    def visit_Attribute(self, node):
+    def visit_Attribute(self, node) -> str:
         attr = node.attr
 
         value_id = self.visit(node.value)
@@ -171,7 +163,7 @@ class NimTranspiler(CLikeTranspiler):
             return ret(self, node, value_id, attr)
         return ret
 
-    def _visit_object_literal(self, node, fname: str, fndef: ast.ClassDef):
+    def _visit_object_literal(self, node, fname: str, fndef: ast.ClassDef) -> str:
         vargs = []  # visited args
         if not hasattr(fndef, "declarations"):
             raise AstClassUsedBeforeDeclaration(fndef, node)
@@ -186,7 +178,7 @@ class NimTranspiler(CLikeTranspiler):
         args = ", ".join(vargs)
         return f"{fname}({args})"
 
-    def visit_Call(self, node):
+    def visit_Call(self, node) -> str:
         fname = self.visit(node.func)
         fndef = node.scopes.find(fname)
 
@@ -209,7 +201,7 @@ class NimTranspiler(CLikeTranspiler):
             args = ""
         return f"{fname}({args})"
 
-    def visit_For(self, node):
+    def visit_For(self, node) -> str:
         target = self.visit(node.target)
         it = self.visit(node.iter)
         buf = []
@@ -219,36 +211,36 @@ class NimTranspiler(CLikeTranspiler):
         )
         return "\n".join(buf)
 
-    def visit_While(self, node):
+    def visit_While(self, node) -> str:
         buf = []
-        buf.append("while {0}:".format(self.visit(node.test)))
+        buf.append(f"while {self.visit(node.test)}:")
         buf.extend(
             [self.indent(self.visit(n), level=node.level + 1) for n in node.body]
         )
         return "\n".join(buf)
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         return "" + super().visit_Str(node) + ""
 
-    def visit_Bytes(self, node):
-        bytes_str = "{0}".format(node.s)
+    def visit_Bytes(self, node) -> str:
+        bytes_str = f"{node.s}"
         return bytes_str.replace("'", '"')  # replace single quote with double quote
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id == "None":
             return "nil"
         else:
             return super().visit_Name(node)
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is None:
             return "nil"
         else:
             return super().visit_NameConstant(node)
 
-    def visit_If(self, node):
-        body_vars = set([get_id(v) for v in node.scopes[-1].body_vars])
-        orelse_vars = set([get_id(v) for v in node.scopes[-1].orelse_vars])
+    def visit_If(self, node) -> str:
+        body_vars = {get_id(v) for v in node.scopes[-1].body_vars}
+        orelse_vars = {get_id(v) for v in node.scopes[-1].orelse_vars}
         node.common_vars = body_vars.intersection(orelse_vars)
 
         body = "\n".join(
@@ -270,29 +262,17 @@ class NimTranspiler(CLikeTranspiler):
             orelse = ""
         return f"if {test}:\n{body}\n{orelse}"
 
-    def visit_UnaryOp(self, node):
+    def visit_UnaryOp(self, node) -> str:
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_BinOp(self, node):
-        if (
-            isinstance(node.left, ast.List)
-            and isinstance(node.op, ast.Mult)
-            and isinstance(node.right, ast.Num)
-        ):
-            return "std::vector ({0},{1})".format(
-                self.visit(node.right), self.visit(node.left.elts[0])
-            )
-        else:
-            return super().visit_BinOp(node)
-
-    def visit_ClassDef(self, node):
+    def visit_ClassDef(self, node) -> str:
         extractor = DeclarationExtractor(NimTranspiler())
         extractor.visit(node)
         declarations = node.declarations = extractor.get_declarations()
@@ -315,7 +295,7 @@ class NimTranspiler(CLikeTranspiler):
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
-                typename = "ST{0}".format(index)
+                typename = f"ST{index}"
                 index += 1
             fields.append(f"{declaration}: {typename}")
 
@@ -330,7 +310,7 @@ class NimTranspiler(CLikeTranspiler):
         body = "\n".join(body)
         return f"{object_def}\n{body}\n"
 
-    def visit_IntEnum(self, node):
+    def visit_IntEnum(self, node) -> str:
         fields = []
         for member, var in node.class_assignments.items():
             var = self.visit(var)
@@ -341,7 +321,7 @@ class NimTranspiler(CLikeTranspiler):
         fields = "\n".join([self.indent(f) for f in fields])
         return f"type {node.name} = enum\n{fields}\n\n"
 
-    def visit_IntFlag(self, node):
+    def visit_IntFlag(self, node) -> str:
         fields = []
         for member, var in node.class_assignments.items():
             var = self.visit(var)
@@ -361,7 +341,7 @@ class NimTranspiler(CLikeTranspiler):
             ]
         )
 
-    def visit_StrEnum(self, node):
+    def visit_StrEnum(self, node) -> str:
         fields = []
         for member, var in node.class_assignments.items():
             var = self.visit(var)
@@ -375,7 +355,7 @@ class NimTranspiler(CLikeTranspiler):
     def _import(self, name: str) -> str:
         return f"import {name}"
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         names = ", ".join(names)
         if len(names) == 1:
             # TODO: make this more generic so it works for len(names) > 1
@@ -386,39 +366,39 @@ class NimTranspiler(CLikeTranspiler):
                 return f"from {nim_module_name} import {nim_name}"
         return f"from {module_name} import {names}"
 
-    def visit_List(self, node):
+    def visit_List(self, node) -> str:
         elements = [self.visit(e) for e in node.elts]
         elements = ", ".join(elements)
         return f"@[{elements}]"
 
-    def visit_Set(self, node):
+    def visit_Set(self, node) -> str:
         self._usings.add("sets")
         elements = [self.visit(e) for e in node.elts]
         elements_str = ", ".join(elements)
         return f"toHashSet([{elements_str}])"
 
-    def visit_Dict(self, node):
+    def visit_Dict(self, node) -> str:
         self._usings.add("tables")
         keys = [self.visit(k) for k in node.keys]
         values = [self.visit(k) for k in node.values]
         kv_pairs = ", ".join([f"{k}: {v}" for k, v in zip(keys, values)])
         return f"{{{kv_pairs}}}.newTable"
 
-    def visit_Subscript(self, node):
+    def visit_Subscript(self, node) -> str:
         value = self.visit(node.value)
         index = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
-            if value in self.CONTAINER_TYPE_MAP:
-                value = self.CONTAINER_TYPE_MAP[value]
+            if value in self._container_type_map:
+                value = self._container_type_map[value]
             if value == "Tuple":
                 return f"({index})"
             return f"{value}[{index}]"
         return f"{value}[{index}]"
 
-    def visit_Index(self, node):
+    def visit_Index(self, node) -> str:
         return self.visit(node.value)
 
-    def visit_Slice(self, node):
+    def visit_Slice(self, node) -> str:
         lower = ""
         if node.lower:
             lower = self.visit(node.lower)
@@ -426,46 +406,26 @@ class NimTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Tuple(self, node):
+    def visit_Tuple(self, node) -> str:
         elts = [self.visit(e) for e in node.elts]
         elts = ", ".join(elts)
         if hasattr(node, "is_annotation"):
             return elts
-        return "({0})".format(elts)
+        return f"({elts})"
 
-    def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
+    def visit_Assert(self, node) -> str:
+        return f"assert({self.visit(node.test)})"
 
-        for handler in node.handlers:
-            buf += self.visit(handler)
-        # buf.append("\n".join(excepts));
-
-        if finallybody:
-            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
-
-        return "\n".join(buf)
-
-    def visit_ExceptHandler(self, node):
-        exception_type = ""
-        if node.type:
-            exception_type = self.visit(node.type)
-        name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(node, name, node.body)
-        return body
-
-    def visit_Assert(self, node):
-        return "assert({0})".format(self.visit(node.test))
-
-    def visit_AnnAssign(self, node):
+    def visit_AnnAssign(self, node) -> str:
         target, type_str, val = super().visit_AnnAssign(node)
         kw = "var" if is_mutable(node.scopes, target) else "let"
         if type_str == self._default_type:
             return f"{kw} {target} = {val}"
         return f"{kw} {target}: {type_str} = {val}"
 
-    def visit_Assign(self, node):
+    def visit_Assign(self, node) -> str:
         lines = [self._visit_AssignOne(node, target) for target in node.targets]
         if len(lines) > 1:
             line0 = lines[0]
@@ -473,7 +433,7 @@ class NimTranspiler(CLikeTranspiler):
             lines[0] = line0  # parent node is going to add indentation
         return "\n".join(lines)
 
-    def _visit_AssignOne(self, node, target):
+    def _visit_AssignOne(self, node, target) -> str:
         kw = "var" if is_mutable(node.scopes, get_id(target)) else "let"
 
         if isinstance(target, ast.Tuple):
@@ -507,65 +467,20 @@ class NimTranspiler(CLikeTranspiler):
 
             return f"{kw} {target} = {value}"
 
-    def visit_Delete(self, node):
-        target = node.targets[0]
-        return "{0}.drop()".format(self.visit(target))
-
-    def visit_Raise(self, node):
-        if node.exc is not None:
-            return "raise!({0}); //unsupported".format(self.visit(node.exc))
-        # This handles the case where `raise` is used without
-        # specifying the exception.
-        return "raise!(); //unsupported"
-
-    def visit_Await(self, node):
-        return "await!({0})".format(self.visit(node.value))
-
-    def visit_AsyncFunctionDef(self, node):
-        return "#[async]\n{0}".format(self.visit_FunctionDef(node))
-
-    def visit_Yield(self, node):
+    def visit_Yield(self, node) -> str:
         if node.value is not None:
             value = self.visit(node.value)
             return f"yield {value}"
         return "yield nil"
 
-    def visit_Print(self, node):
+    def visit_Print(self, node) -> str:
         buf = []
         for n in node.values:
             value = self.visit(n)
-            buf.append('println("{{:?}}",{0})'.format(value))
+            buf.append(f'println("{{:?}}",{value})')
         return "\n".join(buf)
 
-    def visit_GeneratorExp(self, node):
-        elt = self.visit(node.elt)
-        generator = node.generators[0]
-        target = self.visit(generator.target)
-        iter = self.visit(generator.iter)
-
-        # HACK for dictionary iterators to work
-        if not iter.endswith("keys()") or iter.endswith("values()"):
-            iter += ".iter()"
-
-        map_str = ".map(|{0}| {1})".format(target, elt)
-        filter_str = ""
-        if generator.ifs:
-            filter_str = ".cloned().filter(|&{0}| {1})".format(
-                target, self.visit(generator.ifs[0])
-            )
-
-        return "{0}{1}{2}.collect::<Vec<_>>()".format(iter, filter_str, map_str)
-
-    def visit_ListComp(self, node):
-        return self.visit_GeneratorExp(node)  # right now they are the same
-
-    def visit_Global(self, node):
-        return "//global {0}".format(", ".join(node.names))
-
-    def visit_Starred(self, node):
-        return "starred!({0})/*unsupported*/".format(self.visit(node.value))
-
-    def visit_IfExp(self, node):
+    def visit_IfExp(self, node) -> str:
         body = self.visit(node.body)
         orelse = self.visit(node.orelse)
         test = self.visit(node.test)
