@@ -1,15 +1,16 @@
 import ast
 
-from py2many.clike import CLikeTranspiler as CommonCLikeTranspiler, LifeTime
+from py2many.clike import CLikeTranspiler as CommonCLikeTranspiler
+from py2many.clike import LifeTime
 
 from .inference import (
+    RUST_CONTAINER_TYPE_MAP,
     RUST_RANK_TO_TYPE,
-    RUST_WIDTH_RANK,
     RUST_TYPE_MAP,
+    RUST_WIDTH_RANK,
     is_rust_reference,
 )
 
-
 # allowed as names in Python but treated as keywords in Rust
 RUST_KEYWORDS = frozenset(
     [
@@ -37,24 +38,25 @@ RUST_KEYWORDS = frozenset(
 class CLikeTranspiler(CommonCLikeTranspiler):
     def __init__(self):
         super().__init__()
-        self._type_map = RUST_TYPE_MAP
+        CommonCLikeTranspiler._type_map = RUST_TYPE_MAP
+        CommonCLikeTranspiler._container_type_map = RUST_CONTAINER_TYPE_MAP
         self._keywords = RUST_KEYWORDS
 
-    def _map_type(self, typename, lifetime=LifeTime.UNKNOWN) -> str:
-        ret = super()._map_type(typename, lifetime)
-        if lifetime == LifeTime.STATIC:
-            assert ret[0] == "&"
+    @classmethod
+    def _map_type(cls, typename, lifetime=LifeTime.UNKNOWN) -> str:
+        ret = CommonCLikeTranspiler._map_type(typename, lifetime)
+        if lifetime == LifeTime.STATIC and ret[0] == "&":
             return f"&'static {ret[1:]}"
         return ret
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id in self._keywords:
             return node.id + "_"
         return super().visit_Name(node)
 
-    def visit_BinOp(self, node):
+    def visit_BinOp(self, node) -> str:
         if isinstance(node.op, ast.Pow):
-            return "pow({0}, {1})".format(self.visit(node.left), self.visit(node.right))
+            return f"pow({self.visit(node.left)}, {self.visit(node.right)})"
 
         left = self.visit(node.left)
         op = self.visit(node.op)
@@ -88,7 +90,7 @@ class CLikeTranspiler(CommonCLikeTranspiler):
         else:
             return f"({left} {op} {right})"
 
-    def visit_Compare(self, node):
+    def visit_Compare(self, node) -> str:
         if isinstance(node.ops[0], ast.In):
             return self.visit_In(node)
 
@@ -116,7 +118,7 @@ class CLikeTranspiler(CommonCLikeTranspiler):
 
         return f"{left} {op} {right}"
 
-    def visit_In(self, node):
+    def visit_In(self, node) -> str:
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
-        return "{0}.any({1})".format(right, left)
+        return f"{right}.any({left})"
