@@ -1,19 +1,7 @@
 import ast
-from py2many.rewriters import camel_case
 import textwrap
-
-from .clike import CLikeTranspiler
-from .inference import get_inferred_rust_type, map_type
-from .plugins import (
-    ATTR_DISPATCH_TABLE,
-    CLASS_DISPATCH_TABLE,
-    FUNC_DISPATCH_TABLE,
-    FUNC_USINGS_MAP,
-    MODULE_DISPATCH_TABLE,
-    DISPATCH_MAP,
-    SMALL_DISPATCH_MAP,
-    SMALL_USINGS_MAP,
-)
+from pathlib import Path
+from typing import List, Tuple, Union
 
 from py2many.analysis import (
     FunctionTransformer,
@@ -26,10 +14,21 @@ from py2many.clike import class_for_typename
 from py2many.declaration_extractor import DeclarationExtractor
 from py2many.exceptions import AstClassUsedBeforeDeclaration
 from py2many.inference import is_reference
-from py2many.tracer import is_list, defined_before, is_class_or_module
+from py2many.rewriters import camel_case
+from py2many.tracer import defined_before, is_class_or_module, is_list
 
-from pathlib import Path
-from typing import List, Tuple, Union
+from .clike import CLikeTranspiler
+from .inference import get_inferred_rust_type, map_type
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    FUNC_DISPATCH_TABLE,
+    FUNC_USINGS_MAP,
+    MODULE_DISPATCH_TABLE,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
 
 class RustLoopIndexRewriter(ast.NodeTransformer):
@@ -78,18 +77,9 @@ class RustStringJoinRewriter(ast.NodeTransformer):
 class RustTranspiler(CLikeTranspiler):
     NAME = "rust"
 
-    CONTAINER_TYPE_MAP = {
-        "List": "Vec",
-        "Dict": "HashMap",
-        "Set": "HashSet",
-        "Optional": "Option",
-        "Result": "Result",
-    }
-
     def __init__(self, extension: bool = False, no_prologue: bool = False):
         super().__init__()
-        self._container_type_map = self.CONTAINER_TYPE_MAP
-        self._default_type = "_"
+        CLikeTranspiler._default_type = "_"
         self._extension = extension
         self._rust_ignored_module_set = {"argparse_dataclass"}
         self._no_prologue = no_prologue
@@ -100,6 +90,7 @@ class RustTranspiler(CLikeTranspiler):
         self._func_usings_map = FUNC_USINGS_MAP
         self._attr_dispatch_table = ATTR_DISPATCH_TABLE
         self._allows = set()
+        self._rust_mods = set()
 
     def usings(self):
         if self._extension:
@@ -107,29 +98,39 @@ class RustTranspiler(CLikeTranspiler):
             self._usings.add("pyo3::wrap_pyfunction")
         usings = sorted(list(set(self._usings)))
         deps = sorted(
-            set(mod.split("::")[0] for mod in usings if not mod.startswith("std:"))
+            {mod.split("::")[0] for mod in usings if not mod.startswith("std:")}
         )
-        externs = [f"extern crate {dep};" for dep in deps]
+        crates = [dep.replace("-", "_") for dep in deps]
+        externs = [f"extern crate {dep};" for dep in crates]
+        externs += [f"mod {dep};" for dep in self._rust_mods]
         deps_str = "\n//! ".join([f'{dep} = "*"' for dep in deps])
         externs = "\n".join(externs)
-        uses = "\n".join(
-            f"use {mod};"
+        uses = [
+            mod.replace("-", "_")
             for mod in usings
-            if mod not in ("strum", "lazy_static") and mod not in deps
-        )
+            if mod not in ("strum", "lazy_static", "float-ord") and mod not in deps
+        ]
+        uses = "\n".join(f"use {mod};" for mod in uses)
         # This should not contain lints which mask possibly erroneous semantics, e.g. float_cmp
         # Those, and lints triggered by explicitly bad test logic, belong in
         # .github/workflows/clippy.yml
         lint_ignores = (
             textwrap.dedent(
                 """
+        #![allow(clippy::assertions_on_constants)]
+        #![allow(clippy::bool_comparison)]
         #![allow(clippy::collapsible_else_if)]
-        #![allow(clippy::double_parens)]  // https://github.com/adsharma/py2many/issues/17
+        #![allow(clippy::comparison_to_empty)]
+        #![allow(clippy::double_parens)] // https://github.com/adsharma/py2many/issues/17
+        #![allow(clippy::eq_op)]
+        #![allow(clippy::let_with_type_underscore)]
         #![allow(clippy::map_identity)]
         #![allow(clippy::needless_return)]
+        #![allow(clippy::nonminimal_bool)]
+        #![allow(clippy::partialeq_to_none)]
         #![allow(clippy::print_literal)]
         #![allow(clippy::ptr_arg)]
-        #![allow(clippy::redundant_static_lifetimes)]  // https://github.com/adsharma/py2many/issues/266
+        #![allow(clippy::redundant_static_lifetimes)] // https://github.com/adsharma/py2many/issues/266
         #![allow(clippy::unnecessary_cast)]
         #![allow(clippy::upper_case_acronyms)]
         #![allow(clippy::useless_vec)]
@@ -145,18 +146,14 @@ class RustTranspiler(CLikeTranspiler):
             else ""
         )
         lint_ignores += "\n".join(f"#![allow({allow})]" for allow in self._allows)
-        cargo_toml = (
-            f"""\
+        cargo_toml = f"""\
         //! ```cargo
         //! [package]
-        //! edition = "2018"
+        //! edition = "2021"
         //! [dependencies]
         //! {deps_str}
         //! ```
         """
-            if not self._no_prologue
-            else ""
-        )
         return f"{cargo_toml}\n{lint_ignores}\n\n{externs}\n{uses}\n"
 
     def features(self):
@@ -188,12 +185,12 @@ class RustTranspiler(CLikeTranspiler):
             )
         return ""
 
-    def visit_Expr(self, node):
+    def visit_Expr(self, node) -> str:
         if hasattr(node, "unused"):
             self._allows.add("clippy::no_effect")
         return super().visit_Expr(node)
 
-    def visit_FunctionDef(self, node, async_prefix=""):
+    def visit_FunctionDef(self, node, async_prefix="") -> str:
         body = "\n".join([self.visit(n) for n in node.body])
         typenames, args = self.visit(node.args)
 
@@ -214,10 +211,10 @@ class RustTranspiler(CLikeTranspiler):
             arg = args[i]
 
             if typename == "T":
-                typename = "T{0}".format(index)
+                typename = f"T{index}"
                 typedecls.append(typename)
                 index += 1
-            args_list.append("{0}: {1}".format(arg, typename))
+            args_list.append(f"{arg}: {typename}")
 
         return_type = "" if not is_python_main else "-> Result<()>"
         if node.returns:
@@ -228,7 +225,9 @@ class RustTranspiler(CLikeTranspiler):
                 if node.no_return:
                     typename = None
                 else:
-                    typename = self._generic_typename_from_type_node(node.returns)
+                    typename = CLikeTranspiler._generic_typename_from_type_node(
+                        node.returns
+                    )
                 typename = map_type(typename, extension=True, return_type=True)
             if typename != "_":
                 return_type = f"-> {typename}"
@@ -239,7 +238,7 @@ class RustTranspiler(CLikeTranspiler):
 
         template = ""
         if len(typedecls) > 0:
-            template = "<{0}>".format(", ".join(typedecls))
+            template = "<{}>".format(", ".join(typedecls))
 
         extension = "#[pyfunction]\n" if self.extension else ""
         args_list = ", ".join(args_list)
@@ -257,6 +256,7 @@ class RustTranspiler(CLikeTranspiler):
         if node.annotation:
             if not self._extension:
                 typename = self._typename_from_annotation(node)
+                typename = map_type(typename)
             else:
                 typename = self._generic_typename_from_annotation(node)
                 typename = map_type(typename, extension=True)
@@ -268,7 +268,7 @@ class RustTranspiler(CLikeTranspiler):
                 typename = f"&{mut}{typename}"
         return (typename, id)
 
-    def visit_Return(self, node):
+    def visit_Return(self, node) -> str:
         fndef = None
         for scope in node.scopes:
             if isinstance(scope, ast.FunctionDef):
@@ -298,13 +298,13 @@ class RustTranspiler(CLikeTranspiler):
                 return "return Ok(())"
         return "return;"
 
-    def visit_Lambda(self, node):
+    def visit_Lambda(self, node) -> str:
         _, args = self.visit(node.args)
         args_string = ", ".join(args)
         body = self.visit(node.body)
-        return "|{0}| {1}".format(args_string, body)
+        return f"|{args_string}| {body}"
 
-    def visit_Attribute(self, node):
+    def visit_Attribute(self, node) -> str:
         attr = node.attr
         if attr in self._keywords:
             attr = attr + "_"
@@ -323,7 +323,7 @@ class RustTranspiler(CLikeTranspiler):
             value_id = ""
 
         if is_class_or_module(value_id, node.scopes):
-            return "{0}::{1}".format(value_id, attr)
+            return f"{value_id}::{attr}"
 
         ret = f"{value_id}.{attr}"
         if ret in self._attr_dispatch_table:
@@ -343,7 +343,7 @@ class RustTranspiler(CLikeTranspiler):
         else:
             return ("", splits[0])
 
-    def _visit_struct_literal(self, node, fname: str, fndef: ast.ClassDef):
+    def _visit_struct_literal(self, node, fname: str, fndef: ast.ClassDef) -> str:
         vargs = []  # visited args
         if not hasattr(fndef, "declarations"):
             raise AstClassUsedBeforeDeclaration(fndef, node)
@@ -362,7 +362,7 @@ class RustTranspiler(CLikeTranspiler):
         args = ", ".join(vargs)
         return f"{fname}{{{args}}}"
 
-    def visit_Call(self, node):
+    def visit_Call(self, node) -> str:
         fname = self.visit(node.func)
         fndef = node.scopes.find(fname)
 
@@ -399,24 +399,24 @@ class RustTranspiler(CLikeTranspiler):
         unwrap = "?" if node_result_type or node_func_result_type else ""
         return f"{fname}({args}){unwrap}"
 
-    def visit_For(self, node):
+    def visit_For(self, node) -> str:
         target = self.visit(node.target)
         it = self.visit(node.iter)
         buf = []
-        buf.append("for {0} in {1} {{".format(target, it))
+        buf.append(f"for {target} in {it} {{")
         buf.extend([self.visit(c) for c in node.body])
         buf.append("}")
         return "\n".join(buf)
 
-    def visit_Str(self, node):
+    def visit_Str(self, node) -> str:
         return "" + super().visit_Str(node) + ""
 
-    def visit_Bytes(self, node):
+    def visit_Bytes(self, node) -> str:
         bytes_str = node.s
         bytes_str = bytes_str.replace(b'"', b'\\"')
         return 'b"' + bytes_str.decode("ascii", "backslashreplace") + '"'
 
-    def visit_Compare(self, node):
+    def visit_Compare(self, node) -> str:
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
 
@@ -428,17 +428,25 @@ class RustTranspiler(CLikeTranspiler):
             if value_type and value_type[0] == "Dict":
                 right += ".keys()"
 
-        if not right.endswith(".keys()") and not right.endswith(".values()"):
-            right += ".iter()"
+        cmpop = "contains"
+        left = f"&{left}"
+        if right.endswith(".keys()"):
+            right = right[:-7]  # remove the .keys()
+            cmpop = "contains_key"
+        elif right.endswith(".values()"):
+            if isinstance(node.ops[0], ast.In):
+                return f"{right}.any(|x| x == {left})"
+            elif isinstance(node.ops[0], ast.NotIn):
+                return f"{right}.all(|x| x != {left})"
 
         if isinstance(node.ops[0], ast.In):
-            return "{0}.any(|&x| x == {1})".format(right, left)  # is it too much?
+            return f"{right}.{cmpop}({left})"
         elif isinstance(node.ops[0], ast.NotIn):
-            return "{0}.all(|&x| x != {1})".format(right, left)  # is it even more?
+            return f"!{right}.{cmpop}({left})"
 
         return super().visit_Compare(node)
 
-    def visit_Name(self, node):
+    def visit_Name(self, node) -> str:
         if node.id == "None":
             return "None"
         else:
@@ -452,7 +460,7 @@ class RustTranspiler(CLikeTranspiler):
                 return f"*{ret}"
             return ret
 
-    def visit_NameConstant(self, node):
+    def visit_NameConstant(self, node) -> str:
         if node.value is True:
             return "true"
         elif node.value is False:
@@ -462,9 +470,9 @@ class RustTranspiler(CLikeTranspiler):
         else:
             return super().visit_NameConstant(node)
 
-    def visit_If(self, node):
-        body_vars = set([get_id(v) for v in node.scopes[-1].body_vars])
-        orelse_vars = set([get_id(v) for v in node.scopes[-1].orelse_vars])
+    def visit_If(self, node) -> str:
+        body_vars = {get_id(v) for v in node.scopes[-1].body_vars}
+        orelse_vars = {get_id(v) for v in node.scopes[-1].orelse_vars}
         node.common_vars = body_vars.intersection(orelse_vars)
 
         # TODO find out if this can be useful
@@ -484,24 +492,24 @@ class RustTranspiler(CLikeTranspiler):
             return f"{ret};"
         return ret
 
-    def visit_While(self, node):
+    def visit_While(self, node) -> str:
         test = self.visit(node.test)
         if test == "true":
             body = [self.visit(n) for n in node.body]
             return "loop {{\n{}\n}}\n".format("\n".join(body))
         return super().visit_While(node, use_parens=False)
 
-    def visit_UnaryOp(self, node):
+    def visit_UnaryOp(self, node) -> str:
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_BinOp(self, node):
+    def visit_BinOp(self, node) -> str:
         if (
             isinstance(node.left, ast.List)
             and isinstance(node.op, ast.Mult)
@@ -512,7 +520,7 @@ class RustTranspiler(CLikeTranspiler):
         else:
             return super().visit_BinOp(node)
 
-    def visit_sealed_class(self, node):
+    def visit_sealed_class(self, node) -> str:
         variants = []
         accessors = []
         tag_checkers = []
@@ -549,7 +557,7 @@ class RustTranspiler(CLikeTranspiler):
         impl_str = "\n".join(accessors) + "\n" + "\n".join(tag_checkers)
         return f"enum {camel_node_name} {{ {body_str} }}\n\n impl {camel_node_name} {{ {impl_str} }}\n\n"
 
-    def visit_ClassDef(self, node):
+    def visit_ClassDef(self, node) -> str:
         extractor = DeclarationExtractor(RustTranspiler())
         extractor.visit(node)
         node.declarations = declarations = extractor.get_declarations()
@@ -577,10 +585,14 @@ class RustTranspiler(CLikeTranspiler):
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
-                typename = "ST{0}".format(index)
+                typename = f"ST{index}"
                 index += 1
             fields.append(f"pub {declaration}: {typename},")
 
+        for b in node.body:
+            if isinstance(b, ast.FunctionDef):
+                b.self_type = node.name
+
         extension = "#[pyclass]\n" if self.extension else ""
         struct_def = "pub struct {0} {{\n{1}\n}}\n\n".format(
             node.name, "\n".join(fields)
@@ -591,7 +603,7 @@ class RustTranspiler(CLikeTranspiler):
         buf_str = "\n".join(buf)
         return f"{extension}{struct_def}{impl_def}{buf_str} \n}}"
 
-    def visit_IntEnum(self, node):
+    def visit_IntEnum(self, node) -> str:
         fields = []
         for member, var in node.class_assignments.items():
             var = self.visit(var)
@@ -602,9 +614,9 @@ class RustTranspiler(CLikeTranspiler):
         fields = "\n".join(fields)
         return f"#[derive(Clone, Eq, Hash, PartialEq)]\npub enum {node.name} {{\n{fields}\n}}\n\n"
 
-    def visit_StrEnum(self, node):
+    def visit_StrEnum(self, node) -> str:
         self._usings.add("strum")
-        self._usings.add("strum_macros::{Display, EnumString, EnumVariantNames}")
+        self._usings.add("strum_macros::{Display, EnumString, VariantNames}")
 
         fields = []
         for member, var in node.class_assignments.items():
@@ -615,9 +627,9 @@ class RustTranspiler(CLikeTranspiler):
                 fields.append(f"#[strum(serialize = {var})]{member},")
         fields = "\n".join(fields)
 
-        return f"#[derive(Clone, Debug, Display, EnumString, EnumVariantNames, Eq, Hash, PartialEq)]\npub enum {node.name} {{\n{fields}\n}}\n\n"
+        return f"#[derive(Clone, Debug, Display, EnumString, VariantNames, Eq, Hash, PartialEq)]\npub enum {node.name} {{\n{fields}\n}}\n\n"
 
-    def visit_IntFlag(self, node):
+    def visit_IntFlag(self, node) -> str:
         self._usings.add("flagset::flags")
         self._usings.add("std::os::raw::c_int")
         fields = []
@@ -637,10 +649,13 @@ class RustTranspiler(CLikeTranspiler):
             self._usings.add(name)
         return ""
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         if module_name in self._rust_ignored_module_set:
             return ""
-        self._usings.add(module_name)
+        if level > 0:
+            self._rust_mods.add(module_name)
+        else:
+            self._usings.add(module_name)
         if len(names) == 1:
             # TODO: make this more generic so it works for len(names) > 1
             name = names[0]
@@ -652,24 +667,26 @@ class RustTranspiler(CLikeTranspiler):
         names = ", ".join(names)
         return f"use {module_name}::{{{names}}};"
 
-    def visit_List(self, node):
+    def visit_List(self, node) -> str:
         self._usings.add("std::collections")
         if len(node.elts) > 0:
             elements = [self.visit(e) for e in node.elts]
-            return "vec![{0}]".format(", ".join(elements))
+            return "vec![{}]".format(", ".join(elements))
 
         else:
             return "vec![]"
 
-    def visit_Dict(self, node):
+    def visit_Dict(self, node) -> str:
         self._usings.add("std::collections::HashMap")
         if len(node.keys) > 0:
             self._usings.add("std::collections::HashMap")
             kv_string = []
             for i in range(len(node.keys)):
-                key = self.visit(node.keys[i])
+                # we hit this case when using d1 = {..., **d2}
+                # The generated code isn't quite right, but it fixes a test error
+                key = self.visit(node.keys[i]) if node.keys[i] else "_"
                 value = self.visit(node.values[i])
-                kv_string.append("({0}, {1})".format(key, value))
+                kv_string.append(f"({key}, {value})")
             initialization = "[{0}].iter().cloned().collect::<HashMap<_,_>>()"
             return initialization.format(", ".join(kv_string))
         else:
@@ -678,16 +695,16 @@ class RustTranspiler(CLikeTranspiler):
     def _cast(self, name: str, to) -> str:
         return f"{name} as {to}"
 
-    def visit_Subscript(self, node):
+    def visit_Subscript(self, node) -> str:
         value = self.visit(node.value)
         index = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
-            if value in self.CONTAINER_TYPE_MAP:
+            if value in self._container_type_map:
                 self._usings.add("std::collections")
-                value = self.CONTAINER_TYPE_MAP[value]
+                value = self._container_type_map[value]
             if value == "Tuple":
-                return "({0})".format(index)
-            return "{0}<{1}>".format(value, index)
+                return f"({index})"
+            return f"{value}<{index}>"
         # TODO: optimize this. We need to compute value_type once per definition
         self._generic_typename_from_annotation(node.value)
         if hasattr(node.value, "annotation"):
@@ -703,12 +720,12 @@ class RustTranspiler(CLikeTranspiler):
                 index_typename = get_inferred_rust_type(self._slice_value(node))
                 if index_typename == value_type[1][0]:
                     index = "&" + index
-        return "{0}[{1}]".format(value, index)
+        return f"{value}[{index}]"
 
-    def visit_Index(self, node):
+    def visit_Index(self, node) -> str:
         return self.visit(node.value)
 
-    def visit_Slice(self, node):
+    def visit_Slice(self, node) -> str:
         lower = ""
         if node.lower:
             lower = self.visit(node.lower)
@@ -716,41 +733,17 @@ class RustTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Tuple(self, node):
+    def visit_Tuple(self, node) -> str:
         elts = [self.visit(e) for e in node.elts]
         elts = ", ".join(elts)
         if hasattr(node, "is_annotation"):
             return elts
-        return "({0})".format(elts)
+        return f"({elts})"
 
-    def visit_Try(self, node, finallybody=None):
-        # buf = self.visit_unsupported_body(node, "try_dummy", node.body)
-        buf = ["if true {"]
-        buf += [self.visit(n) for n in node.body]
-
-        for handler in node.handlers:
-            buf += self.visit(handler)
-        # buf.append("\n".join(excepts));
-
-        if finallybody:
-            buf += [self.visit(n) for n in finallybody]
-
-        buf += ["}"]
-        return "\n".join(buf)
-
-    def visit_ExceptHandler(self, node):
-        exception_type = ""
-        if node.type:
-            exception_type = self.visit(node.type)
-        name = "// except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(node, name, node.body)
-        body = [self.comment(b) for b in body]
-        return body
-
-    def visit_Assert(self, node):
-        return "assert!({0});".format(self.visit(node.test))
+    def visit_Assert(self, node) -> str:
+        return f"assert!({self.visit(node.test)});"
 
     def _compute_kw(self, node, target) -> str:
         kw = "let"
@@ -763,7 +756,7 @@ class RustTranspiler(CLikeTranspiler):
             kw = "let mut"
         return kw
 
-    def visit_AnnAssign(self, node):
+    def visit_AnnAssign(self, node) -> str:
         target, type_str, val = super().visit_AnnAssign(node)
         kw = self._compute_kw(node, node.target)
         return f"{kw} {target}: {type_str} = {val};"
@@ -783,7 +776,7 @@ class RustTranspiler(CLikeTranspiler):
         # python/rust annotations provided to customize the cast if necessary
         return f"{value_str} as {cast_to}"
 
-    def visit_AugAssign(self, node):
+    def visit_AugAssign(self, node) -> str:
         target = node.target
         target_str = self.visit(node.target)
         op = self.visit(node.op)
@@ -797,7 +790,7 @@ class RustTranspiler(CLikeTranspiler):
             )
         return f"{target_str} {op}= {value};"
 
-    def _visit_AssignOne(self, node, target):
+    def _visit_AssignOne(self, node, target) -> str:
         kw = self._compute_kw(node, target)
 
         if isinstance(node.scopes[-1], ast.If):
@@ -805,14 +798,14 @@ class RustTranspiler(CLikeTranspiler):
             target_id = self.visit(target)
             if target_id in outer_if.common_vars:
                 value = self.visit(node.value)
-                return "{0} = {1};".format(target_id, value)
+                return f"{target_id} = {value};"
 
         if isinstance(target, ast.Subscript) or isinstance(target, ast.Attribute):
             target = self.visit(target)
             value = self.visit(node.value)
             if value == None:
                 value = "None"
-            return "{0} = {1};".format(target, value)
+            return f"{target} = {value};"
 
         definition = node.scopes.parent_scopes.find(get_id(target))
         if definition is None:
@@ -883,26 +876,26 @@ class RustTranspiler(CLikeTranspiler):
             )
             return f"{kw} {target_str}{optional_typename} = {value};"
 
-    def visit_Delete(self, node):
+    def visit_Delete(self, node) -> str:
         target = node.targets[0]
         target_str = self.visit(target)
         return f"drop({target_str});"
 
-    def visit_Raise(self, node):
+    def visit_Raise(self, node) -> str:
         if node.exc is not None:
-            return "raise!({0}); //unsupported".format(self.visit(node.exc))
+            return f"raise!({self.visit(node.exc)}); //unsupported"
         # This handles the case where `raise` is used without
         # specifying the exception.
         return "raise!(); //unsupported"
 
-    def visit_Await(self, node):
+    def visit_Await(self, node) -> str:
         value = self.visit(node.value)
         return f"{value}.await"
 
-    def visit_AsyncFunctionDef(self, node):
+    def visit_AsyncFunctionDef(self, node) -> str:
         return self.visit_FunctionDef(node, async_prefix="async ")
 
-    def visit_Yield(self, node):
+    def visit_Yield(self, node) -> str:
         self._features.add("generators")
         self._features.add("generator_trait")
         self._usings.add("std::ops::Generator")
@@ -913,14 +906,14 @@ class RustTranspiler(CLikeTranspiler):
         else:
             return "yield None;"
 
-    def visit_Print(self, node):
+    def visit_Print(self, node) -> str:
         buf = []
         for n in node.values:
             value = self.visit(n)
-            buf.append('println!("{{:?}}",{0});'.format(value))
+            buf.append(f'println!("{{:?}}",{value});')
         return "\n".join(buf)
 
-    def visit_GeneratorExp(self, node):
+    def visit_GeneratorExp(self, node) -> str:
         elt = self.visit(node.elt)
         generator = node.generators[0]
         target = self.visit(generator.target)
@@ -935,25 +928,25 @@ class RustTranspiler(CLikeTranspiler):
         if not (iter.endswith("keys()") or iter.endswith("values()")) and not is_range:
             iter += ".iter()"
 
-        map_str = ".map(|{0}| {1})".format(target, elt)
+        map_str = f".map(|{target}| {elt})"
         filter_str = ""
         if generator.ifs:
-            filter_str = ".cloned().filter(|&{0}| {1})".format(
+            filter_str = ".cloned().filter(|&{}| {})".format(
                 target, self.visit(generator.ifs[0])
             )
 
-        return "{0}{1}{2}.collect::<Vec<_>>()".format(iter, filter_str, map_str)
+        return f"{iter}{filter_str}{map_str}.collect::<Vec<_>>()"
 
-    def visit_ListComp(self, node):
+    def visit_ListComp(self, node) -> str:
         return self.visit_GeneratorExp(node)  # right now they are the same
 
-    def visit_Global(self, node):
-        return "//global {0}".format(", ".join(node.names))
+    def visit_Global(self, node) -> str:
+        return "//global {}".format(", ".join(node.names))
 
-    def visit_Starred(self, node):
-        return "starred!({0})/*unsupported*/".format(self.visit(node.value))
+    def visit_Starred(self, node) -> str:
+        return f"starred!({self.visit(node.value)})/*unsupported*/"
 
-    def visit_Set(self, node):
+    def visit_Set(self, node) -> str:
         self._usings.add("std::collections::HashSet")
         elts = []
         for i in range(len(node.elts)):
@@ -966,8 +959,29 @@ class RustTranspiler(CLikeTranspiler):
         else:
             return "HashSet::new()"
 
-    def visit_IfExp(self, node):
+    def visit_IfExp(self, node) -> str:
         body = self.visit(node.body)
         orelse = self.visit(node.orelse)
         test = self.visit(node.test)
         return f"if {test} {{ {body} }} else {{ {orelse} }}"
+
+    def visit_Try(self, node, finallybody=None) -> str:
+        super().visit_Try(node, finallybody)
+        # if we got here, parent didn't throw. This can only
+        # be because of --no-strict
+        self._features.add("try_blocks")
+        buf = ["let result: Result<_, std::error::Error> = try { "]
+        body = "\n".join([self.visit(n) for n in node.body])
+        buf.append(body)
+        buf.append("};")
+
+        for handler in node.handlers:
+            buf.append("//" + self.visit(handler))
+
+        if finallybody:
+            buf.append("//" + self.visit(finallybody))
+
+        return "\n".join(buf)
+
+    def visit_ExceptHandler(self, node) -> str:
+        return "unsupported exception handler"
