@@ -1,17 +1,21 @@
 import ast
 import string
-from typing import Dict, List, Optional, Set
+from typing import Dict, List, Optional, Set, Tuple, Union
 
-from py2many.tracer import is_list, defined_before
-from py2many.exceptions import AstNotImplementedError
 from py2many.analysis import get_id, is_mutable, is_void_function
+from py2many.ast_helpers import create_ast_node
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.exceptions import AstNotImplementedError
+from py2many.tracer import defined_before, is_list
 
 from .clike import CLikeTranspiler
 from .inference import V_WIDTH_RANK
 from .plugins import (
     ATTR_DISPATCH_TABLE,
-    FUNC_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
     DISPATCH_MAP,
+    FUNC_DISPATCH_TABLE,
     SMALL_DISPATCH_MAP,
     SMALL_USINGS_MAP,
 )
@@ -25,6 +29,16 @@ def is_mutable(scopes, target: str) -> bool:
     return _is_mutable(scopes, target)
 
 
+_create_ast_node = create_ast_node
+
+
+def create_ast_node(code, at_node=None) -> ast.AST:
+    res = _create_ast_node(code, at_node=at_node)
+    if isinstance(res, ast.Expr):
+        res = res.value
+    return res
+
+
 def is_dict(node: ast.AST) -> bool:
     if isinstance(node, (ast.Dict, ast.DictComp)):
         return True
@@ -49,17 +63,63 @@ class VDictRewriter(ast.NodeTransformer):
         if (
             isinstance(node.func, ast.Attribute) and node.func.attr == "values"
         ):  # and is_dict(node.func.value):
-            new_node: ast.Call = ast.parse("a.keys().map(a[it])").body[0].value
+            new_node: ast.Call = create_ast_node("a.keys().map(a[it])", at_node=node)
             new_node.func.value.func.value = node.func.value
             new_node.args[0].value = node.func.value
-            new_node.lineno = node.lineno
-            new_node.col_offset = node.col_offset
-            ast.fix_missing_locations(new_node)
-            node = new_node
-
+            return new_node
         return node
 
 
+class VComprehensionRewriter(ast.NodeTransformer):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.redirects: Dict[str, str] = {}
+
+    def visit_Name(self, node: ast.Name) -> Union[ast.Name, ast.Subscript]:
+        if node.id in self.redirects:
+            return create_ast_node(self.redirects[node.id], at_node=node)
+        return node
+
+    def visit_GeneratorExp(self, node: ast.GeneratorExp) -> ast.Call:
+        new_node = None
+        for comp in node.generators:
+            if isinstance(comp.target, ast.Name):
+                self.redirects[comp.target.id] = "it"
+            elif isinstance(comp.target, ast.Tuple):
+                for idx, elem in enumerate(comp.target.elts):
+                    assert isinstance(elem, ast.Name)
+                    self.redirects[elem.id] = f"it[{idx}]"
+            else:
+                raise AstNotImplementedError(
+                    f"Unknown target type {type(node.target).__qualname__}", node
+                )
+
+            subnode = comp.iter
+
+            for cmp in comp.ifs:
+                chain = create_ast_node("placeholder.filter(placeholder)", at_node=node)
+                chain.func.value = subnode
+                chain.args[0] = cmp
+                subnode = chain
+
+            chain = create_ast_node("placeholder.map(placeholder)", at_node=node)
+            chain.func.value = subnode
+            chain.args[0] = node.elt
+            subnode = chain
+
+            if new_node is None:
+                new_node = subnode
+            else:
+                new_node.args[0] = subnode
+
+        self.visit(new_node)
+        self.redirects.clear()
+        return new_node
+
+    def visit_ListComp(self, node: ast.ListComp) -> ast.Call:
+        return self.visit_GeneratorExp(node)
+
+
 class VNoneCompareRewriter(ast.NodeTransformer):
     def visit_Compare(self, node: ast.Compare):
         left: ast.AST = self.visit(node.left)
@@ -78,21 +138,13 @@ class VNoneCompareRewriter(ast.NodeTransformer):
 class VTranspiler(CLikeTranspiler):
     NAME: str = "v"
 
-    CONTAINER_TYPE_MAP: Dict[str, str] = {
-        "List": "[]",
-        "Dict": "map",
-        "Set": "set",
-        "Optional": "?",
-    }
-
     ALLOW_MODULE_LIST: List[str] = ["math"]
 
     def __init__(self, indent: int = 2):
         super().__init__()
-        self._headers = set([])
+        self._headers = set()
         self._indent = " " * indent
-        self._default_type = "any"
-        self._container_type_map = self.CONTAINER_TYPE_MAP
+        CLikeTranspiler._default_type = "any"
         self._dispatch_map = DISPATCH_MAP
         self._small_dispatch_map = SMALL_DISPATCH_MAP
         self._small_usings_map = SMALL_USINGS_MAP
@@ -107,9 +159,10 @@ class VTranspiler(CLikeTranspiler):
         uses: str = "\n".join(f"import {mod}" for mod in usings)
         # Module statement needs to be here as uses is applied to the top of the file
         # but V expects the module statement to be at the top.
-        return f"module main\n{uses}"
+        return f"[translated]\nmodule main\n{uses}"
 
-    def _combine_value_index(self, value_type: str, index_type: str) -> str:
+    @classmethod
+    def _combine_value_index(cls, value_type, index_type) -> str:
         return f"{value_type}{index_type}"
 
     def comment(self, text: str) -> str:
@@ -119,49 +172,68 @@ class VTranspiler(CLikeTranspiler):
         # Suppress all imports for now until a reliable way to differentiate submodule imports is used.
         return ""
 
-    def _import_from(self, module_name: str, names: List[str]) -> str:
+    def _import_from(self, module_name: str, names: List[str], level: int = 0) -> str:
         # Suppress all imports for now until a reliable way to differentiate submodule imports is used.
         return ""  # f"import {module_name} {{{' '.join(names)}}}"
 
-    def function_signature(self, node: ast.FunctionDef) -> str:
+    def visit_arg(self, node) -> Tuple[Optional[str], str]:
+        id = get_id(node)
+        if id == "self":
+            return (None, "self")
+        typename = ""
+        if node.annotation:
+            typename = self._typename_from_annotation(node)
+        if is_mutable(node.scopes, id):
+            id = f"mut {id}"
+        return (typename, id)
+
+    def visit_FunctionDef(self, node) -> str:
         signature = ["fn"]
-        if node.scopes[-1] is ast.ClassDef:
-            raise AstNotImplementedError("Class methods are not supported yet.", node)
+        is_class_method: bool = False
+        if (
+            node.scopes is not None
+            and len(node.scopes) > 1
+            and isinstance(node.scopes[-2], ast.ClassDef)
+        ):
+            is_class_method = True
 
-        if name := get_id(node):
-            signature.append(name)
-
-        args: List[str] = []
-        generic_count: int = 0
+        generics: Set[str] = set()
+        args: List[Tuple[str, str]] = []
+        receiver: str = ""
         for arg in node.args.args:
-            typename: str = string.ascii_uppercase[generic_count]
-            id: str = get_id(arg)
-            if is_mutable(node.scopes, id):
-                id = f"mut {id}"
-            if getattr(arg, "annotation", None):
-                typename = self._typename_from_annotation(arg, attr="annotation")
-            if len(typename) == 1 and typename.isupper():
-                generic_count += 1
-            args.append(f"{id} {typename}")
+            typename, id = self.visit(arg)
+            if typename is None and is_class_method:  # receiver
+                receiver = id
+                continue
+            elif len(typename) == 1 and typename.isupper():
+                generics.add(typename)
+            args.append((typename, id))
 
-        if generic_count:
-            signature.append(f"<{', '.join(string.ascii_uppercase[:generic_count])}>")
+        if is_class_method:
+            signature.append(f"({receiver} {get_id(node.scopes[-2])})")
+        signature.append(node.name)
 
-        signature.append(f"({', '.join(args)})")
-        if isinstance(node, ast.Lambda):
-            if getattr(node, "annotation", None):
-                typename: str = self._typename_from_annotation(node, attr="annotation")
-                signature.append(typename)
-        elif not is_void_function(node):
-            if getattr(node, "returns", None):
-                typename: str = self._typename_from_annotation(node, attr="returns")
-                signature.append(typename)
+        str_args: List[str] = []
+        for typename, id in args:
+            if typename == "":
+                for c in string.ascii_uppercase:
+                    if c not in generics:
+                        generics.add(c)
+                        typename = c
+                        break
+            if typename == "":
+                raise AstNotImplementedError(
+                    "Cannot use more than 26 generics in a function.", node
+                )
 
-        return " ".join(signature)
+            str_args.append(f"{id} {typename}")
+        signature.append(f"({', '.join(str_args)})")
+
+        if not is_void_function(node):
+            signature.append(self._typename_from_annotation(node, attr="returns"))
 
-    def visit_FunctionDef(self, node):
         body = "\n".join([self.indent(self.visit(n)) for n in node.body])
-        return f"{self.function_signature(node)} {{\n{body}\n}}"
+        return f"{' '.join(signature)} {{\n{body}\n}}"
 
     def visit_Return(self, node: ast.Return) -> str:
         if node.value:
@@ -170,7 +242,7 @@ class VTranspiler(CLikeTranspiler):
         return "return"
 
         if node.value:
-            return "return {0}".format(self.visit(node.value))
+            return f"return {self.visit(node.value)}"
         return "return"
 
     def visit_Lambda(self, node: ast.Lambda) -> str:
@@ -204,7 +276,7 @@ class VTranspiler(CLikeTranspiler):
                 value: str = self.visit(kw.value)
                 vargs += [f"{kw.arg}: {value}"]
         args: str = ", ".join(vargs)
-        return f"{fname}({args})"
+        return f"{fname}{{{args}}}"
 
     def visit_Call(self, node: ast.Call) -> str:
         fname: str = self.visit(node.func)
@@ -213,17 +285,15 @@ class VTranspiler(CLikeTranspiler):
         if isinstance(fndef, ast.ClassDef):
             return self._visit_object_literal(node, fname, fndef)
 
-        if isinstance(node.func, ast.Attribute) and isinstance(
-            node.func.value, (ast.Dict, ast.DictComp)
-        ):
-            raise AstNotImplementedError(
-                "Cannot call methods on dict literals yet in V.", node
-            )  # https://github.com/vlang/v/issues/10780
-
         vargs: List[str] = []
 
-        if node.args:
-            vargs.extend(map(self.visit, node.args))
+        for idx, arg in enumerate(node.args):
+            if hasattr(fndef, "args") and is_mutable(
+                fndef.scopes, fndef.args.args[idx].arg
+            ):
+                vargs.append(f"mut {self.visit(arg)}")
+            else:
+                vargs.append(self.visit(arg))
         if node.keywords:
             vargs += [self.visit(kw.value) for kw in node.keywords]
 
@@ -282,8 +352,8 @@ class VTranspiler(CLikeTranspiler):
         return f"[{', '.join(chars)}]"
 
     def visit_If(self, node: ast.If) -> str:
-        body_vars: Set[str] = set([get_id(v) for v in node.scopes[-1].body_vars])
-        orelse_vars: Set[str] = set([get_id(v) for v in node.scopes[-1].orelse_vars])
+        body_vars: Set[str] = {get_id(v) for v in node.scopes[-1].body_vars}
+        orelse_vars: Set[str] = {get_id(v) for v in node.scopes[-1].orelse_vars}
         node.common_vars = body_vars.intersection(orelse_vars)
 
         body: str = "\n".join(
@@ -309,14 +379,53 @@ class VTranspiler(CLikeTranspiler):
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Call, ast.Num)):
                 # Shortcut if parenthesis are not needed
-                return "-{0}".format(self.visit(node.operand))
+                return f"-{self.visit(node.operand)}"
             else:
-                return "-({0})".format(self.visit(node.operand))
+                return f"-({self.visit(node.operand)})"
         else:
             return super().visit_UnaryOp(node)
 
-    def visit_ClassDef(self, node: ast.ClassDef) -> str:
-        raise AstNotImplementedError("Classes are not supported yet.", node)
+    def visit_ClassDef(self, node) -> str:
+        extractor = DeclarationExtractor(VTranspiler())
+        extractor.visit(node)
+        node.declarations = declarations = extractor.get_declarations()
+        node.declarations_with_defaults = extractor.get_declarations_with_defaults()
+        node.class_assignments = extractor.class_assignments
+        ret = super().visit_ClassDef(node)
+        if ret is not None:
+            return ret
+
+        decorators = [get_id(d) for d in node.decorator_list]
+
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
+        fields = []
+        if declarations:
+            fields.append("pub mut:")
+        index = 0
+        for declaration, typename in declarations.items():
+            if typename == None:
+                typename = f"ST{index}"
+                index += 1
+            fields.append(f"{declaration} {typename}")
+
+        for b in node.body:
+            if isinstance(b, ast.FunctionDef):
+                b.self_type = node.name
+
+        struct_def = "pub struct {0} {{\n{1}\n}}\n\n".format(
+            node.name, "\n".join(fields)
+        )
+        buf = [self.visit(b) for b in node.body]
+        buf_str = "\n".join(buf)
+        return f"{struct_def}{buf_str}"
 
     def visit_IntEnum(self, node: ast.ClassDef) -> str:
         raise AstNotImplementedError("Enums are not supported yet.", node)
@@ -339,14 +448,14 @@ class VTranspiler(CLikeTranspiler):
         keys: List[str] = [self.visit(k) for k in node.keys]
         values: List[str] = [self.visit(k) for k in node.values]
         kv_pairs: str = " ".join([f"{k}: {v}" for k, v in zip(keys, values)])
-        return f"map{{{kv_pairs}}}"
+        return f"{{{kv_pairs}}}"
 
     def visit_Subscript(self, node: ast.Subscript) -> str:
         value: str = self.visit(node.value)
         index: str = self.visit(node.slice)
         if hasattr(node, "is_annotation"):
-            if value in self.CONTAINER_TYPE_MAP:
-                value = self.CONTAINER_TYPE_MAP[value]
+            if value in self._container_type_map:
+                value = self._container_type_map[value]
             if value == "Tuple":
                 return f"({index})"
             return f"{value}[{index}]"
@@ -363,9 +472,9 @@ class VTranspiler(CLikeTranspiler):
         if node.upper:
             upper = self.visit(node.upper)
 
-        return "{0}..{1}".format(lower, upper)
+        return f"{lower}..{upper}"
 
-    def visit_Elipsis(self, node):
+    def visit_Elipsis(self, node) -> str:
         return ""
 
     def visit_Tuple(self, node: ast.Tuple) -> str:
@@ -373,10 +482,45 @@ class VTranspiler(CLikeTranspiler):
         return self.visit_List(node)
 
     def visit_Try(self, node: ast.Try, finallybody: bool = None) -> str:
-        raise AstNotImplementedError("Exceptions are not supported yet.", node)
+        self._usings.add("div72.vexc")
+        buf = []
+        buf.append("if C.try() {")
+        buf.extend(map(self.visit, node.body))
+        buf.append("vexc.end_try()")
+        buf.append("}")
+        if len(node.handlers) == 1 and not node.handlers[0].type:
+            # Just except:
+            buf.append("else {")
+            buf.extend(map(self.visit, node.handlers[0].body))
+            buf.append("}")
+        elif node.handlers:
+            buf.append("else {")
+            buf.append("match vexc.get_curr_exc().name {")
+            has_else: bool = False
+            for handler in node.handlers:
+                buf2 = self.visit(handler)
+                if buf2.startswith("else"):
+                    has_else = True
+                buf.append(buf2)
+            if not has_else:
+                buf.append("else {}")
+            buf.append("}")
+            buf.append("}")
+        if node.finalbody:
+            buf.extend(map(self.visit, node.finalbody))
+        return "\n".join(buf)
 
     def visit_ExceptHandler(self, node) -> str:
-        raise AstNotImplementedError("Exceptions are not supported yet.", node)
+        buf = []
+        if node.type:
+            buf.append(f"'{get_id(node.type)}' {{")
+        else:
+            buf.append("else {")
+        if node.name:
+            buf.append(f"{node.name} := vexc.get_curr_exc()")
+        buf.extend(map(self.visit, node.body))
+        buf.append("}")
+        return "\n".join(buf)
 
     def visit_Assert(self, node: ast.Assert) -> str:
         return f"assert {self.visit(node.test)}"
@@ -449,13 +593,22 @@ class VTranspiler(CLikeTranspiler):
         target: str = self.visit(node.target)
         op: str = self.visit(node.op)
         val: str = self.visit(node.value)
-        return "{0} {1}= {2}".format(target, op, val)
+        return f"{target} {op}= {val}"
 
     def visit_Delete(self, node: ast.Delete) -> str:
         raise AstNotImplementedError("`delete` statements are not supported yet.", node)
 
     def visit_Raise(self, node: ast.Raise) -> str:
-        raise AstNotImplementedError("Exceptions are not supported yet.", node)
+        self._usings.add("div72.vexc")
+        name: str = f'"{get_id(node.exc)}"'
+        msg: str = '""'
+        if node.exc is None:
+            return "vexc.raise('Exception', '')"
+        elif isinstance(node.exc, ast.Call):
+            name = f'"{get_id(node.exc.func)}"'
+            if node.exc.args:
+                msg = self.visit(node.exc.args[0])
+        return f"vexc.raise({name}, {msg})"
 
     def visit_With(self, node: ast.With) -> str:
         raise AstNotImplementedError("`with` statements are not supported yet.", node)
@@ -474,7 +627,7 @@ class VTranspiler(CLikeTranspiler):
 
     def visit_GeneratorExp(self, node: ast.GeneratorExp) -> str:
         raise AstNotImplementedError(
-            "Generator expressions are not supported yet.", node
+            "Comprehensions should have been handled in the rewriter.", node
         )
 
     def visit_ListComp(self, node: ast.ListComp) -> str:
