@@ -9,16 +9,16 @@ def show():
     finally:
         print("Finally")
 
-    try:
-        3 / 0
-    except ZeroDivisionError:
-        print("OK")
-
     try:
         raise Exception("foo")
     except:
         print("Got it")
 
+    try:
+        raise Exception("foo")
+    except Exception as e:
+        assert "foo" in str(e)
+
 
 if __name__ == "__main__":
     show()
