@@ -1,3 +1,6 @@
+#!/usr/bin/env python3
+
+
 def fib(i: int) -> int:
     if i == 0 or i == 1:
         return 1
@@ -5,7 +8,4 @@ def fib(i: int) -> int:
 
 
 if __name__ == "__main__":
-	# Using a variable to workaround
-	# https://github.com/adsharma/py2many/issues/64
-	rv = fib(5)
-	print(rv)
+    print(fib(5))
