@@ -1,4 +1,7 @@
-fun compare_assert(a: Int, b: Int) {
+fun compare_assert(
+    a: Int,
+    b: Int,
+) {
     assert(a == b)
     assert(!(0 == 1))
 }
