@@ -1,10 +1,9 @@
 #include <cassert>   // NOLINT(build/include_order)
+#include <cstdint>   // NOLINT(build/include_order)
 #include <iostream>  // NOLINT(build/include_order)
+#include <string>    // NOLINT(build/include_order)
 #include <vector>    // NOLINT(build/include_order)
 
-#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
-#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
-
 inline void main_func() {
   std::vector<bool> ands = {};
   std::vector<bool> ors = {};
@@ -32,7 +31,4 @@ inline void main_func() {
   std::cout << std::endl;
 }
 
-int main(int argc, char** argv) {
-  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
-  main_func();
-}
+int main(int argc, char** argv) { main_func(); }
