@@ -11,7 +11,7 @@ def bubble_sort(seq: List[int]) -> List[int]:
         for n in range(1, L):
             if seq[n] < seq[n - 1]:
                 if True:
-                    (__tmp1, __tmp2) = (seq[n], seq[n - 1])
+                    __tmp1, __tmp2 = (seq[n], seq[n - 1])
                     seq[n - 1] = __tmp1
                     seq[n] = __tmp2
     return seq
