@@ -1,4 +1,4 @@
-// @dart=2.9
+// @dart=3.4
 import 'package:sprintf/sprintf.dart';
 
 show() {
@@ -9,16 +9,16 @@ show() {
   } finally {
     print(sprintf("%s", ["Finally"]));
   }
-  try {
-    (3 ~/ 0);
-  } on IntegerDivisionByZeroException {
-    print(sprintf("%s", ["OK"]));
-  }
   try {
     throw new Exception("foo");
   } catch (e) {
     print(sprintf("%s", ["Got it"]));
   }
+  try {
+    throw new Exception("foo");
+  } on Exception catch (e) {
+    assert(e.toString().contains("foo"));
+  }
 }
 
 main(List<String> argv) {
