@@ -1,7 +1,5 @@
+#include <cstdint>   // NOLINT(build/include_order)
 #include <iostream>  // NOLINT(build/include_order)
-
-#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
-#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 inline int fib(int i) {
   if (i == 0 || i == 1) {
     return 1;
@@ -10,7 +8,6 @@ inline int fib(int i) {
 }
 
 int main(int argc, char** argv) {
-  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   std::cout << fib(5);
   std::cout << std::endl;
 }
