@@ -1,15 +1,22 @@
 //! ```cargo
 //! [package]
-//! edition = "2018"
+//! edition = "2021"
 //! [dependencies]
 //! anyhow = "*"
 //! lazy_static = "*"
 //! ```
 
+#![allow(clippy::assertions_on_constants)]
+#![allow(clippy::bool_comparison)]
 #![allow(clippy::collapsible_else_if)]
+#![allow(clippy::comparison_to_empty)]
 #![allow(clippy::double_parens)] // https://github.com/adsharma/py2many/issues/17
+#![allow(clippy::eq_op)]
+#![allow(clippy::let_with_type_underscore)]
 #![allow(clippy::map_identity)]
 #![allow(clippy::needless_return)]
+#![allow(clippy::nonminimal_bool)]
+#![allow(clippy::partialeq_to_none)]
 #![allow(clippy::print_literal)]
 #![allow(clippy::ptr_arg)]
 #![allow(clippy::redundant_static_lifetimes)] // https://github.com/adsharma/py2many/issues/266
@@ -44,7 +51,7 @@ lazy_static! {
         .collect::<HashMap<_, _>>();
 }
 pub fn main() -> Result<()> {
-    assert!(l_b.iter().any(|&x| x == "a"));
+    assert!(l_b.contains(&"a"));
     println!("{}", "OK");
     Ok(())
 }
