@@ -1,9 +1,7 @@
+#include <cstdint>   // NOLINT(build/include_order)
 #include <iostream>  // NOLINT(build/include_order)
-
-#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
-#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
+#include <string>    // NOLINT(build/include_order)
 int main(int argc, char** argv) {
-  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   std::cout << std::string{"Hello world!"};
   std::cout << std::endl;
   std::cout << std::string{"Hello"};
