@@ -1,6 +1,6 @@
 
 function show()
-    myfunc = (x, y) -> (x + y)
+    myfunc = (x, y) -> x + y
     println(join([myfunc(1, 2)], " "))
 end
 
