@@ -1,5 +1,5 @@
 function for_with_break()
-    for i in (0:4-1)
+    for i = 0:4-1
         if i == 2
             break
         end
@@ -8,7 +8,7 @@ function for_with_break()
 end
 
 function for_with_continue()
-    for i in (0:4-1)
+    for i = 0:4-1
         if i == 2
             continue
         end
@@ -17,9 +17,13 @@ function for_with_continue()
 end
 
 function for_with_else()
-    for i in (0:4-1)
+    has_break = false
+    for i = 0:4-1
         println(join([i], " "))
     end
+    if has_break != true
+        println(join(["OK"], " "))
+    end
 end
 
 function while_with_break()
@@ -47,6 +51,7 @@ end
 function main()
     for_with_break()
     for_with_continue()
+    for_with_else()
     while_with_break()
     while_with_continue()
 end
