@@ -1,10 +1,9 @@
 #include <algorithm>  // NOLINT(build/include_order)
+#include <cstdint>    // NOLINT(build/include_order)
 #include <iostream>   // NOLINT(build/include_order)
 #include <map>        // NOLINT(build/include_order)
+#include <string>     // NOLINT(build/include_order)
 #include <vector>     // NOLINT(build/include_order)
-
-#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
-#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 inline bool nested_containers() {
   std::map<std::string, std::vector<int>> CODES =
       std::map<std::string, std::vector<int>>{{std::string{"KEY"}, {1, 3}}};
@@ -14,7 +13,6 @@ inline bool nested_containers() {
 }
 
 int main(int argc, char** argv) {
-  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   if (nested_containers()) {
     std::cout << std::string{"OK"};
     std::cout << std::endl;
