@@ -1,9 +1,10 @@
-/* This file implements a rectangle class  */
+// This file implements a rectangle class
 
-data class Rectangle(val height: Int, val length: Int) {
-    fun is_square(): Boolean {
-        return height == length
-    }
+data class Rectangle(
+    val height: Int,
+    val length: Int,
+) {
+    fun is_square(): Boolean = height == length
 }
 
 fun show() {
