@@ -1,16 +1,15 @@
 import argparse
-import filecmp
 import logging
 import os.path
-import unittest
+import platform
 import sys
-
-from distutils import spawn
 from functools import lru_cache
+from itertools import product
 from pathlib import Path
 from subprocess import run
 from unittest.mock import Mock
-from unittest_expander import foreach, expand
+
+import pytest
 
 from py2many.cli import (
     _create_cmd,
@@ -19,6 +18,12 @@ from py2many.cli import (
     _relative_to_cwd,
     main,
 )
+from py2many.process_helpers import find_executable
+
+try:
+    from py2many.pycpp import _conan_include_args
+except ImportError:
+    from pycpp import _conan_include_args
 
 import py2many.cli
 
@@ -38,38 +43,64 @@ ENV = {
     "rust": {"RUSTFLAGS": "--deny warnings"},
 }
 COMPILERS = {
-    "cpp": [CXX, "-std=c++14", "-I", str(ROOT_DIR)]
-    + (["-stdlib=libc++"] if CXX == "clang++" else [])
+    "cpp": [CXX, "-std=c++17", "-I", str(ROOT_DIR)]
+    + _conan_include_args()
+    + (["-stdlib=libc++"] if CXX.startswith("clang++") else [])
     + (["-o", "{exe}", "{filename}"] if sys.platform == "win32" else []),
+    "dlang": ["dmd"],
     "dart": ["dart", "compile", "exe"],
     "go": ["go", "build"],
-    "kotlin": ["kotlinc"],
     "nim": ["nim", "compile", "--nimcache:."],
-    "rust": ["cargo", "script", "--build-only", "--debug"],
-    "vlang": ["v", "-translated"],
+    "rust": [
+        "../../scripts/rust-runner.sh",
+        "compile",
+    ],
+    "vlang": ["v"],
+    "mojo": ["mojo", "build"],
+    "zig": ["zig", "build-exe"],
 }
+
 INVOKER = {
     "dart": ["dart", "--enable-asserts"],
+    "dlang": ["dmd", "-run"],
     "go": ["go", "run"],
     "julia": ["julia", "--compiled-modules=yes"],
-    "kotlin": ["kscript"],
     "python": [sys.executable],
-    "rust": ["cargo", "script"],
-    "vlang": ["v", "-translated", "run"],
+    "rust": [
+        "../../scripts/rust-runner.sh",
+        "run",
+    ],
+    "smt": ["z3", "-smt2"],
+    "vlang": ["v", "run"],
+    "mojo": ["mojo"],
+    "zig": [
+        "../../scripts/zig-runner.sh",
+        "run",
+    ],
 }
 
-TEST_CASES = [
-    item.stem
-    for item in (TESTS_DIR / "cases").glob("*.py")
-    if not item.stem.startswith("test_") and not item.stem in ["generator"]
-]
+# kscript requires a KOTLIN_HOME.
+# If it isnt present, set compilation using kotlinc which should also be missing,
+# which will cause Kotlin cases to be skipped.
+if os.getenv("KOTLIN_HOME"):
+    INVOKER["kotlin"] = [
+        "jgo",
+        "--log-level=DEBUG",
+        "--additional-endpoints",
+        "commons-cli:commons-cli",
+        "commons-codec:commons-codec",
+        "io.github.kscripting:shell",
+        "io.github.kscripting:kscript:io.github.kscripting.kscript.KscriptKt",
+        platform.system().lower(),
+    ]
+else:
+    COMPILERS["kotlin"] = ["kotlinc"]
 
-CASE_ARGS = {
-    "sys_argv": ("arg1",),
-}
-CASE_EXPECTED_EXITCODE = {
-    "sys_exit": 1,
-}
+TEST_CASES = [item.stem for item in (TESTS_DIR / "cases").glob("*.py")]
+TEST_PARAMS = [(case, lang) for case, lang in product(TEST_CASES, LANGS)]
+
+CASE_ARGS = {"sys_argv": ("arg1",)}
+CASE_EXPECTED_EXITCODE = {"sys_exit": 1}
 
 EXTENSION_TEST_CASES = [
     item.stem
@@ -81,17 +112,11 @@ EXPECTED_LINT_FAILURES = []
 
 EXPECTED_COMPILE_FAILURES = []
 
-CARGO_TOML = [l.strip() for l in open(ROOT_DIR / "Cargo.toml").readlines()]
-
 a_dot_out = "a.out"
 
 logger = logging.Logger("test_cli")
 
 
-def in_cargo_toml(case: str):
-    return f'path = "tests/expected/{case}.rs"' in CARGO_TOML
-
-
 def has_main_lines(lines):
     return any("def main" in line or "__main__" in line for line in lines)
 
@@ -115,7 +140,11 @@ def get_exe_filename(case, ext):
 
 @lru_cache()
 def get_python_case_output(case_filename, main_args, exit_code):
-    proc = run([sys.executable, str(case_filename), *main_args], capture_output=True)
+    proc = run(
+        [sys.executable, str(case_filename), *main_args],
+        capture_output=True,
+        check=False,
+    )
     if exit_code:
         assert proc.returncode == exit_code
     elif proc.returncode:
@@ -126,7 +155,7 @@ def get_python_case_output(case_filename, main_args, exit_code):
 def standardise_python(code):
     """Ignore differences in black output.
 
-    black 21.6b0 outputs slightly different source between Python 3.8 amd 3.9
+    black outputs slightly different source between Python versions.
     For tuples, it is not consistent adding round brackets.
     And sometimes there are fewer blank newlines.
     """
@@ -135,8 +164,20 @@ def standardise_python(code):
     )
 
 
-@expand
-class CodeGeneratorTests(unittest.TestCase):
+def standardise_eol(code):
+    """Ignore differences in EOLs."""
+    if sys.platform != "win32":
+        return code
+    return code.replace("\r\n", "\n").replace("\n\n", "\n")
+
+
+def is_declarative(ext):
+    """The scripts in these languages can't be run. They declare some
+    constraints that could be verified later in the COMPILER. No INVOKER."""
+    return ext in {".smt"}
+
+
+class TestCodeGenerator:
     maxDiff = None
 
     SHOW_ERRORS = SHOW_ERRORS
@@ -144,13 +185,13 @@ class CodeGeneratorTests(unittest.TestCase):
     UPDATE_EXPECTED = UPDATE_EXPECTED
     LINT = os.environ.get("LINT", True)
 
-    def setUp(self):
+    @classmethod
+    def setup_class(cls):
         os.makedirs(BUILD_DIR, exist_ok=True)
         os.chdir(BUILD_DIR)
         py2many.cli.CWD = BUILD_DIR
 
-    @foreach(sorted(LANGS))
-    @foreach(sorted(TEST_CASES))
+    @pytest.mark.parametrize("case,lang", TEST_PARAMS)
     def test_generated(self, case, lang):
         env = os.environ.copy()
         if ENV.get(lang):
@@ -165,11 +206,11 @@ class CodeGeneratorTests(unittest.TestCase):
             and not self.KEEP_GENERATED
             and not os.path.exists(expected_filename)
         ):
-            raise unittest.SkipTest(f"{expected_filename} not found")
+            raise pytest.skip(f"{expected_filename} not found")
 
         if settings.formatter:
-            if not spawn.find_executable(settings.formatter[0]):
-                raise unittest.SkipTest(f"{settings.formatter[0]} not available")
+            if not find_executable(settings.formatter[0]):
+                raise pytest.skip(f"{settings.formatter[0]} not available")
 
         case_filename = TESTS_DIR / "cases" / f"{case}.py"
         case_output = GENERATED_DIR / f"{case}{ext}"
@@ -178,15 +219,17 @@ class CodeGeneratorTests(unittest.TestCase):
         exe.unlink(missing_ok=True)
 
         is_script = has_main(case_filename)
-        self.assertTrue(is_script)
+        if not is_script and not is_declarative(ext):
+            raise pytest.skip(f"{case} is declarative, not suitable for {lang}")
 
-        main_args = CASE_ARGS.get(case, tuple())
-        expected_exit_code = CASE_EXPECTED_EXITCODE.get(case, 0)
-        expected_output = get_python_case_output(
-            case_filename, main_args, expected_exit_code
-        )
-        self.assertTrue(expected_output, "Test cases must print something")
-        expected_output = expected_output.splitlines()
+        if not is_declarative(ext):
+            main_args = CASE_ARGS.get(case, tuple())
+            expected_exit_code = CASE_EXPECTED_EXITCODE.get(case, 0)
+            expected_output = get_python_case_output(
+                case_filename, main_args, expected_exit_code
+            )
+            assert expected_output, "Test cases must print something"
+            expected_output = expected_output.splitlines()
 
         args = [
             f"--{lang}=1",
@@ -204,12 +247,13 @@ class CodeGeneratorTests(unittest.TestCase):
                     with open(expected_filename) as f2:
                         expected_case_contents = f2.read()
                         generated_cleaned = generated
-                        if ext == ".py":
+                        # Only require equivalent output on latest Python version
+                        if ext == ".py" and sys.version_info < (3, 12, 0):
                             expected_case_contents = standardise_python(
                                 expected_case_contents
                             )
                             generated_cleaned = standardise_python(generated)
-                        self.assertEqual(expected_case_contents, generated_cleaned)
+                        assert expected_case_contents == generated_cleaned
                         print("expected = generated")
 
             expect_failure = (
@@ -219,48 +263,50 @@ class CodeGeneratorTests(unittest.TestCase):
             if not expect_failure:
                 assert rv == 0, "formatting failed"
             elif rv:
-                raise unittest.SkipTest("formatting failed")
+                raise pytest.skip("formatting failed")
 
             compiler = COMPILERS.get(lang)
             if compiler:
-                if not spawn.find_executable(compiler[0]):
-                    raise unittest.SkipTest(f"{compiler[0]} not available")
+                if not find_executable(compiler[0]):
+                    raise pytest.skip(f"{compiler[0]} not available")
                 expect_compile_failure = (
                     not self.SHOW_ERRORS and f"{case}{ext}" in EXPECTED_COMPILE_FAILURES
                 )
                 if expect_compile_failure:
                     return
                 cmd = _create_cmd(compiler, filename=case_output, exe=exe)
-                print(f"Running {cmd} ...")
-                proc = run(cmd, env=env, check=not expect_failure)
+                print(f"Compiling {cmd} ...")
+                proc = run(cmd, env=env, check=False)
 
-                if proc.returncode:
-                    raise unittest.SkipTest(f"{case}{ext} doesnt compile")
+                if proc.returncode and not expect_failure:
+                    raise pytest.skip(f"{case}{ext} doesnt compile")
 
                 if self.UPDATE_EXPECTED or not os.path.exists(expected_filename):
                     with open(expected_filename, "w") as f:
                         f.write(generated)
 
+            if is_declarative(ext):
+                return
+
             stdout = None
             if ext == ".cpp" and (BUILD_DIR / a_dot_out).exists():
                 os.rename(BUILD_DIR / a_dot_out, exe)
 
             if INVOKER.get(lang):
                 invoker = INVOKER.get(lang)
-                if not spawn.find_executable(invoker[0]):
-                    raise unittest.SkipTest(f"{invoker[0]} not available")
+                if os.path.exists(invoker[0]):
+                    pass
+                elif not find_executable(invoker[0]):
+                    raise pytest.skip(f"{invoker[0]} not available")
                 cmd = _create_cmd(invoker, filename=case_output, exe=exe)
                 cmd += main_args
-                proc = run(
-                    cmd,
-                    env=env,
-                    capture_output=True,
-                )
+                print(f"Invoking {cmd} ...")
+                proc = run(cmd, env=env, capture_output=True)
 
                 stdout = proc.stdout
 
                 if expect_failure and expected_exit_code != proc.returncode:
-                    raise unittest.SkipTest(f"Execution of {case}{ext} failed")
+                    raise pytest.skip(f"Execution of {case}{ext} failed")
                 assert (
                     expected_exit_code == proc.returncode
                 ), f"Execution of {case}{ext} failed:\n{stdout}{proc.stderr}"
@@ -278,23 +324,24 @@ class CodeGeneratorTests(unittest.TestCase):
             else:
                 raise RuntimeError(f"Compiled output {exe} not detected")
 
-            self.assertTrue(stdout, "Invoked code produced no stdout")
+            assert stdout, "Invoked code produced no stdout"
             stdout = stdout.splitlines()
-            self.assertEqual(expected_output, stdout)
+            assert expected_output == stdout
 
             if settings.linter and self.LINT:
-                if not spawn.find_executable(settings.linter[0]):
-                    raise unittest.SkipTest(f"{settings.linter[0]} not available")
+                if not find_executable(settings.linter[0]):
+                    raise pytest.skip(f"{settings.linter[0]} not available")
                 if settings.ext == ".kt" and case_output.is_absolute():
                     # KtLint does not support absolute path in globs
                     case_output = _relative_to_cwd(case_output)
                 linter = _create_cmd(settings.linter, case_output)
+                print(f"Running linter {linter} ...")
                 if ext == ".cpp":
                     linter.append("-Wno-unused-variable")
                     if case == "coverage":
                         linter.append(
                             "-Wno-null-arithmetic"
-                            if CXX == "clang++"
+                            if CXX.startswith("clang++")
                             else "-Wno-pointer-arith"
                         )
                 proc = run(linter, env=env)
@@ -302,8 +349,8 @@ class CodeGeneratorTests(unittest.TestCase):
                 if proc.returncode and linter[0] == "golint":
                     expect_failure = True
                 if proc.returncode and expect_failure:
-                    raise unittest.SkipTest(f"{case}{ext} failed linter")
-                self.assertFalse(proc.returncode)
+                    raise pytest.skip(f"{case}{ext} failed linter")
+                assert not proc.returncode
 
                 if expect_failure:
                     raise AssertionError(f"{case}{ext} passed unexpectedly")
@@ -312,10 +359,8 @@ class CodeGeneratorTests(unittest.TestCase):
             if not self.KEEP_GENERATED:
                 case_output.unlink(missing_ok=True)
                 exe.unlink(missing_ok=True)
-        if settings.ext == ".rs":
-            assert in_cargo_toml(case)
 
-    @foreach(sorted(TEST_CASES))
+    @pytest.mark.parametrize("case", TEST_CASES)
     # This test name must be alpha before `test_generated` otherwise
     # KEEP_GENERATED does not work.
     def test_env_cxx_gcc(self, case):
@@ -323,20 +368,20 @@ class CodeGeneratorTests(unittest.TestCase):
         ext = ".cpp"
         expected_filename = TESTS_DIR / "expected" / f"{case}{ext}"
         if not os.path.exists(expected_filename):
-            raise unittest.SkipTest(f"{expected_filename} not found")
+            raise pytest.skip(f"{expected_filename} not found")
 
         env = os.environ.copy()
         env["CXX"] = "g++-11" if sys.platform == "darwin" else "g++"
-        env["CXXFLAGS"] = "-std=c++14 -Wall -Werror"
+        env["CXXFLAGS"] = "-std=c++17 -Wall -Werror"
 
-        if not spawn.find_executable(env["CXX"]):
-            raise unittest.SkipTest(f"{env['CXX']} not available")
+        if not find_executable(env["CXX"]):
+            raise pytest.skip(f"{env['CXX']} not available")
 
         settings = _get_all_settings(Mock(indent=4), env=env)[lang]
         assert settings.linter[0].startswith("g++")
 
-        if not spawn.find_executable("astyle"):
-            raise unittest.SkipTest("astyle not available")
+        if not find_executable("astyle"):
+            raise pytest.skip("astyle not available")
 
         settings.formatter = ["astyle"]
 
@@ -366,23 +411,12 @@ class CodeGeneratorTests(unittest.TestCase):
             proc = run(linter, env=env)
             assert not proc.returncode
         except FileNotFoundError as e:
-            raise unittest.SkipTest(f"Failed invoking {env['CXX']} or {linter}: {e}")
+            raise pytest.skip(f"Failed invoking {env['CXX']} or {linter}: {e}")
         finally:
             if not KEEP_GENERATED:
                 case_output.unlink(missing_ok=True)
             exe.unlink(missing_ok=True)
 
-    def test_env_clang_format_style(self):
-        lang = "cpp"
-        env = {"CLANG_FORMAT_STYLE": "Google"}
-        settings = _get_all_settings(Mock(indent=4), env=env)[lang]
-        self.assertIn("-style=Google", settings.formatter)
-
-    def test_arg_nim_indent(self):
-        lang = "nim"
-        settings = _get_all_settings(Mock(indent=2))[lang]
-        self.assertIn("--indent:2", settings.formatter)
-
     @staticmethod
     def _rmdir_recursive(path: Path):
         try:
@@ -390,13 +424,13 @@ class CodeGeneratorTests(unittest.TestCase):
                 if child.is_file():
                     child.unlink()
                 else:
-                    CodeGeneratorTests._rmdir_recursive(path)
+                    TestCodeGenerator._rmdir_recursive(path)
             path.rmdir()
         except Exception as e:
             logger.debug(f"{repr(e)}: when removing dir: {path}")
 
-    @foreach(sorted(EXTENSION_TEST_CASES))
-    def test_ext(self, case):
+    @pytest.mark.parametrize("case", EXTENSION_TEST_CASES)
+    def test_rust_ext(self, case):
         lang = "rust"
         env = os.environ.copy()
         if ENV.get(lang):
@@ -423,7 +457,7 @@ class CodeGeneratorTests(unittest.TestCase):
                 generated = actual.read()
                 if os.path.exists(expected_filename) and not self.UPDATE_EXPECTED:
                     with open(expected_filename) as f2:
-                        self.assertEqual(f2.read(), generated)
+                        assert f2.read() == generated
                         print("expected = generated")
 
             if self.UPDATE_EXPECTED or not os.path.exists(expected_filename):
@@ -434,13 +468,18 @@ class CodeGeneratorTests(unittest.TestCase):
             if not self.KEEP_GENERATED:
                 case_output.unlink(missing_ok=True)
 
-    @foreach(sorted(LANGS))
-    @foreach(["test1"])
+    @pytest.mark.parametrize("case, lang", product(["test1"], LANGS))
     def test_directory(self, case, lang):
         env = os.environ.copy()
         if ENV.get(lang):
             env.update(ENV.get(lang))
 
+        settings = _get_all_settings(Mock(indent=4), env=env)[lang]
+        if settings.formatter:
+            if not find_executable(settings.formatter[0]):
+                raise pytest.skip(f"{settings.formatter[0]} not available")
+
+        ext = settings.ext
         TEST_OUTPUT = f"{case}-{lang}-generated"
         EXPECTED_OUTPUT = f"{case}-{lang}-expected"
         case_dirname = ROOT_DIR / Path("tests/dir_cases") / case
@@ -450,30 +489,40 @@ class CodeGeneratorTests(unittest.TestCase):
         args = [
             f"--{lang}=1",
             str(case_dirname),
+            "--project=0",
             "--outdir",
             str(output_dir),
         ]
 
         try:
             rv = main(args=args, env=env)
-            if sys.platform == "win32" and rv != 0:
-                raise unittest.SkipTest("transpiler failed")
             assert rv == 0
-            output = filecmp.dircmp(expected_output_dir, output_dir)
-            if sys.platform == "win32" and output.diff_files != []:
-                raise unittest.SkipTest("directories don't match")
-            assert output.diff_files == []
+
+            filenames = list(expected_output_dir.glob(f"*{ext}"))
+            assert filenames
+            for expected_filename in filenames:
+                output_filename = output_dir / (expected_filename.stem + ext)
+                print(f"Checking {expected_filename} vs {output_filename}")
+                with open(expected_filename) as fh:
+                    expected_contents = fh.read()
+                assert standardise_eol(expected_contents)
+
+                with open(output_filename) as fh:
+                    generated_contents = fh.read()
+                assert standardise_eol(generated_contents)
+
+                assert standardise_eol(generated_contents) == standardise_eol(
+                    expected_contents
+                )
         finally:
             if not self.KEEP_GENERATED:
                 self._rmdir_recursive(output_dir)
 
-
-class TestPaths(unittest.TestCase):
     def test_output_path(self):
         base = Path(".")
-        self.assertEqual(_get_output_path(Path("foo.py"), ".rs", base), Path("foo.rs"))
-        self.assertEqual(
-            _get_output_path(Path("dir/foo.py"), ".rs", base), Path("dir") / "foo.rs"
+        assert _get_output_path(Path("foo.py"), ".rs", base) == Path("foo.rs")
+        assert (
+            _get_output_path(Path("dir/foo.py"), ".rs", base) == Path("dir") / "foo.rs"
         )
 
 
@@ -494,10 +543,10 @@ if __name__ == "__main__":
     )
     args, rest = parser.parse_known_args()
 
-    CodeGeneratorTests.SHOW_ERRORS |= args.show_errors
-    CodeGeneratorTests.KEEP_GENERATED |= args.keep_generated
-    CodeGeneratorTests.UPDATE_EXPECTED |= args.update_expected
-    CodeGeneratorTests.LINT |= args.lint
+    TestCodeGenerator.SHOW_ERRORS |= args.show_errors
+    TestCodeGenerator.KEEP_GENERATED |= args.keep_generated
+    TestCodeGenerator.UPDATE_EXPECTED |= args.update_expected
+    TestCodeGenerator.LINT |= args.lint
 
     rest = [sys.argv[0]] + rest
-    unittest.main(argv=rest)
+    pytest.main(rest)
